@@ -18,11 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: aguix.h,v 1.21 2002/03/18 01:12:41 ralf Exp $ */
+/* $Id: aguix.h,v 1.45 2004/03/10 22:26:15 ralf Exp $ */
 
 #ifndef AGUIX_H
 #define AGUIX_H
 
+#include "aguixdefs.h"
+#include "lowlevelfunc.h"
+#include "message.h"
+#include "util.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,84 +37,72 @@
 #include <X11/cursorfont.h>
 #include <X11/Xatom.h>
 
-#include "aguixdefs.h"
-#include "lowlevelfunc.h"
-#include "message.h"
-#include "util.h"
-
 class AWindow;
 class AGUIXFont;
 
 enum {MES_WAIT,MES_GET};
 enum {NON_GADGET,CLOSE_GADGET,BUTTON_GADGET,SHADOW_GADGET,CHOOSE_GADGET,LISTVIEW,ONELVC,MORELVC,STRING_GADGET};
 
-struct aguix_windowlist {
-  AWindow *window;
-  struct aguix_windowlist *next;
-  struct aguix_windowlist *prev;
-};
-
 class AGUIX {
 public:
   AGUIX();
   ~AGUIX();
-  int initX(int argc,char **argv,char *classname);
+  int initX( int argc, char **argv, const char *classname );
   int checkX();
   void closeX();
-  int getDepth();
-  Colormap getColormap();
-  Display *getDisplay();
-  int getScreen();
+  int getDepth() const;
+  Colormap getColormap() const;
+  Display *getDisplay() const;
+  int getScreen() const;
   void panic(const char *);
   void setFG(int color);
   void setBG(int color);
   void setFG(GC gc,int color);
   void setBG(GC gc,int color);
-  int getCharHeight();
-  int getCharWidth();
+  int getCharHeight() const;
+  int getCharWidth() const;
   void FillRectangle(Drawable buffer,int x,int y,int w,int h);
   void DrawText(Drawable buffer,const char *text,int x,int y);
-  void DrawLine(Drawable buffer,int x1,int y1,int x2,int y2);
+  void DrawLine(Drawable buffer,int px1,int py1,int px2,int py2);
   void DrawPoint(Drawable buffer,int x,int y);
   void FillRectangle(Drawable buffer,GC gc,int x,int y,int w,int h);
   void DrawText(Drawable buffer,AGUIXFont *,const char *text,int x,int y);
-  void DrawLine(Drawable buffer,GC gc,int x1,int y1,int x2,int y2);
+  void DrawLine(Drawable buffer,GC gc,int px1,int py1,int px2,int py2);
   void DrawPoint(Drawable buffer,GC gc,int x,int y);
 
-  void DrawTriangleFilled(Drawable buffer, int x1, int y1, int x2, int y2, int x3, int y3);
-  void DrawTriangleFilled(Drawable buffer, GC tgc, int x1, int y1, int x2, int y2, int x3, int y3);
+  void DrawTriangleFilled(Drawable buffer, int px1, int py1, int px2, int py2, int px3, int py3);
+  void DrawTriangleFilled(Drawable buffer, GC tgc, int px1, int py1, int px2, int py2, int px3, int py3);
 
   void ClearWin(Window win);
   void SetWindowBG(Window win,int color);
-  int getFontBaseline();
+  int getFontBaseline() const;
   int AddColor(int red,int green,int blue);
   void removeLastColor();
   int changeColor(int,int,int,int);
-  int getMaxCols();
-  unsigned long getPixel(int color);
-  char *getClassname();
+  int getMaxCols() const;
+  unsigned long getPixel(int color) const;
+  char *getClassname() const;
   Atom *getCloseAtom();
   void insertWindow(AWindow *win);
   void removeWindow(AWindow *win);
   AGMessage *GetMessage(AWindow *parent);
   AGMessage *WaitMessage(AWindow *parent);
   void ReplyMessage(AGMessage *);
-  Message *wait4mess(int mode,bool);
-  Message *wait4messReact(int mode,bool);
+  Message *wait4mess( int mode );
   void copyArea(Drawable source,Drawable dest,int s_x,int s_y,int width,int height,int d_x,int d_y);
   void Flush();
-  int getargc();
-  char **getargv();
+  int getargc() const;
+  char **getargv() const;
   void putAGMsg(AGMessage *msg);
   AGMessage *getAGMsg();
   void WindowtoFront(Window);
   void WindowtoBack(Window);
-  AGUIXFont *getFont(char*);
-  int setFont(char*);
+  AGUIXFont *getFont( const char* );
+  int setFont( const char* );
   void ExposeHandler(Message*);
   AWindow *findAWindow(Window);
-  char *getNameOfKey(KeySym,unsigned int);
-  Window getGroupWin();
+  char *getNameOfKey(KeySym,unsigned int) const;
+  Window getGroupWin() const;
   int queryPointer(Window win,int *x,int *y);
   int queryRootPointer(int *x,int *y);
   int queryPointer(Window win,int *x,int *y,unsigned int *buttons);
@@ -120,12 +113,12 @@ public:
   int startCut(GUIElement *elem,const char *buffer);
   void cancelCut();
   int startPaste(GUIElement *elem);
-  bool amiOwner();
-  const char *getCutBuffer();
+  bool amiOwner() const;
+  const char *getCutBuffer() const;
   void requestCut(Window win);
   void cancelCutPaste(GUIElement *elem);
-  bool isDoubleClick(struct timeval *t1,struct timeval *t2);
-  bool isDoubleClick( Time t1, Time t2 );
+  bool isDoubleClick(struct timeval *t1,struct timeval *t2) const;
+  bool isDoubleClick( Time t1, Time t2 ) const;
 
   static int scaleElementsW( int wantedWidth,
 			     int borderwidth,
@@ -134,19 +127,60 @@ public:
 			     bool allowShrink,
 			     bool allowStretch,
 			     GUIElement **elem,
+			     int *minWidths,
 			     int nr );
   static int centerElementsY( GUIElement *element,
 			      GUIElement *center2element );
-  int getRootWindowWidth();
-  int getRootWindowHeight();
+  int getRootWindowWidth() const;
+  int getRootWindowHeight() const;
+
+  Pixmap createPixmap( Drawable d, int width, int height );
+  void freePixmap( Pixmap p );
+
+  void DrawDottedLine( Drawable buffer, int px1, int py1, int px2, int py2 );
+  void DrawDottedRectangle( Drawable buffer, int x, int y, int w, int h );
+  void setDottedFG( int color );
+
+  void DrawDashXorLine( Drawable buffer, int px1, int py1, int px2, int py2 );
+  void DrawDashXorRectangle( Drawable buffer, int x, int y, int w, int h );
+  
+  void DrawDashDLine( Drawable buffer, int px1, int py1, int px2, int py2 );
+  void DrawDashDRectangle( Drawable buffer, int x, int y, int w, int h );
+  void setDashDFG( int color );
+  void setDashDBG( int color );
+
+  static bool isModifier( KeySym key );
+  static char *getStringForKeySym( KeySym key );
+  static KeySym getKeySymForString( const char *str1 );
+  void rebuildBackgroundPixmap();
+  void setWindowBackgroundPixmap( Window win );
+  void doXMsgs( AWindow *parent, bool onlyexpose );
+  void doXMsgs( int mode, AWindow *parent, bool onlyexpose );
+  bool noMoreMessages() const;
+#ifdef USE_XIM
+  XIMStyle getXIMStyle() const;
+  XIM getXIM() const;
+  void IMInstCallback( Display *calldsp );
+  void IMDestCallback();
+#endif
+  void setTransientWindow( AWindow *twin = NULL );
+  const AWindow* getTransientWindow() const;
+  int addDefaultColors();
+  KeySym getLastKeyRelease();
+  unsigned int getLastMouseRelease();
+  void msgLock( GUIElement *e );
+  void msgUnlock( const GUIElement *e );
+  void enableTimer();
+  void disableTimer();
+
+  static long timerEvent;
 private:
   Display *dsp;
   int scr;
   Colormap cmap;
-  GC gc;
+  GC gc, dotted_gc, dashxor_gc, dashdouble_gc;
   Atom WM_delete_window;
   char *classname;
-  struct aguix_windowlist *firstwin;
   unsigned long white,black;
   XFontStruct *myfont;
   AGUIXFont *mainfont;
@@ -166,7 +200,7 @@ private:
   void destroyGroupWin();
   
   bool ReactMessage(Message*,AWindow*);
-  void buildAGMessage();
+  void buildAGMessage( Message *agmsg );
   
   bool privatecmap;
   void changeColormap();
@@ -176,6 +210,29 @@ private:
   char *cutbuffer;
 
   unsigned int rootWindowWidth, rootWindowHeight;
+  Pixmap backpm;
+#ifdef USE_XIM
+  XIM inputmethod;
+  XIMStyle im_style;
+
+  int openIM();
+  void closeIM();
+#endif
+  char *keybuf;
+  int keybuf_len;
+
+  AWindow *transientwindow;
+  KeySym lastkeyrelease;
+  unsigned int lastmouserelease;
+
+  GUIElement *msgLockElement;
+  bool timerEnabled;
+#ifndef USE_AGUIXTIMER
+  struct timeval timerStart;
+#endif
+  int msgHandler( int mode, AWindow *parent, bool onlyExpose );
+  enum { TIMER_TICKS = 25 }; //HARDCODED
+  long lastTimerEventNr;
 };
 
 class AGUIXFont
@@ -183,14 +240,14 @@ class AGUIXFont
 public:
   AGUIXFont(AGUIX*);
   ~AGUIXFont();
-  int setFont(char*);
+  int setFont( const char* );
   void removeFont();
-  GC getGC();
-  const char *getName();
-  int getCharHeight();
-  int getCharWidth();
-  int getBaseline();
-  XFontStruct *getFontStruct();
+  GC getGC() const;
+  const char *getName() const;
+  int getCharHeight() const;
+  int getCharWidth() const;
+  int getBaseline() const;
+  XFontStruct *getFontStruct() const;
 private:
   char *name;
   GC gc;
@@ -200,4 +257,3 @@ private:
 };
 
 #endif
-
