@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: button.cc,v 1.13 2002/01/29 00:52:24 ralf Exp $ */
+/* $Id: button.cc,v 1.25 2004/01/22 21:18:05 ralf Exp $ */
 
 #include "button.h"
 #include "awindow.h"
@@ -30,9 +30,6 @@ Button::~Button()
 {
   if(text[0]!=NULL) _freesafe(text[0]);
   if(text[1]!=NULL) _freesafe(text[1]);
-  if(parent!=NULL) {
-    parent->removeSubWin(win);
-  }
 }
 
 Button::Button(AGUIX *taguix):GUIElement(taguix)
@@ -54,6 +51,9 @@ Button::Button(AGUIX *taguix):GUIElement(taguix)
   font=NULL;
   active[0]=true;
   active[1]=true;
+  canHandleFocus = true;
+  setAcceptFocus( true );
+  allowWheel = false;
 }
 
 Button::Button(AGUIX *taguix,int tx,int ty,int width,
@@ -61,7 +61,7 @@ Button::Button(AGUIX *taguix,int tx,int ty,int width,
 {
   this->x=tx;
   this->y=ty;
-  this->w=width;
+  if ( width > 0 )this->w = width;
   this->h=taguix->getCharHeight()+4;
   this->dual=false;
   this->data=tdata;
@@ -79,6 +79,9 @@ Button::Button(AGUIX *taguix,int tx,int ty,int width,
   font=NULL;
   active[0]=true;
   active[1]=true;
+  canHandleFocus = true;
+  setAcceptFocus( true );
+  allowWheel = false;
 }
 
 Button::Button(AGUIX *taguix,int tx,int ty,
@@ -104,6 +107,9 @@ Button::Button(AGUIX *taguix,int tx,int ty,
   font=NULL;
   active[0]=true;
   active[1]=true;
+  canHandleFocus = true;
+  setAcceptFocus( true );
+  allowWheel = false;
 }
 
 Button::Button(AGUIX *taguix,int tx,int ty,int width,int height,
@@ -111,8 +117,8 @@ Button::Button(AGUIX *taguix,int tx,int ty,int width,int height,
 {
   this->x=tx;
   this->y=ty;
-  this->w=width;
-  this->h=height;
+  if ( width > 0 ) this->w = width;
+  if ( height > 0 ) this->h = height;
   this->dual=false;
   this->data=tdata;
   this->state=0;
@@ -129,6 +135,9 @@ Button::Button(AGUIX *taguix,int tx,int ty,int width,int height,
   font=NULL;
   active[0]=true;
   active[1]=true;
+  canHandleFocus = true;
+  setAcceptFocus( true );
+  allowWheel = false;
 }
 
 Button::Button(AGUIX *taguix,int tx,int ty,int width,
@@ -138,7 +147,7 @@ Button::Button(AGUIX *taguix,int tx,int ty,int width,
 {
   this->x=tx;
   this->y=ty;
-  this->w=width;
+  if ( width > 0 ) this->w = width;
   this->h=taguix->getCharHeight()+4;
   this->dual=true;
   this->data=tdata;
@@ -155,9 +164,12 @@ Button::Button(AGUIX *taguix,int tx,int ty,int width,
   font=NULL;
   active[0]=true;
   active[1]=true;
+  canHandleFocus = true;
+  setAcceptFocus( true );
+  allowWheel = false;
 }
 
-const char *Button::getText(int index2)
+const char *Button::getText(int index2) const
 {
   if((index2<0)||(index2>1)) return NULL;
   return dupstring(text[index2]);
@@ -171,7 +183,7 @@ void Button::setText(int index2,const char *new_text)
   redraw();
 }
 
-bool Button::getDualState()
+bool Button::getDualState() const
 {
   return dual;
 }
@@ -191,7 +203,7 @@ void Button::setFG(int index2,int tfg)
   redraw();
 }
 
-int Button::getFG(int index2)
+int Button::getFG(int index2) const
 {
   if((index2<0)||(index2>1)) return 0;
   return fg[index2];
@@ -206,13 +218,13 @@ void Button::setBG(int index2,int tbg)
   redraw();
 }
 
-int Button::getBG(int index2)
+int Button::getBG(int index2) const
 {
   if((index2<0)||(index2>1)) return 0;
   return bg[index2];
 }
 
-int Button::getData()
+int Button::getData() const
 {
   return data;
 }
@@ -227,7 +239,12 @@ void Button::redraw()
   int newbg;
   GC usegc;
   bool showe=false;
-  int x1=0,y1=0,x2=0,y2=0,x3=0,y3=0;
+  int px1 = 0,
+      py1 = 0,
+      px2 = 0,
+      py2 = 0,
+      px3 = 0,
+      py3 = 0;
   int dw,dw2;
   int dy;
   unsigned int maxdisplay;
@@ -239,7 +256,7 @@ void Button::redraw()
 
   /* Clear the window */
 
-  if((dual==false)||(state<2)) {
+  if ( ( dual == false ) || ( state != 2 ) ) {
     newbg=bg[0];
   } else {
     newbg=bg[1];
@@ -251,24 +268,24 @@ void Button::redraw()
 
   /* calculate dimension of the turned-down corner */
 
-  if ( ( dual == true ) && ( showdual == true ) && ( state < 2 ) ) {
+  if ( ( dual == true ) && ( showdual == true ) && ( state != 2 ) ) {
     showe = true;
 
     dw=w/4;
     dw2=(int)((double)(h/3)*1.3);
     if( dw2 < dw ) dw = dw2;
 
-    x1 = w-dw;
-    y1 = 0;
-    x2 = w-dw;
-    y2 = dw-1;
-    x3 = w-1;
-    y3 = dw-1;
+    px1 = w-dw;
+    py1 = 0;
+    px2 = w-dw;
+    py2 = dw-1;
+    px3 = w-1;
+    py3 = dw-1;
   }
 
   /* draw the text */
 
-  if((dual==false)||(state<2)) {
+  if ( ( dual == false ) || ( state != 2 ) ) {
     aguix->setFG(usegc,fg[0]);
   } else {
     aguix->setFG(usegc,fg[1]);
@@ -283,7 +300,7 @@ void Button::redraw()
 
   dy=h/2-ch/2;
   maxdisplay=(w-2)/cw;
-  if((dual==false)||(state<2)) {
+  if ( ( dual == false ) || ( state != 2 ) ) {
     tstr=dupstring(text[0]);
   } else {
     tstr=dupstring(text[1]);
@@ -298,13 +315,19 @@ void Button::redraw()
   }
   _freesafe(tstr);
 
+  // draw the focus
+  if ( ( hasFocus() == true ) && ( getAcceptFocus() == true ) ) {
+    aguix->setDottedFG( 1 );
+    aguix->DrawDottedRectangle( win, 1, 1, w - 2, h - 2 );
+  }
+
   /* now draw the button */
 
   if( showe == true ) {
     aguix->setFG( usegc, 2 );
-    aguix->DrawTriangleFilled( win, usegc, x1, y1, x2, y2, x3, y3 );
+    aguix->DrawTriangleFilled( win, usegc, px1, py1, px2, py2, px3, py3 );
     aguix->setFG( usegc, 0 );
-    aguix->DrawTriangleFilled( win, usegc, x1, y1, w-1, 0, x3, y3 );
+    aguix->DrawTriangleFilled( win, usegc, px1, py1, w-1, 0, px3, py3 );
   }
 
   if( state == 0 )
@@ -315,9 +338,9 @@ void Button::redraw()
   aguix->DrawLine( win, usegc, 0, h-1, 0, 0 );
 
   if( showe == true ) {
-    aguix->DrawLine( win, usegc, 0, 0, x1, y1 );
+    aguix->DrawLine( win, usegc, 0, 0, px1, py1 );
     aguix->setFG( usegc, 1 );
-    aguix->DrawLine( win, usegc, x1, y1, x2, y2 );
+    aguix->DrawLine( win, usegc, px1, py1, px2, py2 );
   } else
     aguix->DrawLine( win, usegc, 0, 0, w-1, 0 );
 
@@ -329,10 +352,10 @@ void Button::redraw()
   aguix->DrawLine( win, usegc, 0, h-1, w-1, h-1 );
 
   if( showe == true ) {
-    aguix->DrawLine( win, usegc, x3, y3, w-1, h-1 );
+    aguix->DrawLine( win, usegc, px3, py3, w-1, h-1 );
     aguix->setFG( usegc, 1 );
-    aguix->DrawLine( win, usegc, x2, y2, x3, y3 );
-    aguix->DrawLine( win, usegc, x1, y1, x3, y3 );
+    aguix->DrawLine( win, usegc, px2, py2, px3, py3 );
+    aguix->DrawLine( win, usegc, px1, py1, px3, py3 );
   } else
     aguix->DrawLine(win,usegc,w-1,h-1,w-1,1);
 
@@ -353,12 +376,12 @@ void Button::setState(int tstate)
   redraw();
 }
 
-int Button::getState()
+int Button::getState() const
 {
   return state;
 }
 
-bool Button::isInside(int px,int py)
+bool Button::isInside(int px,int py) const
 {
   if((px>0)&&(px<=w)) {
     if((py>0)&&(py<=h)) return true;
@@ -369,69 +392,114 @@ bool Button::isInside(int px,int py)
 bool Button::handleMessage(XEvent *E,Message *msg)
 {
   bool returnvalue;
+
+  if ( parent == NULL ) return false;
+
   returnvalue=false;
-  if(msg->window!=win) return false;
   if((msg->type==ButtonPress)||(msg->type==ButtonRelease)) {
-    if((msg->button==Button3)||(msg->button==Button1)) {
-      int mx,my;
-      mx=msg->mousex;
-      my=msg->mousey;
-      if(msg->type==ButtonPress) {
-        if(msg->window==win) {
-	  if( ( ( msg->button == Button1 ) && ( active[0] == true ) ) ||
-	      ( ( msg->button == Button3 ) && ( active[1] == true ) ) ) {
-	    AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-	    agmsg->type=AG_BUTTONPRESSED;
-	    agmsg->button.button=this;
-	    if((msg->button==Button3)&&(dual==true)) {
-	      setState(2);
-	      instate=2;
-	      agmsg->button.state=2;
-	    } else {
-	      setState(1);
-	      instate=1;
-	      agmsg->button.state=1;
+    if ( msg->window == win ) {
+      if ( ( msg->button == Button3 ) ||
+	   ( msg->button == Button1 ) ||
+	   ( ( msg->button == Button4 ) && ( allowWheel == true ) ) ||
+	   ( ( msg->button == Button5 ) && ( allowWheel == true ) ) ) {
+	int mx,my;
+	mx=msg->mousex;
+	my=msg->mousey;
+	if(msg->type==ButtonPress) {
+	  takeFocus();
+	  if(msg->window==win) {
+	    if( ( ( ( msg->button == Button1 ) || ( msg->button == Button4 ) || ( msg->button == Button5 ) ) && ( active[0] == true ) ) ||
+		( ( msg->button == Button3 ) && ( active[1] == true ) ) ) {
+	      AGMessage *agmsg = AGUIX_allocAGMessage();
+	      agmsg->type=AG_BUTTONPRESSED;
+	      agmsg->button.button=this;
+	      if((msg->button==Button3)&&(dual==true)) {
+		setState(2);
+		instate=2;
+		agmsg->button.state=2;
+	      } else {
+		switch ( msg->button ) {
+		case Button4:
+		  instate = 3;
+		  break;
+		case Button5:
+		  instate = 4;
+		  break;
+		default:
+		  instate=1;
+		  break;
+		}
+		setState( instate );
+		agmsg->button.state = instate;
+	      }
+	      aguix->putAGMsg(agmsg);
+	      returnvalue=true;
 	    }
+	  }
+	} else {
+	  if((state!=0)&&(instate!=0)) {
+	    AGMessage *agmsg = AGUIX_allocAGMessage();
+	    agmsg->type=AG_BUTTONRELEASED;
+	    agmsg->button.button=this;
+	    agmsg->button.state=instate;
 	    aguix->putAGMsg(agmsg);
+	    agmsg = AGUIX_allocAGMessage();
+	    agmsg->type=AG_BUTTONCLICKED;
+	    agmsg->button.button=this;
+	    agmsg->button.state=instate;
+	    aguix->putAGMsg(agmsg);
+	  }
+	  if(instate!=0) {
+	    setState(0);
+	    instate=0;
 	    returnvalue=true;
 	  }
-        }
-      } else {
-        if((state!=0)&&(instate!=0)) {
-          AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-          agmsg->type=AG_BUTTONRELEASED;
-          agmsg->button.button=this;
-          agmsg->button.state=instate;
-          aguix->putAGMsg(agmsg);
-          agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-          agmsg->type=AG_BUTTONCLICKED;
-          agmsg->button.button=this;
-          agmsg->button.state=instate;
-          aguix->putAGMsg(agmsg);
-        }
-        if(instate!=0) {
-          setState(0);
-          instate=0;
-          returnvalue=true;
-        }
+	}
       }
     }
   } else if(msg->type==EnterNotify) {
     // alles hier und alles mit instate wird benutzt, damit Button sich anpat, wenn
     // Mauszeiger im Button oder auerhalb des Buttons ist
-    if(instate!=0) {
-      if(state!=instate) {
-        setState(instate);
+    if ( msg->window == win ) {
+      if(instate!=0) {
+	if(state!=instate) {
+	  setState(instate);
+	}
       }
     }
   } else if(msg->type==LeaveNotify) {
     // alles hier und alles mit instate wird benutzt, damit Button sich anpat, wenn
     // Mauszeiger im Button oder auerhalb des Buttons ist
-    if(instate!=0) {
-      setState(0);
+    if ( msg->window == win ) {
+      if(instate!=0) {
+	setState(0);
+      }
     }
   } else if(msg->type==Expose) {
-    redraw();
+    if ( msg->window == win ) {
+      redraw();
+    }
+  } else if ( msg->type == KeyPress ) {
+    if ( ( getAcceptFocus() == true ) && ( hasFocus() == true ) ) {
+      if ( msg->key == XK_space ) {
+	if ( isVisible() == true ) {
+	  if ( parent->isTopParent( msg->window ) == true ) {
+	    AGMessage *agmsg;
+
+	    agmsg = AGUIX_allocAGMessage();
+	    agmsg->type=AG_BUTTONCLICKED;
+	    agmsg->button.button=this;
+	    agmsg->button.state=1;
+	    aguix->putAGMsg(agmsg);
+	    if(instate!=0) {
+	      setState(0);
+	      instate=0;
+	      returnvalue=true;
+	    }
+	  }
+	}
+      }
+    }
   }
   if(returnvalue==true) {
     // jetzt noch die Message mit den Werten fllen
@@ -442,7 +510,7 @@ bool Button::handleMessage(XEvent *E,Message *msg)
   return false; // see cyclebutton.cc
 }
 
-bool Button::getShowDual()
+bool Button::getShowDual() const
 {
   return showdual;
 }
@@ -462,12 +530,12 @@ int Button::setFont(char *fontname)
   return 0;
 }
 
-const char *Button::getType()
+const char *Button::getType() const
 {
   return type;
 }
 
-bool Button::isType(const char *qtype)
+bool Button::isType(const char *qtype) const
 {
   if(strcmp(type,qtype)==0) return true;
   return false;
@@ -489,6 +557,7 @@ void Button::deactivate(int mode)
 {
   if( ( mode == 0 ) || ( mode == 1 ) ) {
     active[mode]=false;
+    if ( mode == 0 ) setAcceptFocus( false );
   }
 }
 
@@ -498,3 +567,13 @@ void Button::activate(int mode)
     active[mode]=true;
   }
 }
+
+bool Button::getAllowWheel() const
+{
+  return allowWheel;
+}
+
+void Button::setAllowWheel( bool nv )
+{
+  allowWheel = nv;
+}
