@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: cyclebutton.h,v 1.4 2001/11/03 17:15:18 ralf Exp $ */
+/* $Id: cyclebutton.h,v 1.5 2004/01/22 21:18:35 ralf Exp $ */
 
 #ifndef CYCLEBUTTON_H
 #define CYCLEBUTTON_H
@@ -40,24 +40,24 @@ public:
   virtual ~CycleButton();
   char *getOption(int);
   int addOption(const char *);
-  int getSelectedOption();
+  int getSelectedOption() const;
   void setFG(int);
-  int getFG();
+  int getFG() const;
   void setBG(int);
-  int getBG();
-  int getData();
+  int getBG() const;
+  int getData() const;
   void setData(int);
   void setState(int);
-  int getState();
+  int getState() const;
   virtual void redraw();
   virtual void flush();
-  bool isInside(int x,int y);
+  bool isInside(int x,int y) const;
   virtual bool handleMessage(XEvent *E,Message *msg);
   int setFont(char*);
-  virtual const char *getType();
-  virtual bool isType(const char *type);
+  virtual const char *getType() const;
+  virtual bool isType(const char *type) const;
   void setOption(int index);
-  int getMaxSize();
+  int getMaxSize() const;
 protected:
   List *options;
   int act_opt;
