@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: dndelement.h,v 1.4 2001/11/03 17:15:20 ralf Exp $ */
+/* $Id: dndelement.h,v 1.6 2003/11/06 21:32:15 ralf Exp $ */
 
 #ifndef DNDELEMENT_H
 #define DNDELEMENT_H
@@ -32,16 +32,18 @@ class GUIElement;
 class DNDElement
 {
 public:
-  DNDElement(AGUIX *,GUIElement *,int,int,void*);
+  DNDElement( AGUIX * );
   virtual ~DNDElement();
-  virtual void handler();
+  virtual void handler( Message *msg );
   virtual void redraw();
+  virtual void create();
 protected:
   virtual void close();
-  virtual void create();
   AGUIX *aguix;
   Window win;
   int delx,dely;
+  int grabbed;
+  bool created;
 };
 
 #endif
