@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: dndtext.h,v 1.4 2001/11/03 17:15:22 ralf Exp $ */
+/* $Id: dndtext.h,v 1.6 2003/11/06 21:32:29 ralf Exp $ */
 
 #ifndef DNDTEXT_H
 #define DNDTEXT_H
@@ -31,12 +31,12 @@ class AGUIXFont;
 class DNDText:public DNDElement
 {
 public:
-  DNDText(AGUIX *,GUIElement *,int,int,void*,char*,int,int,AGUIXFont*);
+  DNDText( AGUIX *, char*, int, int, AGUIXFont* );
   virtual ~DNDText();
   virtual void redraw();
+  virtual void create();
 protected:
   virtual void close();
-  virtual void create();
   AGUIXFont *font;
   char *text;
   int fg,bg;
