@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: guielement.h,v 1.6 2001/12/09 02:08:42 ralf Exp $ */
+/* $Id: guielement.h,v 1.15 2004/02/01 21:10:31 ralf Exp $ */
 
 #ifndef GUIELEMENT_H
 #define GUIELEMENT_H
@@ -34,31 +34,37 @@ public:
   GUIElement() {}
   GUIElement(AGUIX *aguix);
   virtual ~GUIElement();
-  void move(int x,int y);
+  virtual void move(int x,int y);
   virtual void resize(int w,int h);
   virtual void redraw();
   virtual void flush();
-  virtual const char *getType();
-  virtual bool isType(const char *type);
+  virtual const char *getType() const;
+  virtual bool isType(const char *type) const;
   virtual bool handleMessage(XEvent *E,Message *msg);
-  void getPos(int*,int*);
-  void getSize(int*,int*);
-  int getWidth();
-  int getHeight();
-  int getX();
-  int getY();
-  virtual void setParent(AWindow *parent);
-  void toBack();
-  void toFront();
-  bool hasWin(Window win);
-  Window getWindow();
-  virtual bool isParent(Window);
+  virtual void getPos(int*,int*) const;
+  virtual void getSize(int*,int*) const;
+  virtual int getWidth() const;
+  virtual int getHeight() const;
+  virtual int getX() const;
+  virtual int getY() const;
+  void setParent(AWindow *parent);
+  virtual void toBack();
+  virtual void toFront();
+  bool hasWin(Window win) const;
+  Window getWindow() const;
+  virtual bool isParent(Window) const;
   virtual void paste(unsigned char*);
   virtual void cancelpaste();
   virtual void cancelcut();
-  void takeFocus();
-  void leaveFocus();
-  bool hasFocus();
+  virtual void takeFocus();
+  virtual void leaveFocus();
+  bool hasFocus() const;
+  virtual void hide();
+  virtual void show();
+  void setAcceptFocus( bool nv );
+  bool getAcceptFocus() const;
+  bool isVisible() const;
+  virtual int handleMessageLock( XEvent *E, Message *msg );
 protected:
   int x,y;
   int w,h;
@@ -67,6 +73,14 @@ protected:
   Window win;
   static const char *type;
   bool focus;
+
+  virtual void doCreateStuff();
+  virtual void doDestroyStuff();
+  bool canHandleFocus;  // indicates that this element can handle a focus
+  bool acceptFocus;  // this indicates that this element actual want to handle a focus
+  void registerFocusElement();
+  void unregisterFocusElement();
+  bool visible;
 };
 
 #endif
