@@ -18,18 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: message.h,v 1.8 2002/03/09 00:13:13 ralf Exp $ */
+/* $Id: message.h,v 1.15 2003/12/09 22:39:05 ralf Exp $ */
 
 #ifndef MESSAGE_H
 #define MESSAGE_H
 
+#include "aguixdefs.h"
 #include <X11/Xlib.h>
 
 class Button;
 class ChooseButton;
 class StringGadget;
-class ListView;
-class ListViewContent;
 class GUIElement;
 class CycleButton;
 class AWindow;
@@ -37,8 +36,13 @@ class FieldListView;
 
 typedef struct message
 {
-  Window window;
   int type;
+  enum { NONE, TIMEREVENT } specialType;
+  GUIElement *lockElement;
+  bool ack;
+  int loop;
+
+  Window window;
   KeySym key;
   unsigned int keystate;
   unsigned int button;
@@ -47,6 +51,8 @@ typedef struct message
   int gadgettype; // brauchen wir eigentlich nicht mehr, da jedes GUIElement einen Typ hat
   void *gadget;  // kann auch GUIElement *gadget werden
   Time time;
+  char *keybuf;
+  int x, y, width, height;
 } Message;
 
 enum {AG_NONE,AG_KEYPRESSED,AG_KEYRELEASED,AG_MOUSEPRESSED,AG_MOUSERELEASED,AG_MOUSECLICKED,AG_MOUSEMOVE,
@@ -54,18 +60,17 @@ enum {AG_NONE,AG_KEYPRESSED,AG_KEYRELEASED,AG_MOUSEPRESSED,AG_MOUSERELEASED,AG_M
       AG_BUTTONCLICKED,AG_CHOOSEPRESSED,AG_CHOOSERELEASED,AG_CHOOSECLICKED,
       AG_STRINGGADGET_ACTIVATE,AG_STRINGGADGET_DEACTIVATE,AG_STRINGGADGET_OK,
       AG_STRINGGADGET_CANCEL,AG_STRINGGADGET_CONTENTCHANGE,AG_STRINGGADGET_CURSORCHANGE,
-      AG_LISTVIEW_ONESELECT,AG_LISTVIEW_MULTISELECT,AG_LISTVIEW_PRESSED,
       AG_SIZECHANGED,
       AG_DND_START,AG_DND_END,AG_DND_CANCEL,
       AG_CYCLEBUTTONCLICKED,
-      AG_FIELDLV_ONESELECT,AG_FIELDLV_MULTISELECT,AG_FIELDLV_PRESSED};
+      AG_FIELDLV_ONESELECT,AG_FIELDLV_MULTISELECT,AG_FIELDLV_PRESSED,AG_FIELDLV_HEADERCLICKED};
 
 typedef struct agkey {
   int type;
   Window window;
   KeySym key;
   unsigned int keystate;
-  char keyname[10];
+  char *keybuf;
 } AGKey;
 
 typedef AGKey AGKeyPressed;
@@ -139,17 +144,6 @@ typedef AGStringGadget AGStringGadgetCancel;
 typedef AGStringGadget AGStringGadgetContentChange;
 typedef AGStringGadget AGStringGadgetCursorChange;
 
-typedef struct aglistview {
-  int type;
-  ListView *lv;
-  ListViewContent *lvc;
-  Time time;
-} AGListView;
-
-typedef AGListView AGListViewOneSelect;
-typedef AGListView AGListViewMultiSelect;
-typedef AGListView AGListViewPressed;
-
 typedef struct agsizechange {
   int type;
   Window window;
@@ -161,7 +155,11 @@ typedef struct agdnd {
   int relx,rely;
   GUIElement *element;
   AWindow *window;
-  void *specialinfo;
+  struct {
+    void *voidP;
+    int value;
+    class FieldLVRowData *rowDataP;
+  } specialinfo;
 } AGDND;
 
 typedef AGDND AGDNDSTART;
@@ -181,6 +179,8 @@ typedef struct agfieldlv {
   FieldListView *lv;
   int row;
   Time time;
+  bool mouse;
+  unsigned int button;
 } AGFieldLV;
 
 typedef AGFieldLV AGFieldLVOneSelect;
@@ -197,7 +197,6 @@ typedef union agmessage {
   AGButton button;
   AGChoose choose;
   AGStringGadget stringgadget;
-  AGListView listview;
   AGSizeChange size;
   AGDND dnd;
   AGCycleButton cyclebutton;
@@ -205,5 +204,9 @@ typedef union agmessage {
   long direct[20];
 } AGMessage;
 
-#endif
+Message *AGUIX_allocMessage();
+void AGUIX_freeMessage( Message *);
+AGMessage *AGUIX_allocAGMessage();
+void AGUIX_freeAGMessage( AGMessage *);
 
+#endif
