@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: request.h,v 1.9 2002/02/06 20:26:23 ralf Exp $ */
+/* $Id: request.h,v 1.13 2003/06/04 22:45:31 ralf Exp $ */
 
 #ifndef AGUIX_REQUEST_H
 #define AGUIX_REQUEST_H
@@ -29,11 +29,24 @@ class Requester
 {
 public:
   Requester(class AGUIX *parent);
+  Requester( class AGUIX *parent, class AWindow *twin );
   ~Requester();
+
+  typedef enum { REQUEST_NONE = 0,
+		 REQUEST_CURSORLEFT = 1,
+		 REQUEST_CURSORRIGHT = 2,
+		 REQUEST_SELECTALL = 3,
+		 REQUEST_CANCELWITHLEFT = 4 } request_flags_t;
+
   int request(const char *title,const char *text,const char *buttons);
-  int string_request(char *title,char *lines,const char *default_str,char *buttons,char **return_str);
+  int request(const char *title,const char *text,const char *buttons, request_flags_t flags);
+  int request(const char *title,const char *text,const char *buttons, AWindow *twin, request_flags_t flags);
+  int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str);
+  int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str, request_flags_t flags );
+  int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str, AWindow *twin, request_flags_t flags );
 private:
   class AGUIX *aguix;
+  AWindow *transientawin;
 };
 
 #endif
