@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: solidbutton.h,v 1.4 2001/11/03 17:15:55 ralf Exp $ */
+/* $Id: solidbutton.h,v 1.5 2004/01/22 21:19:36 ralf Exp $ */
 
 #ifndef SOLIDBUTTON_H
 #define SOLIDBUTTON_H
@@ -34,20 +34,20 @@ public:
   SolidButton(class AGUIX *aguix,int x,int y,int width,int height,
          const char *text,int fg,int bg,bool state);
   virtual ~SolidButton();
-  const char *getText();
+  const char *getText() const;
   void setText(const char *);
   void setFG(int);
-  int getFG();
+  int getFG() const;
   void setBG(int);
-  int getBG();
-  bool getState();
+  int getBG() const;
+  bool getState() const;
   void setState(bool);
   virtual void redraw();
   virtual void flush();
   virtual bool handleMessage(XEvent *E,Message *msg);
   int setFont(char*);
-  virtual const char *getType();
-  virtual bool isType(const char *type);
+  virtual const char *getType() const;
+  virtual bool isType(const char *type) const;
 private:
   char *text;
   int fg;
