@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: stringgadget.cc,v 1.21 2002/01/30 02:18:47 ralf Exp $ */
+/* $Id: stringgadget.cc,v 1.39 2004/01/22 21:19:41 ralf Exp $ */
 
 #include "stringgadget.h"
 #include "awindow.h"
@@ -28,12 +28,9 @@ const char *StringGadget::type="StringGadget";
 
 StringGadget::~StringGadget()
 {
+  if ( parent != NULL ) parent->remove( this );
   aguix->cancelCutPaste(this);
   _freesafe(text);
-  if(parent!=NULL) {
-    parent->removeSubWin(twin);
-    parent->removeSubWin(win);
-  }
 }
 
 StringGadget::StringGadget(AGUIX *taguix):GUIElement(taguix)
@@ -54,6 +51,9 @@ StringGadget::StringGadget(AGUIX *taguix):GUIElement(taguix)
   strongkeycapture=true;
   wantpaste=false;
   pasterequest=0;
+  twin = 0;
+  canHandleFocus = true;
+  setAcceptFocus( true );
 }
 
 StringGadget::StringGadget(AGUIX *taguix,int tx,int ty,int width,int height,const char *ttext,int tdata):GUIElement(taguix)
@@ -61,8 +61,14 @@ StringGadget::StringGadget(AGUIX *taguix,int tx,int ty,int width,int height,cons
   this->data=tdata;
   this->x=tx;
   this->y=ty;
-  this->w=width;
-  this->h=height;
+  if ( width > 8 )
+    this->w = width;
+  else
+    this->w = 8;
+  if ( height > 8 )
+    this->h = height;
+  else
+    this->h = 8;
   xoffset=0;
   cursorpos=0;
   active=false;
@@ -75,6 +81,9 @@ StringGadget::StringGadget(AGUIX *taguix,int tx,int ty,int width,int height,cons
   strongkeycapture=true;
   wantpaste=false;
   pasterequest=0;
+  twin = 0;
+  canHandleFocus = true;
+  setAcceptFocus( true );
 }
 
 StringGadget::StringGadget(AGUIX *taguix,int tx,int ty,int width,const char *ttext,int tdata):GUIElement(taguix)
@@ -82,7 +91,10 @@ StringGadget::StringGadget(AGUIX *taguix,int tx,int ty,int width,const char *tte
   this->data=tdata;
   this->x=tx;
   this->y=ty;
-  this->w=width;
+  if ( width > 8 )
+    this->w = width;
+  else
+    this->w = 8;
   this->h=taguix->getCharHeight()+2+6;
   xoffset=0;
   cursorpos=0;
@@ -97,6 +109,9 @@ StringGadget::StringGadget(AGUIX *taguix,int tx,int ty,int width,const char *tte
   strongkeycapture=true;
   wantpaste=false;
   pasterequest=0;
+  twin = 0;
+  canHandleFocus = true;
+  setAcceptFocus( true );
 }
 
 void StringGadget::resize(int tw,int th)
@@ -112,7 +127,7 @@ void StringGadget::resize(int tw,int th)
   redraw();
 }
 
-int StringGadget::getData()
+int StringGadget::getData() const
 {
   return data;
 }
@@ -244,7 +259,7 @@ void StringGadget::flush()
 {
 }
 
-bool StringGadget::isInside(int px,int py)
+bool StringGadget::isInside(int px,int py) const
 {
 /*  if((px>x)&&(px<=(x+width))) {
     if((py>y)&&(py<=(y+height))) return true;
@@ -260,6 +275,8 @@ bool StringGadget::handleMessage(XEvent *E,Message *msg)
   int tx,ty;
   AGMessage *agmsg;
   
+  if ( parent == NULL ) return false;
+
   if(font==NULL) {
     cw=aguix->getCharWidth();
   } else {
@@ -271,7 +288,7 @@ bool StringGadget::handleMessage(XEvent *E,Message *msg)
         active=false;
         tredraw();
 //        returnvalue=true;
-        agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
+        agmsg = AGUIX_allocAGMessage();
         agmsg->type=AG_STRINGGADGET_DEACTIVATE;
         agmsg->stringgadget.sg=this;
 	agmsg->stringgadget.ok = false;
@@ -280,256 +297,236 @@ bool StringGadget::handleMessage(XEvent *E,Message *msg)
         if(forbidPosChange==false) {
 	  aguix->queryPointer(msg->window,&tx,&ty);
           int nc=tx/cw;
-          cursorpos=nc+xoffset;
-          if(cursorpos>=(int)strlen(text)) cursorpos=(int)strlen(text);
-          if(cursorpos<0) cursorpos=0;
-          selstart=selend=cursorpos;
-          if(msg->button==Button2) insertSelection();
-          redraw();
+	  if ( nc < maxdisplay ) {
+	    cursorpos=nc+xoffset;
+	    if(cursorpos>=(int)strlen(text)) cursorpos=(int)strlen(text);
+	    if(cursorpos<0) cursorpos=0;
+	    selstart=selend=cursorpos;
+	    if(msg->button==Button2) insertSelection();
+	    redraw();
+	  }
 	}
       }
     } else {
       if(msg->type==KeyPress) {
-        ignoreRelease=false;
-        int keystate=KEYSTATEMASK(msg->keystate);
-        if((msg->key==XK_Right)||
-	   (msg->key==XK_End)||
-	   ((msg->key==XK_e)&&(keystate==ControlMask))||
-	   ((msg->key==XK_f)&&(keystate==ControlMask))) {
-	  if(strongkeycapture==true) returnvalue=true;
-	  if(forbidPosChange==false) {
-            if((((keystate==0)||(keystate==ShiftMask))&&
-	       (msg->key==XK_Right))||
+	if ( isVisible() == true ) {
+	  if ( parent->isTopParent( msg->window ) == true ) {
+	    ignoreRelease=false;
+	    int keystate=KEYSTATEMASK(msg->keystate);
+	    if((msg->key==XK_Right)||
+	       (msg->key==XK_End)||
+	       ((msg->key==XK_e)&&(keystate==ControlMask))||
 	       ((msg->key==XK_f)&&(keystate==ControlMask))) {
-              if(cursorpos<(int)strlen(text)) {
-                cursorpos++;
-                if((cursorpos-xoffset)>=maxdisplay) xoffset++;
-	        if(keystate!=ShiftMask) selstart=selend=cursorpos;
-  	        else {
- 	          selend=cursorpos;
-  	          applySelection();
-                }
-                tredraw();
-                returnvalue=true;
-              }
-            } else if(((keystate&Mod1Mask)==Mod1Mask)||
-	              (msg->key==XK_End)||
-		      ((msg->key==XK_e)&&(keystate==ControlMask))) {
-              cursorpos=(int)strlen(text);
-              while((cursorpos-xoffset)>=maxdisplay) xoffset++;
-  	      if((keystate&ShiftMask)==0) selstart=selend=cursorpos;
-  	      else selend=cursorpos;
-              tredraw();
-              returnvalue=true;
-            }
-            agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-            agmsg->type=AG_STRINGGADGET_CURSORCHANGE;
-            agmsg->stringgadget.sg=this;
-            aguix->putAGMsg(agmsg);
+	      if(strongkeycapture==true) returnvalue=true;
+	      if(forbidPosChange==false) {
+		if((((keystate==0)||(keystate==ShiftMask))&&
+		    (msg->key==XK_Right))||
+		   ((msg->key==XK_f)&&(keystate==ControlMask))) {
+		  if(cursorpos<(int)strlen(text)) {
+		    cursorpos++;
+		    if((cursorpos-xoffset)>=maxdisplay) xoffset++;
+		    if(keystate!=ShiftMask) selstart=selend=cursorpos;
+		    else {
+		      selend=cursorpos;
+		      applySelection();
+		    }
+		    tredraw();
+		    returnvalue=true;
+		  } else if ( ( selstart != selend ) && ( keystate != ShiftMask ) ) {
+		    // selection active and normal cursor right
+		    // => deactivate selection
+		    selstart = selend = cursorpos;
+		    tredraw();
+		    returnvalue = true;
+		  }
+		} else if(((keystate&Mod1Mask)==Mod1Mask)||
+			  (msg->key==XK_End)||
+			  ((msg->key==XK_e)&&(keystate==ControlMask))) {
+		  cursorpos=(int)strlen(text);
+		  while((cursorpos-xoffset)>=maxdisplay) xoffset++;
+		  if((keystate&ShiftMask)==0) selstart=selend=cursorpos;
+		  else selend=cursorpos;
+		  tredraw();
+		  returnvalue=true;
+		}
+		agmsg = AGUIX_allocAGMessage();
+		agmsg->type=AG_STRINGGADGET_CURSORCHANGE;
+		agmsg->stringgadget.sg=this;
+		aguix->putAGMsg(agmsg);
+	      }
+	    } else if((msg->key==XK_Left)||
+		      (msg->key==XK_Home)||
+		      ((msg->key==XK_a)&&(keystate==ControlMask))||
+		      ((msg->key==XK_b)&&(keystate==ControlMask))) {
+	      if(strongkeycapture==true) returnvalue=true;
+	      if(forbidPosChange==false) {
+		if((((keystate==0)||(keystate==ShiftMask))&&
+		    (msg->key==XK_Left))||
+		   ((msg->key==XK_b)&&(keystate==ControlMask))) {
+		  if(cursorpos>0) {
+		    cursorpos--;
+		    if((cursorpos-xoffset)<0) xoffset--;
+		    if(keystate!=ShiftMask) selstart=selend=cursorpos;
+		    else {
+		      selend=cursorpos;
+		      applySelection();
+		    }
+		    tredraw();
+		    returnvalue=true;
+		  } else if ( ( selstart != selend ) && ( keystate != ShiftMask ) ) {
+		    // selection active and normal cursor left
+		    // => deactivate selection
+		    selstart = selend = cursorpos;
+		    tredraw();
+		    returnvalue = true;
+		  }
+		} else if(((keystate&Mod1Mask)==Mod1Mask)||
+			  (msg->key==XK_Home)||
+			  ((msg->key==XK_a)&&(keystate==ControlMask))) {
+		  xoffset=0;
+		  cursorpos=0;
+		  if((keystate&ShiftMask)==0) selstart=selend=cursorpos;
+		  else selend=cursorpos;
+		  tredraw();
+		  returnvalue=true;
+		}
+		agmsg = AGUIX_allocAGMessage();
+		agmsg->type=AG_STRINGGADGET_CURSORCHANGE;
+		agmsg->stringgadget.sg=this;
+		aguix->putAGMsg(agmsg);
+	      }
+	    } else if((msg->key==XK_Delete)||((msg->key==XK_d)&&(keystate==ControlMask))) {
+	      int len=strlen(text);
+	      if(strongkeycapture==true) returnvalue=true;
+	      if(selstart!=selend) {
+		removeSelection();
+		tredraw();
+	      } else if((cursorpos<len)&&(len>0)) {
+		char *tstr=(char*)_allocsafe(len);
+		strncpy(tstr,text,cursorpos);
+		tstr[cursorpos]=0;
+		if(cursorpos<len) strcat(tstr,text+cursorpos+1);
+		_freesafe(text);
+		text=tstr;
+		tredraw();
+		returnvalue=true;
+		agmsg = AGUIX_allocAGMessage();
+		agmsg->type=AG_STRINGGADGET_CONTENTCHANGE;
+		agmsg->stringgadget.sg=this;
+		aguix->putAGMsg(agmsg);
+	      }
+	    } else if((msg->key==XK_BackSpace)||((msg->key==XK_h)&&(keystate==ControlMask))) {
+	      int len=strlen(text);
+	      if(strongkeycapture==true) returnvalue=true;
+	      if(selend!=selstart) {
+		removeSelection();
+		tredraw();
+	      } else if(cursorpos>0) {
+		char *tstr=(char*)_allocsafe(len);
+		tstr[0]=0;
+		if(cursorpos>1) {
+		  strncpy(tstr,text,cursorpos-1);
+		  tstr[cursorpos-1]=0;
+		}
+		strcat(tstr,text+cursorpos);
+		_freesafe(text);
+		text=tstr;
+		cursorpos--;
+		if((cursorpos-xoffset)<0) xoffset--;
+		selstart=selend=cursorpos;
+		tredraw();
+		returnvalue=true;
+		agmsg = AGUIX_allocAGMessage();
+		agmsg->type=AG_STRINGGADGET_CONTENTCHANGE;
+		agmsg->stringgadget.sg=this;
+		aguix->putAGMsg(agmsg);
+	      }
+	    } else if ( msg->key == XK_Return ) {
+	      if ( strongkeycapture == true ) returnvalue = true;
+	    } else if ( IsModifierKey( msg->key ) ||
+			IsCursorKey( msg->key ) ||
+			IsPFKey( msg->key ) ||
+			IsFunctionKey( msg->key ) ||
+			IsMiscFunctionKey( msg->key ) ||
+			( ( msg->key >= XK_BackSpace ) && ( msg->key <= XK_Escape ) ) ) {
+	      // catch this special keys even they are not handled
+	      // so they will not be added to the text
+	    } else if ( ( strlen( msg->keybuf ) > 0 ) && ( keystate != ControlMask ) ) {
+	      int len = strlen( text );
+	      if ( selstart != selend ) {
+		removeSelection();
+		tredraw();
+	      }
+	      char *tstr = (char*)_allocsafe(len + strlen( msg->keybuf ) + 1 );
+	      strncpy( tstr, text, cursorpos );
+	      strcpy( tstr + cursorpos, msg->keybuf );
+	      strcat( tstr, text + cursorpos);
+	      _freesafe( text );
+	      text = tstr;
+	      cursorpos += strlen( msg->keybuf );
+	      while ( ( cursorpos - xoffset ) >= maxdisplay ) xoffset++;
+	      selstart = selend = cursorpos;
+	      tredraw();
+	      returnvalue = true;
+	      agmsg = AGUIX_allocAGMessage();
+	      agmsg->type = AG_STRINGGADGET_CONTENTCHANGE;
+	      agmsg->stringgadget.sg = this;
+	      aguix->putAGMsg( agmsg );
+	    }
 	  }
-        } else if((msg->key==XK_Left)||
-	          (msg->key==XK_Home)||
-		  ((msg->key==XK_a)&&(keystate==ControlMask))||
-		  ((msg->key==XK_b)&&(keystate==ControlMask))) {
-	  if(strongkeycapture==true) returnvalue=true;
-          if(forbidPosChange==false) {
-            if((((keystate==0)||(keystate==ShiftMask))&&
-	       (msg->key==XK_Left))||
-	       ((msg->key==XK_b)&&(keystate==ControlMask))) {
-              if(cursorpos>0) {
-                cursorpos--;
-                if((cursorpos-xoffset)<0) xoffset--;
-	        if(keystate!=ShiftMask) selstart=selend=cursorpos;
-	        else {
-	          selend=cursorpos;
-	  	  applySelection();
-  	        }
-                tredraw();
-                returnvalue=true;
-              }
-            } else if(((keystate&Mod1Mask)==Mod1Mask)||
-	              (msg->key==XK_Home)||
-		      ((msg->key==XK_a)&&(keystate==ControlMask))) {
-              xoffset=0;
-              cursorpos=0;
-	      if((keystate&ShiftMask)==0) selstart=selend=cursorpos;
-  	      else selend=cursorpos;
-              tredraw();
-              returnvalue=true;
-            }
-            agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-            agmsg->type=AG_STRINGGADGET_CURSORCHANGE;
-            agmsg->stringgadget.sg=this;
-            aguix->putAGMsg(agmsg);
-	  }
-        } else if((msg->key==XK_Delete)||((msg->key==XK_d)&&(keystate==ControlMask))) {
-          int len=strlen(text);
-	  if(strongkeycapture==true) returnvalue=true;
-	  if(selstart!=selend) {
-	    removeSelection();
-	    tredraw();
-	  } else if((cursorpos<len)&&(len>0)) {
-            char *tstr=(char*)_allocsafe(len);
-            strncpy(tstr,text,cursorpos);
-            tstr[cursorpos]=0;
-            if(cursorpos<len) strcat(tstr,text+cursorpos+1);
-            _freesafe(text);
-            text=tstr;
-            tredraw();
-            returnvalue=true;
-            agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-            agmsg->type=AG_STRINGGADGET_CONTENTCHANGE;
-            agmsg->stringgadget.sg=this;
-            aguix->putAGMsg(agmsg);
-          }
-        } else if((msg->key==XK_BackSpace)||((msg->key==XK_h)&&(keystate==ControlMask))) {
-          int len=strlen(text);
-	  if(strongkeycapture==true) returnvalue=true;
-	  if(selend!=selstart) {
-	    removeSelection();
-	    tredraw();
-	  } else if(cursorpos>0) {
-            char *tstr=(char*)_allocsafe(len);
-            tstr[0]=0;
-            if(cursorpos>1) {
-              strncpy(tstr,text,cursorpos-1);
-              tstr[cursorpos-1]=0;
-            }
-            strcat(tstr,text+cursorpos);
-            _freesafe(text);
-            text=tstr;
-            cursorpos--;
-            if((cursorpos-xoffset)<0) xoffset--;
-	    selstart=selend=cursorpos;
-            tredraw();
-            returnvalue=true;
-            agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-            agmsg->type=AG_STRINGGADGET_CONTENTCHANGE;
-            agmsg->stringgadget.sg=this;
-            aguix->putAGMsg(agmsg);
-          }
-        } else if((((msg->key>=XK_space)&&(msg->key<=0xff))||
-	           (msg->key==XK_KP_0)||
-	           (msg->key==XK_KP_1)||
-	           (msg->key==XK_KP_2)||
-	           (msg->key==XK_KP_3)||
-	           (msg->key==XK_KP_4)||
-	           (msg->key==XK_KP_5)||
-	           (msg->key==XK_KP_6)||
-	           (msg->key==XK_KP_7)||
-	           (msg->key==XK_KP_8)||
-	           (msg->key==XK_KP_9))&&(keystate!=ControlMask)) {
-          int len=strlen(text);
-	  if(selstart!=selend) {
-	    removeSelection();
-	    tredraw();
-	  }
-          char *tstr=(char*)_allocsafe(len+2);
-          tstr[0]=0;
-          strncpy(tstr,text,cursorpos);
-          tstr[cursorpos]=0;
-          len=strlen(tstr);
-	  switch(msg->key) {
-	    case XK_KP_0:
-	      tstr[len]='0';
-	      break;
-	    case XK_KP_1:
-	      tstr[len]='1';
-	      break;
-	    case XK_KP_2:
-	      tstr[len]='2';
-	      break;
-	    case XK_KP_3:
-	      tstr[len]='3';
-	      break;
-	    case XK_KP_4:
-	      tstr[len]='4';
-	      break;
-	    case XK_KP_5:
-	      tstr[len]='5';
-	      break;
-	    case XK_KP_6:
-	      tstr[len]='6';
-	      break;
-	    case XK_KP_7:
-	      tstr[len]='7';
-	      break;
-	    case XK_KP_8:
-	      tstr[len]='8';
-	      break;
-	    case XK_KP_9:
-	      tstr[len]='9';
-	      break;
-	    default:
-	      tstr[len]=(char)msg->key;
-	      break;
-	  }
-          tstr[len+1]=0;
-          strcat(tstr,text+cursorpos);
-          _freesafe(text);
-          text=tstr;
-          cursorpos++;
-          if((cursorpos-xoffset)>=maxdisplay) xoffset++;
-	  selstart=selend=cursorpos;
-          tredraw();
-          returnvalue=true;
-          agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-          agmsg->type=AG_STRINGGADGET_CONTENTCHANGE;
-          agmsg->stringgadget.sg=this;
-          aguix->putAGMsg(agmsg);
-        } else if(msg->key==XK_Return) {
-	  if(strongkeycapture==true) returnvalue=true;
 	}
       } else if((msg->type==KeyRelease)&&(ignoreRelease==false)) {
-        int keystate=KEYSTATEMASK(msg->keystate);
-        if((msg->key==XK_Return)||(msg->key==XK_Escape)) {
-          active=false;
-	  selstart=selend=cursorpos;
-          tredraw();
-          returnvalue=true;
-          agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-          agmsg->type=AG_STRINGGADGET_DEACTIVATE;
-          agmsg->stringgadget.sg=this;
-	  agmsg->stringgadget.ok = ( msg->key == XK_Return ) ? true : false;
-          aguix->putAGMsg(agmsg);
-          if(msg->key==XK_Return) {
-            agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-            agmsg->type=AG_STRINGGADGET_OK;
-            agmsg->stringgadget.sg=this;
-            aguix->putAGMsg(agmsg);
-          } else {
-            agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
-            agmsg->type=AG_STRINGGADGET_CANCEL;
-            agmsg->stringgadget.sg=this;
-            aguix->putAGMsg(agmsg);
-          }
-        } else if((msg->key==XK_Right)||
-		  (msg->key==XK_End)||
-		  ((msg->key==XK_e)&&(keystate==ControlMask))||
-		  ((msg->key==XK_f)&&(keystate==ControlMask))) {
-	  if(strongkeycapture==true) returnvalue=true;
-        } else if((msg->key==XK_Left)||
-	          (msg->key==XK_Home)||
-		  ((msg->key==XK_a)&&(keystate==ControlMask))||
-		  ((msg->key==XK_b)&&(keystate==ControlMask))) {
-	  if(strongkeycapture==true) returnvalue=true;
-        } else if((msg->key==XK_Delete)||((msg->key==XK_d)&&(keystate==ControlMask))) {
-	  if(strongkeycapture==true) returnvalue=true;
-        } else if((msg->key==XK_BackSpace)||((msg->key==XK_h)&&(keystate==ControlMask))) {
-	  if(strongkeycapture==true) returnvalue=true;
-        } else if((((msg->key>=XK_space)&&(msg->key<=0xff))||
-	           (msg->key==XK_KP_0)||
-	           (msg->key==XK_KP_1)||
-	           (msg->key==XK_KP_2)||
-	           (msg->key==XK_KP_3)||
-	           (msg->key==XK_KP_4)||
-	           (msg->key==XK_KP_5)||
-	           (msg->key==XK_KP_6)||
-	           (msg->key==XK_KP_7)||
-	           (msg->key==XK_KP_8)||
-	           (msg->key==XK_KP_9))&&(keystate!=ControlMask)) {
-	  if(strongkeycapture==true) returnvalue=true;
-        }
+	if ( isVisible() == true ) {
+	  if ( parent->isTopParent( msg->window ) == true ) {
+	    int keystate=KEYSTATEMASK(msg->keystate);
+	    if((msg->key==XK_Return)||(msg->key==XK_Escape)) {
+	      active=false;
+	      selstart=selend=cursorpos;
+	      tredraw();
+	      returnvalue=true;
+	      agmsg = AGUIX_allocAGMessage();
+	      agmsg->type=AG_STRINGGADGET_DEACTIVATE;
+	      agmsg->stringgadget.sg=this;
+	      agmsg->stringgadget.ok = ( msg->key == XK_Return ) ? true : false;
+	      aguix->putAGMsg(agmsg);
+	      if(msg->key==XK_Return) {
+		agmsg = AGUIX_allocAGMessage();
+		agmsg->type=AG_STRINGGADGET_OK;
+		agmsg->stringgadget.sg=this;
+		aguix->putAGMsg(agmsg);
+	      } else {
+		agmsg = AGUIX_allocAGMessage();
+		agmsg->type=AG_STRINGGADGET_CANCEL;
+		agmsg->stringgadget.sg=this;
+		aguix->putAGMsg(agmsg);
+	      }
+	    } else if((msg->key==XK_Right)||
+		      (msg->key==XK_End)||
+		      ((msg->key==XK_e)&&(keystate==ControlMask))||
+		      ((msg->key==XK_f)&&(keystate==ControlMask))) {
+	      if(strongkeycapture==true) returnvalue=true;
+	    } else if((msg->key==XK_Left)||
+		      (msg->key==XK_Home)||
+		      ((msg->key==XK_a)&&(keystate==ControlMask))||
+		      ((msg->key==XK_b)&&(keystate==ControlMask))) {
+	      if(strongkeycapture==true) returnvalue=true;
+	    } else if((msg->key==XK_Delete)||((msg->key==XK_d)&&(keystate==ControlMask))) {
+	      if(strongkeycapture==true) returnvalue=true;
+	    } else if((msg->key==XK_BackSpace)||((msg->key==XK_h)&&(keystate==ControlMask))) {
+	      if(strongkeycapture==true) returnvalue=true;
+	    } else if ( IsModifierKey( msg->key ) ||
+			IsCursorKey( msg->key ) ||
+			IsPFKey( msg->key ) ||
+			IsFunctionKey( msg->key ) ||
+			IsMiscFunctionKey( msg->key ) ||
+			( ( msg->key >= XK_BackSpace ) && ( msg->key <= XK_Escape ) ) ) {
+	      // Some of these keys could create a char in keybuf but because we don't
+	      // handle them here do nothing
+	    } else if ( ( strlen( msg->keybuf ) > 0 ) && ( keystate != ControlMask ) ) {
+	      if(strongkeycapture==true) returnvalue=true;
+	    }
+	  }
+	}
       } else if(msg->type==ButtonRelease) {
         applySelection();
       } else if(E->type==MotionNotify) {
@@ -552,20 +549,23 @@ bool StringGadget::handleMessage(XEvent *E,Message *msg)
     }
   } else if(msg->type==ButtonPress) {
     if(msg->window==twin) {
+      takeFocus();
       active=true;
 //      returnvalue=true;
-      agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
+      agmsg = AGUIX_allocAGMessage();
       agmsg->type=AG_STRINGGADGET_ACTIVATE;
       agmsg->stringgadget.sg=this;
       aguix->putAGMsg(agmsg);
       if(forbidPosChange==false) {
         aguix->queryPointer(msg->window,&tx,&ty);
         int nc=tx/cw;
-        cursorpos=nc+xoffset;
-        if(cursorpos>=(int)strlen(text)) cursorpos=(int)strlen(text);
-        if(cursorpos<0) cursorpos=0;
-        selstart=selend=cursorpos;
-        if(msg->button==Button2) insertSelection();
+	if ( nc < maxdisplay ) {
+	  cursorpos=nc+xoffset;
+	  if(cursorpos>=(int)strlen(text)) cursorpos=(int)strlen(text);
+	  if(cursorpos<0) cursorpos=0;
+	  selstart=selend=cursorpos;
+	  if(msg->button==Button2) insertSelection();
+	}
       }
       redraw();
     }
@@ -592,12 +592,12 @@ void StringGadget::setText(const char *new_text)
   tredraw();
 }
 
-const char *StringGadget::getText()
+const char *StringGadget::getText() const
 {
   return text;
 }
 
-bool StringGadget::isActive()
+bool StringGadget::isActive() const
 {
   return active;
 }
@@ -615,22 +615,14 @@ void StringGadget::updateWin()
     if ( maxdisplay > 0 ) {
       // makes no sense if there is no char visible
       i1 = a_min( selstart, selend );
-      if ( ( i1 - xoffset + 1) > maxdisplay )
-	xoffset = i1 - maxdisplay + 1;
+      // try to move xoffset as most left as possible while cursor remains visible
+      xoffset = i1 - maxdisplay + 1;
+      if ( xoffset < 0 ) xoffset = 0;
     }
     tredraw();
   }
 }
 
-void StringGadget::setParent(AWindow *tparent)
-{
-  this->parent=tparent;
-  win=tparent->getSubWindow(0,x,y,w,h);
-  twin=tparent->getSubWindow(win,3,3,w-6,h-6);
-  if(font==NULL) maxdisplay=(w-6-2)/aguix->getCharWidth();
-  else maxdisplay=(w-6-2)/font->getCharWidth();
-}
-
 void StringGadget::activate()
 {
   active=true;
@@ -667,7 +659,7 @@ void StringGadget::removeSelection()
   x1=(selstart<=selend?selstart:selend);
   x2=(selstart>selend?selstart:selend);
   if(x2==(int)strlen(text)) x2--;
-  if(x1<x2) {
+  if ( x1 <= x2 ) {
     char *tstr;
     tstr=(char*)_allocsafe(strlen(text)-x2+x1+1);
     strncpy(tstr,text,x1);
@@ -682,6 +674,7 @@ void StringGadget::removeSelection()
 
 void StringGadget::insertSelection()
 {
+  if ( parent == NULL ) return;
 #if 0
   Display *display=aguix->getDisplay();
 #endif
@@ -734,35 +727,64 @@ int StringGadget::setFont(char *fontname)
   return 0;
 }
 
-const char *StringGadget::getType()
+const char *StringGadget::getType() const
 {
   return type;
 }
 
-bool StringGadget::isType(const char *qtype)
+bool StringGadget::isType(const char *qtype) const
 {
   if(strcmp(type,qtype)==0) return true;
   return false;
 }
 
-bool StringGadget::isParent(Window child)
+bool StringGadget::isParent(Window child) const
 {
+  if ( parent == NULL ) return false;
   if(child==win) return true;
   if(child==twin) return true;
   return false;
 }
 
-int StringGadget::getXOffset()
+int StringGadget::getXOffset() const
 {
   return xoffset;
 }
 
-int StringGadget::getCursor()
+void StringGadget::setXOffset( int new_pos )
+{
+  xoffset = new_pos;
+
+  if ( ( xoffset + maxdisplay ) >= (int)strlen( text ) ) {
+    xoffset = (int)strlen( text ) - maxdisplay;
+  }
+  if ( xoffset < 0 ) xoffset = 0;
+  tredraw();
+}
+
+int StringGadget::getCursor() const
 {
   return cursorpos;
 }
 
-bool StringGadget::isPosChangeForbidden()
+void StringGadget::setCursor( int new_pos )
+{
+  cursorpos = new_pos;
+
+  if ( cursorpos > (int)strlen( text ) )
+    cursorpos = (int)strlen( text );
+  if ( cursorpos < 0 )
+    cursorpos = 0;
+  selstart = selend = cursorpos;
+
+  if ( ( cursorpos - xoffset ) < 0 )
+    xoffset = cursorpos;
+  if ( ( cursorpos - xoffset ) >= maxdisplay )
+    xoffset = cursorpos - maxdisplay + 1;
+  setXOffset( xoffset );
+}
+
+bool StringGadget::isPosChangeForbidden() const
 {
   return forbidPosChange;
 }
@@ -811,3 +833,79 @@ void StringGadget::cancelcut()
   selstart=selend=cursorpos;
   redraw();
 }
+
+void StringGadget::doCreateStuff()
+{
+  GUIElement::doCreateStuff();
+  if ( twin == 0 ) {
+    twin=parent->getSubWindow(win,3,3,w-6,h-6);
+  }
+  if(font==NULL) maxdisplay=(w-6-2)/aguix->getCharWidth();
+  else maxdisplay=(w-6-2)/font->getCharWidth();
+}
+
+void StringGadget::doDestroyStuff()
+{
+  if ( twin != 0 ) {
+    parent->removeSubWin(twin);
+    twin = 0;
+  }
+  GUIElement::doDestroyStuff();
+}
+
+void StringGadget::takeFocus()
+{
+  GUIElement::takeFocus();
+  if ( active == false ) {
+    //    active = true;
+    //    redraw();
+    activate();
+    AGMessage *agmsg = AGUIX_allocAGMessage();
+    agmsg->type = AG_STRINGGADGET_ACTIVATE;
+    agmsg->stringgadget.sg = this;
+    aguix->putAGMsg( agmsg );
+  }
+}
+
+void StringGadget::leaveFocus()
+{
+  GUIElement::leaveFocus();
+  if ( active == true ) {
+    active = false;
+    tredraw();
+    AGMessage *agmsg = AGUIX_allocAGMessage();
+    agmsg->type = AG_STRINGGADGET_DEACTIVATE;
+    agmsg->stringgadget.sg = this;
+    agmsg->stringgadget.ok = false;
+    aguix->putAGMsg( agmsg );
+  }
+}
+
+void StringGadget::insertAtCursor( const char *str )
+{
+  if ( str == NULL ) return;
+  int tl, nl;
+  char *tstr;
+
+  nl = strlen( str );
+  tl = strlen( text );
+  tstr = (char*)_allocsafe( tl + nl + 1);
+  strncpy( tstr, text, cursorpos );
+  tstr[ cursorpos ] = 0;
+  strncat( tstr, str, nl );
+  tstr[ cursorpos + nl ] = 0;
+  strcat( tstr, text + cursorpos );
+  _freesafe( text );
+  text = tstr;
+  redraw();
+}
+
+void StringGadget::selectAll()
+{
+  cursorpos=(int)strlen(text);
+  while ( ( cursorpos - xoffset ) >= maxdisplay ) xoffset++;
+  selstart = 0;
+  selend = cursorpos;
+  applySelection();
+  tredraw();
+}
