@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: util.cc,v 1.12 2002/02/06 20:27:03 ralf Exp $ */
+/* $Id: util.cc,v 1.14 2004/01/22 21:19:59 ralf Exp $ */
 
 #include "util.h"
 #include "lowlevelfunc.h"
@@ -180,7 +180,7 @@ int List::removeFirstElement()
   return 0;
 }
 
-int List::size()
+int List::size() const
 {
   return elements;
 }
@@ -276,7 +276,7 @@ ListElement::ListElement()
   prev=NULL;
 }
 
-int List::getIndex(void *obj)
+int List::getIndex(void *obj) const
 {
   if(obj==NULL) return -1;
   int x;
@@ -657,7 +657,7 @@ int ArrayList::removeFirstElement()
   return 0;
 }
 
-int ArrayList::size()
+int ArrayList::size() const
 {
   return elements;
 }
@@ -734,7 +734,7 @@ void *ArrayList::getNextElement(int id)
   return array[acts[id]];
 }
 
-int ArrayList::getIndex(void *obj)
+int ArrayList::getIndex(void *obj) const
 {
   if(obj==NULL) return -1;
   if(elements<1) return -1;
@@ -880,13 +880,20 @@ int ArrayList::qs_partition(int p,int r,int (*relfunc)(void*,void*,int),int mode
 int createLines(const char *text,char ***ret_lines)
 {
   int lines;
-  int i;
+  int i, p;
+  short ignorenext;
+
   lines=1;
+  ignorenext = 0;
   for(i=0;i<(int)strlen(text);i++) {
-    if(text[i]=='|') lines++;
+    if ( ignorenext == 0 ) {
+      if(text[i]=='|') lines++;
+      else if ( text[i] == '\\' ) ignorenext = 1;
+    } else {
+      ignorenext = 0;
+    }
   }
   char **liness=(char**)_allocsafe(sizeof(char*)*lines);
-//  char *liness[lines];
   if(lines==1) {
     liness[0]=dupstring(text);
   } else {
@@ -895,12 +902,24 @@ int createLines(const char *text,char ***ret_lines)
     for(i=0;i<lines;i++) {
       liness[i]=dupstring(tstr1);
       int j;
+      ignorenext = 0;
       for(j=0;j<(int)strlen(liness[i]);j++) {
-        if(liness[i][j]=='|') {
-          liness[i][j]=0;
-          break;
-        }
+	if ( ignorenext == 0 ) {
+	  if(liness[i][j]=='|') {
+	    liness[i][j]=0;
+	    break;
+	  } else if ( liness[i][j] == '\\' ) ignorenext = 1;
+	} else {
+	  ignorenext = 0;
+	}
       }
+      for ( p = j = 0; liness[i][j] != '\0'; j++ ) {
+        if ( ( liness[i][j] == '\\' ) && ( liness[i][j+1] == '|' ) ) {
+	} else {
+	  liness[i][p++] = liness[i][j];
+	}
+      }
+      liness[i][p] = '\0';
       if((i+1)>=lines) break; //last line
       tstr2=dupstring(tstr1+j+1);
       _freesafe(tstr1);
