@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: changesymlinkop.cc,v 1.6 2002/03/16 23:26:42 ralf Exp $ */
+/* $Id: changesymlinkop.cc,v 1.8 2003/05/04 00:35:21 ralf Exp $ */
 
 #include "changesymlinkop.h"
 #include "listermode.h"
@@ -27,7 +27,7 @@
 
 const char *ChangeSymlinkOp::name="ChangeSymlinkOp";
 
-ChangeSymlinkOp::ChangeSymlinkOp(Worker *tworker):FunctionProto(tworker)
+ChangeSymlinkOp::ChangeSymlinkOp() : FunctionProto()
 {
 }
 
@@ -36,9 +36,9 @@ ChangeSymlinkOp::~ChangeSymlinkOp()
 }
 
 ChangeSymlinkOp*
-ChangeSymlinkOp::duplicate()
+ChangeSymlinkOp::duplicate() const
 {
-  ChangeSymlinkOp *ta=new ChangeSymlinkOp(worker);
+  ChangeSymlinkOp *ta=new ChangeSymlinkOp();
   return ta;
 }
 
@@ -60,7 +60,7 @@ ChangeSymlinkOp::run(ActionMessage *msg)
   ListerMode *lm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       startlister=l1;
       lm1=startlister->getActiveMode();
@@ -92,6 +92,7 @@ ChangeSymlinkOp::normalmodechangesl()
   if(lm1->isType("NormalMode")!=true) return 1;
   nm1=(NormalMode*)lm1;
 
+  memset( &chslorder, 0, sizeof( chslorder ) );
   if(am->mode==am->AM_MODE_ONLYACTIVE)
     chslorder.source=chslorder.NM_ONLYACTIVE;
   else if(am->mode==am->AM_MODE_DNDACTION) {
