@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: chmodop.cc,v 1.12 2002/03/16 23:27:08 ralf Exp $ */
+/* $Id: chmodop.cc,v 1.20 2003/05/04 00:35:57 ralf Exp $ */
 
 #include "chmodop.h"
 #include "listermode.h"
@@ -27,12 +27,13 @@
 
 const char *ChModOp::name="ChModOp";
 
-ChModOp::ChModOp(Worker *tworker):FunctionProto(tworker)
+ChModOp::ChModOp() : FunctionProto()
 {
-  onfiles=false;
+  onfiles = true;
   ondirs=false;
   recursive=false;
-  requestflags=false;
+  requestflags = true;
+  hasConfigure = true;
 }
 
 ChModOp::~ChModOp()
@@ -40,9 +41,9 @@ ChModOp::~ChModOp()
 }
 
 ChModOp*
-ChModOp::duplicate()
+ChModOp::duplicate() const
 {
-  ChModOp *ta=new ChModOp(worker);
+  ChModOp *ta=new ChModOp();
   ta->onfiles=onfiles;
   ta->ondirs=ondirs;
   ta->recursive=recursive;
@@ -68,7 +69,7 @@ ChModOp::run(ActionMessage *msg)
   ListerMode *lm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       startlister=l1;
       lm1=startlister->getActiveMode();
@@ -83,32 +84,11 @@ ChModOp::run(ActionMessage *msg)
 bool
 ChModOp::save(Datei *fh)
 {
-  int size=4*Datei::getUCharSize();
-  fh->putInt(size);
-
-  fh->putUChar((onfiles==true)?1:0);
-  fh->putUChar((ondirs==true)?1:0);
-  fh->putUChar((recursive==true)?1:0);
-  fh->putUChar((requestflags==true)?1:0);
-  return true;
-}
-
-int
-ChModOp::load(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  int cs=Datei::getUCharSize();
-  if(chunksize>=(4*cs)) {
-    if(fh->getUChar()==1) onfiles=true; else onfiles=false;
-    if(fh->getUChar()==1) ondirs=true; else ondirs=false;
-    if(fh->getUChar()==1) recursive=true; else recursive=false;
-    if(fh->getUChar()==1) requestflags=true; else requestflags=false;
-    chunksize-=4*cs;
-  }
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
-  }
+  if ( fh == NULL ) return false;
+  WConfig::configPutPairBool( fh, "onfiles", onfiles );
+  WConfig::configPutPairBool( fh, "ondirs", ondirs );
+  WConfig::configPutPairBool( fh, "recursive", recursive );
+  WConfig::configPutPairBool( fh, "requestflags", requestflags );
   return true;
 }
 
@@ -143,6 +123,7 @@ ChModOp::normalmodechmod()
   }
   
   if(cont==true) {
+    memset( &cmorder, 0, sizeof( cmorder ) );
     // detect what to copy
     if(am->mode==am->AM_MODE_ONLYACTIVE)
       cmorder.source=cmorder.NM_ONLYACTIVE;
@@ -186,7 +167,7 @@ ChModOp::configure()
 int
 ChModOp::doconfigure(int mode)
 {
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   Button *okb,*cb;
   AWindow *win;
   ChooseButton *ofcb,*odcb,*rfcb=NULL,*rcb;
@@ -202,6 +183,9 @@ ChModOp::doconfigure(int mode)
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(38));
   win->create(NULL,10,10,w,h,0,tstr);
   _freesafe(tstr);
+  
+  Text *tt1 = (Text*)win->add( new Text( aguix, x, y, catalog.getLocaleCom( 38 ), 1 ) );
+  y += tt1->getHeight() + 5;
 
   ofcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(onfiles==true)?1:0,
                                                 catalog.getLocale(295),LABEL_RIGHT,1,0));
@@ -254,6 +238,8 @@ ChModOp::doconfigure(int mode)
   y+=okb->getHeight()+5;
   
   h=y;
+  okb->takeFocus();
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMaxSize(w,h);
   win->setMinSize(w,h);
@@ -269,7 +255,7 @@ ChModOp::doconfigure(int mode)
           if(msg->button.button==okb) endmode=0;
           else if(msg->button.button==cb) endmode=1;
           break;
-        case AG_KEYRELEASED:
+        case AG_KEYPRESSED:
           if(win->isParent(msg->key.window,false)==true) {
             switch(msg->key.key) {
               case XK_1:
@@ -282,7 +268,8 @@ ChModOp::doconfigure(int mode)
                 rcb->setState((rcb->getState()==1)?0:1);
                 break;
               case XK_Return:
-                endmode=0;
+                if ( cb->hasFocus() == false )
+                  endmode=0;
                 break;
               case XK_Escape:
                 endmode=1;
