@@ -18,16 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: copyop.cc,v 1.31 2002/03/22 23:12:50 ralf Exp $ */
+/* $Id: copyop.cc,v 1.60 2004/01/21 21:41:56 ralf Exp $ */
 
-#include "copyop.h"
 #include "listermode.h"
 #include "normalmode.h"
 #include "worker.h"
+#include "copyop.h"
 
 const char *CopyOp::name="CopyOp";
 
-CopyOp::CopyOp(Worker *tworker):FunctionProto(tworker)
+CopyOp::CopyOp() : FunctionProto()
 {
   follow_symlinks=false;
   move=false;
@@ -37,21 +37,18 @@ CopyOp::CopyOp(Worker *tworker):FunctionProto(tworker)
   request_flags=false;
   overwrite=COPYOP_OVERWRITE_NORMAL;
   fastcopy=COPYOP_NORMALCOPY;
-  cp_com[0]=dupstring("");
-  cp_com[1]=dupstring("");
   preserve_attr=true;
+  hasConfigure = true;
 }
 
 CopyOp::~CopyOp()
 {
-  _freesafe(cp_com[0]);
-  _freesafe(cp_com[1]);
 }
 
 CopyOp*
-CopyOp::duplicate()
+CopyOp::duplicate() const
 {
-  CopyOp *ta=new CopyOp(worker);
+  CopyOp *ta=new CopyOp();
   ta->follow_symlinks=follow_symlinks;
   ta->move=move;
   ta->do_rename=do_rename;
@@ -60,10 +57,6 @@ CopyOp::duplicate()
   ta->request_flags=request_flags;
   ta->overwrite=overwrite;
   ta->fastcopy=fastcopy;
-  _freesafe(ta->cp_com[0]);
-  _freesafe(ta->cp_com[1]);
-  ta->cp_com[0]=dupstring(cp_com[0]);
-  ta->cp_com[1]=dupstring(cp_com[1]);
   ta->preserve_attr=preserve_attr;
   return ta;
 }
@@ -86,10 +79,10 @@ CopyOp::run(ActionMessage *msg)
   ListerMode *lm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       startlister=l1;
-      endlister=worker->getOtherLister(startlister);
+      endlister = msg->getWorker()->getOtherLister(startlister);
       lm1=startlister->getActiveMode();
       if(lm1!=NULL)
         if(lm1->isType("NormalMode")==true)
@@ -104,107 +97,36 @@ CopyOp::run(ActionMessage *msg)
 bool
 CopyOp::save(Datei *fh)
 {
-  int size=9*Datei::getUCharSize()+2*Datei::getIntSize();
-  size+=strlen(cp_com[0]);
-  size+=strlen(cp_com[1]);
-  fh->putInt(size);
-
-  fh->putUChar((follow_symlinks==true)?1:0);
-  fh->putUChar((move==true)?1:0);
-  fh->putUChar((do_rename==true)?1:0);
-  fh->putUChar((same_dir==true)?1:0);
-  fh->putUChar((request_dest==true)?1:0);
-  fh->putUChar((request_flags==true)?1:0);
+  if ( fh == NULL ) return false;
+  WConfig::configPutPairBool( fh, "followsymlinks", follow_symlinks );
+  WConfig::configPutPairBool( fh, "move", move );
+  WConfig::configPutPairBool( fh, "rename", do_rename );
+  WConfig::configPutPairBool( fh, "samedir", same_dir );
+  WConfig::configPutPairBool( fh, "requestdest", request_dest );
+  WConfig::configPutPairBool( fh, "requestflags", request_flags );
   switch(overwrite) {
     case COPYOP_OVERWRITE_ALWAYS:
-      fh->putUChar(1);
+      WConfig::configPutPair( fh, "overwrite", "always" );
       break;
     case COPYOP_OVERWRITE_NEVER:
-      fh->putUChar(2);
+      WConfig::configPutPair( fh, "overwrite", "never" );
       break;
     default:
-      fh->putUChar(0);
+      WConfig::configPutPair( fh, "overwrite", "normal" );
+      break;
   }
   switch(fastcopy) {
     case COPYOP_FASTCOPY:
-      fh->putUChar(1);
+      WConfig::configPutPair( fh, "copymode", "fast" );
       break;
     case COPYOP_FASTESTCOPY:
-      fh->putUChar(2);
+      WConfig::configPutPair( fh, "copymode", "fastest" );
       break;
     default:
-      fh->putUChar(0);
-  }
-  fh->putInt(strlen(cp_com[0]));
-  fh->putString(cp_com[0]);
-  fh->putInt(strlen(cp_com[1]));
-  fh->putString(cp_com[1]);
-  fh->putUChar((preserve_attr==true)?1:0);
-  return true;
-}
-
-int
-CopyOp::load(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  int cs=Datei::getUCharSize(),
-      is=Datei::getIntSize(),
-      len;
-  unsigned char c1;
-  if(chunksize>=(8*cs+2*is)) {
-    if(fh->getUChar()==1) follow_symlinks=true; else follow_symlinks=false;
-    if(fh->getUChar()==1) move=true; else move=false;
-    if(fh->getUChar()==1) do_rename=true; else do_rename=false;
-    if(fh->getUChar()==1) same_dir=true; else same_dir=false;
-    if(fh->getUChar()==1) request_dest=true; else request_dest=false;
-    if(fh->getUChar()==1) request_flags=true; else request_flags=false;
-    c1=fh->getUChar();
-    switch(c1) {
-      case 1:
-        overwrite=COPYOP_OVERWRITE_ALWAYS;
-        break;
-      case 2:
-        overwrite=COPYOP_OVERWRITE_NEVER;
-        break;
-      default:
-        overwrite=COPYOP_OVERWRITE_NORMAL;
-    }
-    c1=fh->getUChar();
-    switch(c1) {
-      case 1:
-        fastcopy=COPYOP_FASTCOPY;
-        break;
-      case 2:
-        fastcopy=COPYOP_FASTESTCOPY;
-        break;
-      default:
-        fastcopy=COPYOP_NORMALCOPY;
-    }
-    chunksize-=8*cs;
-
-    len=fh->getInt();
-    chunksize-=is;
-    _freesafe(cp_com[0]);
-    cp_com[0]=fh->getString(len);
-    if(cp_com[0]==NULL) cp_com[0]=dupstring("");
-    chunksize-=len;
-
-    len=fh->getInt();
-    chunksize-=is;
-    _freesafe(cp_com[1]);
-    cp_com[1]=fh->getString(len);
-    if(cp_com[1]==NULL) cp_com[1]=dupstring("");
-    chunksize-=len;
-    
-    if(chunksize>=cs) {
-      if(fh->getUChar()==1) preserve_attr=true; else preserve_attr=false;
-      chunksize-=cs;
-    }
-  }
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
+      WConfig::configPutPair( fh, "copymode", "normal" );
+      break;
   }
+  WConfig::configPutPairBool( fh, "preserveattr", preserve_attr );
   return true;
 }
 
@@ -259,12 +181,12 @@ CopyOp::normalmodecopy()
     trequest_dest=request_dest;
     toverwrite=overwrite;
     tfastcopy=fastcopy;
-    tcp_com[0]=dupstring(cp_com[0]);
-    tcp_com[1]=dupstring(cp_com[1]);
     tpreserve_attr=preserve_attr;
   }
   
   if(cont==true) {
+    memset( &copyorder, 0, sizeof( copyorder ) );
+    copyorder.ignoreLosedAttr = false;
     // detect what to copy
     if(am->mode==am->AM_MODE_ONLYACTIVE)
       copyorder.source=copyorder.NM_ONLYACTIVE;
@@ -302,7 +224,7 @@ CopyOp::normalmodecopy()
     if(trequest_dest==true)
       do_request=true;
     else if(tsame_dir==true) {
-      tstr=nm1->getActualDir();
+      tstr = nm1->getCurrentDir();
       if(tstr!=NULL)
         dest=dupstring(tstr);
     } else if(am->mode==am->AM_MODE_DNDACTION) {
@@ -311,7 +233,7 @@ CopyOp::normalmodecopy()
       else dest=NULL;
     } else {
       if(nm2!=NULL) {
-        tstr=nm2->getActualDir();
+        tstr = nm2->getCurrentDir();
         if(tstr!=NULL)
           dest=dupstring(tstr);
         else
@@ -321,8 +243,22 @@ CopyOp::normalmodecopy()
     }
 
     if(do_request==true) {
+      const char *defdir;
       if(dest!=NULL) _freesafe(dest);
-      if(requestdest(nm1->getActualDir(),&dest)!=0) {
+      
+      if ( nm2 != NULL ) {
+        defdir = nm2->getCurrentDir();
+        if ( defdir != NULL ) {
+          if ( strlen( defdir ) < 1 )
+            defdir = nm1->getCurrentDir();
+        } else {
+          defdir = nm1->getCurrentDir();
+        }
+      } else {
+        defdir = nm1->getCurrentDir();
+      }
+      
+      if ( requestdest( defdir, &dest ) != 0 ) {
         _freesafe(dest);
         dest=NULL;
       }
@@ -331,9 +267,9 @@ CopyOp::normalmodecopy()
        otherwise: */
     if(dest!=NULL) {
       /* if dest starts with no slash then take it as local dir
-         so add actualDir */
+         so add currentDir */
       if(dest[0]!='/') {
-        tstr=catstring(nm1->getActualDir(),"/");
+        tstr=catstring(nm1->getCurrentDir(),"/");
         tstr2=catstring(tstr,dest);
         _freesafe(tstr);
         _freesafe(dest);
@@ -365,11 +301,7 @@ CopyOp::normalmodecopy()
       }
       copyorder.destdir=dest;
       // now start copy process
-      copyorder.cowin=new CopyOpWin(worker->getAGUIX());
-      copyorder.cp_com[0]=NULL;
-      copyorder.cp_com[1]=NULL;
-      if(strlen(tcp_com[0])>0) copyorder.cp_com[0]=tcp_com[0];
-      if(strlen(tcp_com[1])>0) copyorder.cp_com[1]=tcp_com[1];
+      copyorder.cowin=new CopyOpWin( am->getWorker()->getAGUIX());
       nm1->copy(&copyorder);
       delete copyorder.cowin;
     }
@@ -378,8 +310,6 @@ CopyOp::normalmodecopy()
       if(specialsource!=NULL) _freesafe(specialsource);
       delete copyorder.sources;
     }
-    _freesafe(tcp_com[0]);
-    _freesafe(tcp_com[1]);
   }
   return 0;
 }
@@ -387,9 +317,8 @@ CopyOp::normalmodecopy()
 int
 CopyOp::doconfigure(int mode)
 {
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   Button *okb,*cb;
-  StringGadget *ccsg[2];
   AWindow *win;
   ChooseButton *fscb,*mcb,*rcb,*sdcb,*rfcb=NULL,*rdcb,*pacb;
 //  CycleButton *ocyb,*fccyb;
@@ -399,8 +328,8 @@ CopyOp::doconfigure(int mode)
   AGMessage *msg;
   int endmode=-1;
   Requester *req;
-  char *textstr,*buttonstr,*tstr;
-  int erg,i;
+  char *tstr;
+  int i;
   
   req=new Requester(aguix);
   w=h=10;
@@ -411,6 +340,9 @@ CopyOp::doconfigure(int mode)
   win->create(NULL,10,10,w,h,0,tstr);
   _freesafe(tstr);
 
+  ttext = (Text*)win->add( new Text( aguix, x, y, catalog.getLocaleCom( 8 ), 1 ) );
+  y += ttext->getHeight() + 5;
+
   fscb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(follow_symlinks==true)?1:0,
                                                 catalog.getLocale(298),LABEL_RIGHT,1,0));
   y+=fscb->getHeight()+5;
@@ -495,25 +427,6 @@ CopyOp::doconfigure(int mode)
   if(tw>w) w=tw;
   x=5;
 
-  ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(310),1));
-  x+=ttext->getWidth()+5;
-  tw=w-x-5;
-  if(tw<50) tw=50;
-  ccsg[0]=(StringGadget*)win->add(new StringGadget(aguix,x,y,tw,cp_com[0],0));
-  y+=ccsg[0]->getHeight()+5;
-  tw=ccsg[0]->getWidth()+5+x;
-  if(tw>w) w=tw;
-  x=5;
-  ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(311),1));
-  x+=ttext->getWidth()+5;
-  tw=w-x-5;
-  if(tw<50) tw=50;
-  ccsg[1]=(StringGadget*)win->add(new StringGadget(aguix,x,y,tw,cp_com[1],0));
-  y+=ccsg[1]->getHeight()+5;
-  tw=ccsg[1]->getWidth()+5+x;
-  if(tw>w) w=tw;
-  x=5;
-
   if(mode==0) {
     rfcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(request_flags==true)?1:0,
                                                   catalog.getLocale(294),LABEL_RIGHT,1,0));
@@ -546,10 +459,9 @@ CopyOp::doconfigure(int mode)
                                   0));
   y+=okb->getHeight()+5;
   
-  ccsg[0]->resize(w-5-ccsg[0]->getX(),ccsg[0]->getHeight());
-  ccsg[1]->resize(w-5-ccsg[1]->getX(),ccsg[1]->getHeight());
-
   h=y;
+  okb->takeFocus();
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMaxSize(w,h);
   win->setMinSize(w,h);
@@ -563,52 +475,10 @@ CopyOp::doconfigure(int mode)
           break;
         case AG_BUTTONCLICKED:
           if(msg->button.button==okb) {
-            i=0;
-            for(int j=0;j<2;j++) {
-              if(NormalMode::isCorrectCPProg(ccsg[j]->getText())==false) {
-                // show requester
-                textstr=catalog.getLocale(312);
-                buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(313))+1+
-                                            strlen(catalog.getLocale(314))+1);
-                sprintf(buttonstr,"%s|%s",catalog.getLocale(313),
-                                          catalog.getLocale(314));
-                erg=req->request(catalog.getLocale(125),textstr,buttonstr);
-                _freesafe(buttonstr);
-                if(erg==0) {
-                  ccsg[(j==0)?1:0]->deactivate();
-                  ccsg[j]->activate();
-                } else {
-                  ccsg[j]->setText("");
-                }
-                break;
-              } else i++;
-            }
-            if(i==2) endmode=0;
+            endmode = 0;
           } else if(msg->button.button==cb) endmode=1;
           break;
-	case AG_STRINGGADGET_DEACTIVATE:
-          if((msg->stringgadget.sg==ccsg[0])||
-             (msg->stringgadget.sg==ccsg[1])) {
-            if(NormalMode::isCorrectCPProg(msg->stringgadget.sg->getText())==false) {
-              // show requester
-              textstr=catalog.getLocale(312);
-              buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(313))+1+
-                                          strlen(catalog.getLocale(314))+1);
-              sprintf(buttonstr,"%s|%s",catalog.getLocale(313),
-                                        catalog.getLocale(314));
-              erg=req->request(catalog.getLocale(125),textstr,buttonstr);
-              _freesafe(buttonstr);
-              if(erg==0) {
-                if(msg->stringgadget.sg==ccsg[0]) ccsg[1]->deactivate();
-                else ccsg[0]->deactivate();
-                msg->stringgadget.sg->activate();
-              } else {
-                msg->stringgadget.sg->setText("");
-              }
-            }
-          }
-	  break;
-        case AG_KEYRELEASED:
+        case AG_KEYPRESSED:
           if(win->isParent(msg->key.window,false)==true) {
             switch(msg->key.key) {
               case XK_1:
@@ -637,27 +507,9 @@ CopyOp::doconfigure(int mode)
                 fccyb->setOption(i);
                 break;
               case XK_Return:
-                i=0;
-                for(int j=0;j<2;j++) {
-                  if(NormalMode::isCorrectCPProg(ccsg[j]->getText())==false) {
-                    // show requester
-                    textstr=catalog.getLocale(312);
-                    buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(313))+1+
-                                                strlen(catalog.getLocale(314))+1);
-                    sprintf(buttonstr,"%s|%s",catalog.getLocale(313),
-                                              catalog.getLocale(314));
-                    erg=req->request(catalog.getLocale(125),textstr,buttonstr);
-                    _freesafe(buttonstr);
-                    if(erg==0) {
-                      ccsg[(j==0)?1:0]->deactivate();
-                      ccsg[j]->activate();
-                    } else {
-                      ccsg[j]->setText("");
-                    }
-                    break;
-                  } else i++;
+                if ( cb->hasFocus() == false ) {
+                  endmode = 0;
                 }
-                if(i==2) endmode=0;
                 break;
               case XK_Escape:
                 endmode=1;
@@ -680,8 +532,6 @@ CopyOp::doconfigure(int mode)
       tsame_dir=(sdcb->getState()==1)?true:false;
       trequest_dest=(rdcb->getState()==1)?true:false;
       tpreserve_attr=(pacb->getState()==1)?true:false;
-      tcp_com[0]=dupstring(ccsg[0]->getText());
-      tcp_com[1]=dupstring(ccsg[1]->getText());
       toverwrite=COPYOP_OVERWRITE_NORMAL;
 /*      switch(ocyb->getSelectedOption()) {
         case 1:
@@ -714,10 +564,6 @@ CopyOp::doconfigure(int mode)
       request_dest=(rdcb->getState()==1)?true:false;
       request_flags=(rfcb->getState()==1)?true:false;
       preserve_attr=(pacb->getState()==1)?true:false;
-      _freesafe(cp_com[0]);
-      _freesafe(cp_com[1]);
-      cp_com[0]=dupstring(ccsg[0]->getText());
-      cp_com[1]=dupstring(ccsg[1]->getText());
       overwrite=COPYOP_OVERWRITE_NORMAL;
 /*      switch(ocyb->getSelectedOption()) {
         case 1:
@@ -758,10 +604,11 @@ CopyOp::configure()
 }
 
 int
-CopyOp::requestdest(char *defaultstr,char **dest)
+CopyOp::requestdest( const char *defaultstr, char **dest )
 {
-  Requester *req=new Requester(worker->getAGUIX());
-  char *textstr,*buttonstr;
+  Requester *req=new Requester( Worker::getAGUIX());
+  char *buttonstr;
+  const char *textstr;
   int erg;
   
   textstr=catalog.getLocale(212);
@@ -769,7 +616,12 @@ CopyOp::requestdest(char *defaultstr,char **dest)
                               strlen(catalog.getLocale(8))+1);
   sprintf(buttonstr,"%s|%s",catalog.getLocale(11),
                             catalog.getLocale(8));
-  erg=req->string_request(catalog.getLocale(123),textstr,(defaultstr!=NULL)?defaultstr:"",buttonstr,dest);
+  erg = req->string_request( catalog.getLocale( 123 ),
+                             textstr,
+                             ( defaultstr != NULL ) ? defaultstr : "",
+                             buttonstr,
+                             dest,
+                             Requester::REQUEST_SELECTALL );
   _freesafe(buttonstr);
   delete req;
   return erg;
@@ -815,6 +667,11 @@ void CopyOp::setPreserveAttr(bool nv)
   preserve_attr=nv;
 }
 
+void CopyOp::setFastcopy( fastcopy_t nv )
+{
+  fastcopy = nv;
+}
+
 /*******************
  * Class CopyOpWin *
  *******************/
@@ -824,8 +681,8 @@ CopyOpWin::CopyOpWin(AGUIX*taguix)
   this->aguix=taguix;
   source=dupstring("");
   dest=dupstring("");
-  copied_bytes_actfile=0;
-  bytes_actfile=0;
+  copied_bytes_curfile=0;
+  bytes_curfile=0;
   copied_bytes=0;
   bytes=0;
   copied_files=0;
@@ -838,6 +695,21 @@ CopyOpWin::CopyOpWin(AGUIX*taguix)
   melw = 0;
   filenamespace[0] = 0;
   filenamespace[1] = 0;
+  ismessage[0] = false;
+  ismessage[1] = false;
+
+  update_sourcetext = false;
+  update_desttext = false;
+  update_filetext = false;
+  update_globtext = false;
+  update_files2gotext = false;
+  update_dirs2gotext = false;
+  
+  mintime = 1000000;
+  lastbytes = 0;
+  lastrate = 0.0;
+  
+  glob_percent = 0;
 }
 
 CopyOpWin::~CopyOpWin()
@@ -863,8 +735,10 @@ CopyOpWin::open(bool tslim)
   tx=5;
   ty=5;
   sourcetext=(Text*)window->add(new Text(aguix,tx,ty,"",1));
+  ismessage[0] = true;
   ty+=sourcetext->getHeight()+5;
   desttext=(Text*)window->add(new Text(aguix,tx,ty,"",1));
+  ismessage[1] = true;
   ty+=desttext->getHeight()+5;
   fsb=(SolidButton*)window->add(new SolidButton(aguix,tx+1,ty+1,w-2*(tx+1),"",2,3,0));
   sbw=fsb->getWidth();
@@ -873,8 +747,14 @@ CopyOpWin::open(bool tslim)
   ty+=bb1->getHeight()+5;
   tx=10;
   filetext=(Text*)window->add(new Text(aguix,tx,ty,"",1));
-  tx=5;
   ty+=filetext->getHeight()+5;
+
+  ttext = (Text*)window->add( new Text( aguix, tx, ty, catalog.getLocale( 90 ), 1 ) );
+  tx += ttext->getWidth() + 5;
+  fileavgtext = (Text*)window->add( new Text( aguix, tx, ty, "", 1 ) );
+  ty += fileavgtext->getHeight() + 5;
+
+  tx=5;
   bb2=(BevelBox*)window->add(new BevelBox(aguix,tx,ty,w-2*tx,2,0));
   ty+=bb2->getHeight()+5;
 
@@ -915,6 +795,7 @@ CopyOpWin::open(bool tslim)
   cb->move(w/2-cb->getWidth()/2,cb->getY());
   ty+=cb->getHeight()+5;
   h=ty;
+  window->setDoTabCycling( true );
   window->resize(w,h);
   window->setMinSize(w,h);
   //  window->setMaxSize(w,h);
@@ -942,7 +823,7 @@ CopyOpWin::close()
 void
 CopyOpWin::starttimer()
 {
-  starttime=time(NULL);
+  gettimeofday( &starttime, NULL );
 }
 
 void
@@ -970,11 +851,11 @@ CopyOpWin::set_bytes_to_copy(loff_t nbytes)
 }
 
 void
-CopyOpWin::set_bytes_to_copy_actfile(loff_t nbytes)
+CopyOpWin::set_bytes_to_copy_curfile( loff_t nbytes )
 {
-  if(nbytes>0) bytes_actfile=nbytes;
-  else bytes_actfile=nbytes;
-  update_filetext=true;
+  if ( nbytes > 0 ) bytes_curfile = nbytes;
+  else bytes_curfile = 0;
+  update_filetext = true;
 }
 
 void
@@ -993,14 +874,14 @@ CopyOpWin::file_finished()
 }
 
 void
-CopyOpWin::add_actbytes_copied(loff_t nbytes)
+CopyOpWin::add_curbytes_copied( loff_t nbytes )
 {
-  if(nbytes>0) {
-    copied_bytes_actfile+=nbytes;
-    copied_bytes+=nbytes;
+  if ( nbytes > 0 ) {
+    copied_bytes_curfile += nbytes;
+    copied_bytes += nbytes;
   }
-  update_filetext=true;
-  update_globtext=true;
+  update_filetext = true;
+  update_globtext = true;
 }
 
 void
@@ -1016,11 +897,17 @@ CopyOpWin::newfile(char *name,char *ndest)
     dest=dupstring(ndest);
   else
     dest=dupstring("");
-  filestarttime=time(NULL);
+  gettimeofday( &filestarttime, NULL );
   update_sourcetext=true;
   update_desttext=true;
-  copied_bytes_actfile=0;
+  ismessage[0] = false;
+  ismessage[1] = false;
+  copied_bytes_curfile = 0;
   update_filetext=true;
+
+  lastbytes = 0;
+  lasttime = filestarttime;
+  lastrate = 0.0;
 }
 
 int
@@ -1031,16 +918,23 @@ CopyOpWin::redraw()
    *  1 when clicked Cancel-Button or closed window
    *  other not implemented */
   int returnvalue=0;
-  time_t acttime=0,difftime2,rtime;
-  char tstr[1024],buf1[64],buf2[64],*tstr2;
-  float rate;
-  int tm;
+  long rtime;
+  double drtime;
+  struct timeval curtime;
+  double difftime2, lastdiff2;
+  char tstr[1024],
+       buf1[A_BYTESFORNICENUMBER(loff_t)],
+       buf2[A_BYTESFORNICENUMBER(loff_t)],
+       *tstr2;
+  double rate;
+  int tw;
   bool doresize = false;
+  int newglob_percent;
   
   char *tstr3;
 
-  aguix->Flush();
-  if ( update_sourcetext == true ) {
+  timerclear( &curtime );
+  if ( ( update_sourcetext == true ) && ( ismessage[0] == false ) ) {
     if ( source != NULL ) {
       tstr3 = Datei::shrinkFilename( source, filenamespace[0] );
       if ( tstr3 == NULL)
@@ -1057,7 +951,7 @@ CopyOpWin::redraw()
     /* no need for explicit redraw */
     update_sourcetext=false;
   }
-  if ( update_desttext == true ) {
+  if ( ( update_desttext == true ) && ( ismessage[1] == false ) ) {
     if ( dest != NULL ) {
       tstr3 = Datei::shrinkFilename( dest, filenamespace[1] );
       if ( tstr3 == NULL)
@@ -1075,78 +969,113 @@ CopyOpWin::redraw()
     update_desttext=false;
   }
   if(update_filetext==true) {
-    if(acttime==0) acttime=time(NULL);
-    difftime2=acttime-filestarttime;
-    if(difftime2>0) {
-      rate=(float)copied_bytes_actfile/(float)difftime2;
-    } else rate=0.0;
-//    sprintf(tstr,"%ld / %ld @ %.2f KB/s",copied_bytes_actfile,bytes_actfile,rate/1024);
-    buf1[MakeLong2NiceStr(buf1,copied_bytes_actfile)]='\0';
-    buf2[MakeLong2NiceStr(buf2,bytes_actfile)]='\0';
-    sprintf(tstr,"%s / %s @ %.2f KB/s",buf1,buf2,rate/1024);
+    if ( ! timerisset( &curtime ) ) gettimeofday( &curtime, NULL );
+    difftime2 = diffgtod( &curtime, &filestarttime );
+    lastdiff2 = diffgtod( &curtime, &lasttime );
+
+    if ( lastdiff2 >= mintime ) {
+      lastrate = (double)( copied_bytes_curfile - lastbytes ) * 1000000.0 / lastdiff2;
+      lastbytes = copied_bytes_curfile;
+      lasttime = curtime;
+    }
+    
+    buf1[MakeLong2NiceStr(buf1,copied_bytes_curfile)]='\0';
+    buf2[MakeLong2NiceStr(buf2,bytes_curfile)]='\0';
+    snprintf( tstr, sizeof( tstr ), "%s / %s @ %.2f KB/s", buf1, buf2, lastrate / 1024.0 );
+    tstr[ sizeof( tstr ) - 1 ] = '\0';
     filetext->setText(tstr);
-    tm = filetext->getX() + filetext->getWidth() + window->getBorderWidth();
-    if ( tm > melw ) {
-      melw = tm;
+    tw = filetext->getX() + filetext->getWidth() + window->getBorderWidth();
+    if ( tw > melw ) {
+      melw = tw;
       doresize = true;
     }
     
-    if(bytes_actfile>0)
-      rate=(float)copied_bytes_actfile/(float)bytes_actfile;
+    if(difftime2>0) {
+      rate = (double)copied_bytes_curfile * 1000000.0 / difftime2;
+    } else rate=0.0;
+    snprintf( tstr, sizeof( tstr ), "%.2f KB/s", rate / 1024.0 );
+    tstr[ sizeof( tstr ) - 1 ] = '\0';
+    fileavgtext->setText( tstr );
+
+    if(bytes_curfile>0)
+      rate = (double)copied_bytes_curfile/(double)bytes_curfile;
     else
       rate=0.0;
     if ( rate < 0.01 )
       rate = 0.0;
-    sprintf(tstr,"%.4g %%",rate*100);
+    snprintf( tstr, sizeof( tstr ), "%.2f %%", rate * 100.0 );
+    tstr[ sizeof( tstr ) - 1 ] = '\0';
     fsb->setText(tstr);
     fsb->resize( w_max( (int)( sbw * rate ), 2 ), fsb->getHeight() );
     update_filetext=false;
   }
   if(update_globtext==true) {
-    if(acttime==0) acttime=time(NULL);
-    difftime2=acttime-starttime;
-    if(difftime2>0) {
-      rate=(float)copied_bytes/(float)difftime2;
+    if ( ! timerisset( &curtime ) ) gettimeofday( &curtime, NULL );
+    difftime2 = diffgtod( &curtime, &starttime );
+    if(difftime2>0.0) {
+      rate = (double)copied_bytes * 1000000.0 / difftime2;
     } else rate=0.0;
-//    sprintf(tstr,"%ld / %ld @ %.2f KB/s",copied_bytes,bytes,rate/1024);
     buf1[MakeLong2NiceStr(buf1,copied_bytes)]='\0';
     if(slim==true) {
-      sprintf(tstr,"%s @ %.2f KB/s",buf1,rate/1024);
+      snprintf( tstr, sizeof( tstr ), "%s @ %.2f KB/s", buf1, rate / 1024.0 );
+      tstr[ sizeof( tstr ) - 1 ] = '\0';
     } else {
       buf2[MakeLong2NiceStr(buf2,bytes)]='\0';
-      sprintf(tstr,"%s / %s @ %.2f KB/s",buf1,buf2,rate/1024);
+      snprintf( tstr, sizeof( tstr ), "%s / %s @ %.2f KB/s", buf1, buf2, rate / 1024.0 );
+      tstr[ sizeof( tstr ) - 1 ] = '\0';
     }
     
     globtext->setText(tstr);
-    tm = globtext->getX() + globtext->getWidth() + window->getBorderWidth();
-    if ( tm > melw ) {
-      melw = tm;
+    tw = globtext->getX() + globtext->getWidth() + window->getBorderWidth();
+    if ( tw > melw ) {
+      melw = tw;
       doresize = true;
     }
     
     if(slim==false) {
       if(bytes>0)
-        rate=(float)copied_bytes/(float)bytes;
+        rate = (double)copied_bytes/(double)bytes;
       else
         rate=0.0;
       if ( rate < 0.01 )
         rate = 0.0;
-      sprintf(tstr,"%.4g %%",rate*100);
+      snprintf( tstr, sizeof( tstr ), "%.2f %%", rate * 100.0 );
+      tstr[ sizeof( tstr ) - 1 ] = '\0';
       gsb->setText(tstr);
       gsb->resize( w_max( (int)( sbw * rate ), 2 ), gsb->getHeight() );
+      
+      newglob_percent = (int)( rate * 100.0 );
+      if ( newglob_percent != glob_percent ) {
+        glob_percent = newglob_percent;
+        tstr3 = (char*)_allocsafe( strlen( catalog.getLocaleCom( 8 ) ) +
+                                   strlen( " ( %)" ) + 
+                                   A_BYTESFORNUMBER( glob_percent ) +
+                                   1 );
+        if ( ( glob_percent > 0 ) && ( glob_percent <= 100 ) ) {
+          sprintf( tstr3, "%s (%d %%)", catalog.getLocaleCom( 8 ), glob_percent );
+        } else {
+          sprintf( tstr3, "%s", catalog.getLocaleCom( 8 ) );
+        }
+        window->setTitle( tstr3 );
+        _freesafe( tstr3 );
+      }
+      
       if(rate>0) {
-        rtime=(time_t)((float)difftime2/rate);
-        rtime-=difftime2;
+        drtime = difftime2 / rate;
+        drtime -= difftime2;
+        rtime = (long)( drtime / 1000000.0 );
       } else
         rtime = 700000;  // no rate available => no real time calculatable
+      rtime++; // add a second because the last second will shown as 0:00
+               // and at least I don't like this
       if ( rtime > 604800 ) // more then 7 days
         sprintf( tstr, "--:--" );
       else 
         sprintf( tstr, "%ld:%02ld", rtime / 60, rtime % 60 );
       timetext->setText(tstr);
-      tm = timetext->getX() + timetext->getWidth() + window->getBorderWidth();
-      if ( tm > melw ) {
-        melw = tm;
+      tw = timetext->getX() + timetext->getWidth() + window->getBorderWidth();
+      if ( tw > melw ) {
+        melw = tw;
         doresize = true;
       }
     }
@@ -1156,9 +1085,9 @@ CopyOpWin::redraw()
     if(slim==true) sprintf(tstr,"%ld",copied_files);
     else sprintf(tstr,"%ld",files-copied_files);
     files2gotext->setText(tstr);
-    tm = files2gotext->getX() + files2gotext->getWidth() + window->getBorderWidth();
-    if ( tm > melw ) {
-      melw = tm;
+    tw = files2gotext->getX() + files2gotext->getWidth() + window->getBorderWidth();
+    if ( tw > melw ) {
+      melw = tw;
       doresize = true;
     }
     
@@ -1168,9 +1097,9 @@ CopyOpWin::redraw()
     if(slim==true) sprintf(tstr,"%ld",copied_dirs);
     else sprintf(tstr,"%ld",dirs-copied_dirs);
     dirs2gotext->setText(tstr);
-    tm = dirs2gotext->getX() + dirs2gotext->getWidth() + window->getBorderWidth();
-    if ( tm > melw ) {
-      melw = tm;
+    tw = dirs2gotext->getX() + dirs2gotext->getWidth() + window->getBorderWidth();
+    if ( tw > melw ) {
+      melw = tw;
       doresize = true;
     }
     
@@ -1185,11 +1114,13 @@ CopyOpWin::redraw()
     window->setMinSize( melw, bb4->getY() + bb4->getHeight() + 5 +cb->getHeight() + window->getBorderWidth() );
   }
 
+  //window->redraw();
+  aguix->Flush();
   /* update complete
      now check for X-messages */
   AGMessage *msg;
   do {
-    msg=aguix->GetMessage(NULL);
+    msg=aguix->GetMessage( window );
     if(msg!=NULL) {
       switch(msg->type) {
         case AG_CLOSEWINDOW:
@@ -1199,43 +1130,64 @@ CopyOpWin::redraw()
           if(msg->button.button==cb) returnvalue=1;
           break;
         case AG_SIZECHANGED:
-          bb1->resize( msg->size.neww - 2 * window->getBorderWidth(), bb1->getHeight() );
-          sbw = bb1->getWidth() - 2;
-          bb2->resize( msg->size.neww - 2 * window->getBorderWidth(), bb2->getHeight() );
-          if ( slim == false ) {
-            bb3->resize( msg->size.neww - 2 * window->getBorderWidth(), bb3->getHeight() );
+          if ( msg->size.window == window->getWindow() ) {
+            bb1->resize( msg->size.neww - 2 * window->getBorderWidth(), bb1->getHeight() );
+            sbw = bb1->getWidth() - 2;
+            bb2->resize( msg->size.neww - 2 * window->getBorderWidth(), bb2->getHeight() );
+            if ( slim == false ) {
+              bb3->resize( msg->size.neww - 2 * window->getBorderWidth(), bb3->getHeight() );
+            }
+            bb4->resize( msg->size.neww - 2 * window->getBorderWidth(), bb4->getHeight() );
+            cb->move( msg->size.neww / 2 - cb->getWidth() / 2,
+                      msg->size.newh - window->getBorderWidth() - cb->getHeight() );
+            
+            filenamespace[0] = ( msg->size.neww - window->getBorderWidth() - sourcetext->getX() ) / aguix->getCharWidth();
+            filenamespace[0] -= strlen(catalog.getLocale(116)) - 2;
+            filenamespace[1] = ( msg->size.neww - window->getBorderWidth() - desttext->getX() ) / aguix->getCharWidth();
+            filenamespace[1] -= strlen(catalog.getLocale(117)) - 2;
+            update_sourcetext = true;
+            update_desttext = true;
+  
+            // now gsb and fsb are not scaled for new width
+            // but next call of redraw they will so this shouldn't hurt anybody
+            // but a solution would be to first react for messages and then redraw
           }
-          bb4->resize( msg->size.neww - 2 * window->getBorderWidth(), bb4->getHeight() );
-          cb->move( msg->size.neww / 2 - cb->getWidth() / 2, msg->size.newh - window->getBorderWidth() - cb->getHeight() );
-          
-          filenamespace[0] = ( msg->size.neww - window->getBorderWidth() - sourcetext->getX() ) / aguix->getCharWidth();
-          filenamespace[0] -= strlen(catalog.getLocale(116)) - 2;
-          filenamespace[1] = ( msg->size.neww - window->getBorderWidth() - desttext->getX() ) / aguix->getCharWidth();
-          filenamespace[1] -= strlen(catalog.getLocale(117)) - 2;
-
-          // now gsb and fsb are not scaled for new width
-          // but next call of redraw they will so this shouldn't hurt anybody
-          // but a solution would be to first react for messages and then redraw
-          
           break;
       }
       aguix->ReplyMessage(msg);
     }
   } while(msg!=NULL);
+  
+  if ( returnvalue == 1 ) {
+    std::string str1;
+    int erg;
+    
+    // user canceled -> request to be sure
+    str1 = catalog.getLocale( 11 );
+    str1 += "|";
+    str1 += catalog.getLocale( 544 );
+    erg = window->request( catalog.getLocale( 123 ),
+                           catalog.getLocale( 543 ),
+                           str1.c_str() );
+    if ( erg != 0 ) returnvalue = 0;
+  }
+  
   return returnvalue;
 }
 
 void
-CopyOpWin::setmessage(char *msg,int line)
+CopyOpWin::setmessage( const char *msg, int line )
 {
   switch(line) {
     case 0:
       if(msg!=NULL) sourcetext->setText(msg);
       else sourcetext->setText("");
+      ismessage[0] = true;
       break;
     case 1:
       if(msg!=NULL) desttext->setText(msg);
       else desttext->setText("");
+      ismessage[1] = true;
       break;
   }
 }
@@ -1270,14 +1222,70 @@ CopyOpWin::dec_byte_counter(loff_t b)
 void
 CopyOpWin::stoptimer()
 {
-  stoptime=time(NULL);  
+  gettimeofday( &stoptime, NULL );
 }
 
 void
 CopyOpWin::conttimer()
 {
-  double difft=difftime(time(NULL),stoptime);
-  starttime+=(time_t)difft;
-  filestarttime+=(time_t)difft;
+  struct timeval curtime;
+  long s, us;
+
+  gettimeofday( &curtime, NULL );
+  
+  s = curtime.tv_sec - stoptime.tv_sec;
+  us = curtime.tv_usec - stoptime.tv_usec;
+  if ( us < 0 ) {
+    s--;
+    us += 1000000;
+  }
+
+  starttime.tv_sec += s;
+  starttime.tv_usec += us;
+  if ( starttime.tv_usec > 1000000 ) {
+    starttime.tv_sec++;
+    starttime.tv_usec -= 1000000;
+  }
+  filestarttime.tv_sec += s;
+  filestarttime.tv_usec += us;
+  if ( filestarttime.tv_usec > 1000000 ) {
+    filestarttime.tv_sec++;
+    filestarttime.tv_usec -= 1000000;
+  }
+}
+
+int CopyOpWin::request( const char *title,
+                        const char *text,
+                        const char *buttons,
+                        Requester::request_flags_t flags )
+{
+  if ( window != NULL ) {
+    return window->request( title, text, buttons, flags );
+  } else {
+    return Worker::getRequester()->request( title, text, buttons, flags );
+  }
+}
+
+int CopyOpWin::string_request( const char *title,
+                               const char *lines,
+                               const char *default_str,
+                               const char *buttons,
+                               char **return_str,
+                               Requester::request_flags_t flags )
+{
+  if ( window != NULL ) {
+    return window->string_request( title, lines, default_str, buttons, return_str, flags );
+  } else {
+    return Worker::getRequester()->string_request( title, lines, default_str, buttons, return_str, flags );
+  }
+}
+
+void CopyOpWin::file_skipped( loff_t byteSum, bool subCopied )
+{
+  if ( files > 0 ) files--;
+  if ( byteSum > 0 ) bytes -= byteSum;
+  if ( subCopied == true ) copied_bytes -= copied_bytes_curfile;
+  update_globtext = true;
+  update_files2gotext = true;
 }
 
