@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: createsymlinkop.h,v 1.6 2001/11/03 17:08:01 ralf Exp $ */
+/* $Id: createsymlinkop.h,v 1.7 2003/05/04 00:43:51 ralf Exp $ */
 
 #ifndef CREATESYMLINKOP_H
 #define CREATESYMLINKOP_H
@@ -35,14 +35,13 @@
 class CreateSymlinkOp:public FunctionProto
 {
 public:
-  CreateSymlinkOp(class Worker*);
+  CreateSymlinkOp();
   virtual ~CreateSymlinkOp();
-  virtual CreateSymlinkOp *duplicate();
+  virtual CreateSymlinkOp *duplicate() const;
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
   virtual bool save(Datei*);
-  virtual int load(Datei*);
   virtual const char *getDescription();
   virtual int configure();
   
