@@ -18,12 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: cursorops.cc,v 1.12 2001/11/03 17:08:05 ralf Exp $ */
+/* $Id: cursorops.cc,v 1.18 2003/06/04 23:26:17 ralf Exp $ */
 
 #include "cursorops.h"
 #include "listermode.h"
 #include "normalmode.h"
 #include "worker.h"
+#include "wpucontext.h"
+#include "ownop.h"
 
 /******
  RowUp
@@ -41,7 +43,7 @@ const char *RowUp::getName()
   return name;
 }
 
-RowUp::RowUp(Worker *tworker):FunctionProto(tworker)
+RowUp::RowUp() : FunctionProto()
 {
 }
 
@@ -49,9 +51,9 @@ RowUp::~RowUp()
 {
 }
 
-RowUp *RowUp::duplicate()
+RowUp *RowUp::duplicate() const
 {
-  RowUp *ta=new RowUp(worker);
+  RowUp *ta=new RowUp();
   return ta;
 }
 
@@ -61,7 +63,7 @@ int RowUp::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -100,7 +102,7 @@ const char *RowDown::getName()
   return name;
 }
 
-RowDown::RowDown(Worker *tworker):FunctionProto(tworker)
+RowDown::RowDown() : FunctionProto()
 {
 }
 
@@ -108,9 +110,9 @@ RowDown::~RowDown()
 {
 }
 
-RowDown *RowDown::duplicate()
+RowDown *RowDown::duplicate() const
 {
-  RowDown *ta=new RowDown(worker);
+  RowDown *ta=new RowDown();
   return ta;
 }
 
@@ -120,7 +122,7 @@ int RowDown::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -159,7 +161,7 @@ const char *FirstRow::getName()
   return name;
 }
 
-FirstRow::FirstRow(Worker *tworker):FunctionProto(tworker)
+FirstRow::FirstRow() : FunctionProto()
 {
 }
 
@@ -167,9 +169,9 @@ FirstRow::~FirstRow()
 {
 }
 
-FirstRow *FirstRow::duplicate()
+FirstRow *FirstRow::duplicate() const
 {
-  FirstRow *ta=new FirstRow(worker);
+  FirstRow *ta=new FirstRow();
   return ta;
 }
 
@@ -179,7 +181,7 @@ int FirstRow::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -218,7 +220,7 @@ const char *LastRow::getName()
   return name;
 }
 
-LastRow::LastRow(Worker *tworker):FunctionProto(tworker)
+LastRow::LastRow() : FunctionProto()
 {
 }
 
@@ -226,9 +228,9 @@ LastRow::~LastRow()
 {
 }
 
-LastRow *LastRow::duplicate()
+LastRow *LastRow::duplicate() const
 {
-  LastRow *ta=new LastRow(worker);
+  LastRow *ta=new LastRow();
   return ta;
 }
 
@@ -238,7 +240,7 @@ int LastRow::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -277,7 +279,7 @@ const char *PageUp::getName()
   return name;
 }
 
-PageUp::PageUp(Worker *tworker):FunctionProto(tworker)
+PageUp::PageUp() : FunctionProto()
 {
 }
 
@@ -285,9 +287,9 @@ PageUp::~PageUp()
 {
 }
 
-PageUp *PageUp::duplicate()
+PageUp *PageUp::duplicate() const
 {
-  PageUp *ta=new PageUp(worker);
+  PageUp *ta=new PageUp();
   return ta;
 }
 
@@ -297,7 +299,7 @@ int PageUp::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -336,7 +338,7 @@ const char *PageDown::getName()
   return name;
 }
 
-PageDown::PageDown(Worker *tworker):FunctionProto(tworker)
+PageDown::PageDown() : FunctionProto()
 {
 }
 
@@ -344,9 +346,9 @@ PageDown::~PageDown()
 {
 }
 
-PageDown *PageDown::duplicate()
+PageDown *PageDown::duplicate() const
 {
-  PageDown *ta=new PageDown(worker);
+  PageDown *ta=new PageDown();
   return ta;
 }
 
@@ -356,7 +358,7 @@ int PageDown::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -395,7 +397,7 @@ const char *SelectOp::getName()
   return name;
 }
 
-SelectOp::SelectOp(Worker *tworker):FunctionProto(tworker)
+SelectOp::SelectOp() : FunctionProto()
 {
 }
 
@@ -403,9 +405,9 @@ SelectOp::~SelectOp()
 {
 }
 
-SelectOp *SelectOp::duplicate()
+SelectOp *SelectOp::duplicate() const
 {
-  SelectOp *ta=new SelectOp(worker);
+  SelectOp *ta=new SelectOp();
   return ta;
 }
 
@@ -415,7 +417,7 @@ int SelectOp::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -454,7 +456,7 @@ const char *SelectAllOp::getName()
   return name;
 }
 
-SelectAllOp::SelectAllOp(Worker *tworker):FunctionProto(tworker)
+SelectAllOp::SelectAllOp() : FunctionProto()
 {
 }
 
@@ -462,9 +464,9 @@ SelectAllOp::~SelectAllOp()
 {
 }
 
-SelectAllOp *SelectAllOp::duplicate()
+SelectAllOp *SelectAllOp::duplicate() const
 {
-  SelectAllOp *ta=new SelectAllOp(worker);
+  SelectAllOp *ta=new SelectAllOp();
   return ta;
 }
 
@@ -474,7 +476,7 @@ int SelectAllOp::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -513,7 +515,7 @@ const char *SelectNoneOp::getName()
   return name;
 }
 
-SelectNoneOp::SelectNoneOp(Worker *tworker):FunctionProto(tworker)
+SelectNoneOp::SelectNoneOp() : FunctionProto()
 {
 }
 
@@ -521,9 +523,9 @@ SelectNoneOp::~SelectNoneOp()
 {
 }
 
-SelectNoneOp *SelectNoneOp::duplicate()
+SelectNoneOp *SelectNoneOp::duplicate() const
 {
-  SelectNoneOp *ta=new SelectNoneOp(worker);
+  SelectNoneOp *ta=new SelectNoneOp();
   return ta;
 }
 
@@ -533,7 +535,7 @@ int SelectNoneOp::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -572,7 +574,7 @@ const char *InvertAllOp::getName()
   return name;
 }
 
-InvertAllOp::InvertAllOp(Worker *tworker):FunctionProto(tworker)
+InvertAllOp::InvertAllOp() : FunctionProto()
 {
 }
 
@@ -580,9 +582,9 @@ InvertAllOp::~InvertAllOp()
 {
 }
 
-InvertAllOp *InvertAllOp::duplicate()
+InvertAllOp *InvertAllOp::duplicate() const
 {
-  InvertAllOp *ta=new InvertAllOp(worker);
+  InvertAllOp *ta=new InvertAllOp();
   return ta;
 }
 
@@ -592,7 +594,7 @@ int InvertAllOp::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -631,7 +633,7 @@ const char *ParentDirOp::getName()
   return name;
 }
 
-ParentDirOp::ParentDirOp(Worker *tworker):FunctionProto(tworker)
+ParentDirOp::ParentDirOp() : FunctionProto()
 {
 }
 
@@ -639,9 +641,9 @@ ParentDirOp::~ParentDirOp()
 {
 }
 
-ParentDirOp *ParentDirOp::duplicate()
+ParentDirOp *ParentDirOp::duplicate() const
 {
-  ParentDirOp *ta=new ParentDirOp(worker);
+  ParentDirOp *ta=new ParentDirOp();
   return ta;
 }
 
@@ -651,7 +653,7 @@ int ParentDirOp::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -695,11 +697,12 @@ const char *EnterDirOp::getName()
   return name;
 }
 
-EnterDirOp::EnterDirOp(Worker *tworker):FunctionProto(tworker)
+EnterDirOp::EnterDirOp() : FunctionProto()
 {
   dir=(char*)_allocsafe(1);
   dir[0]=0;
   enterdirmode=ENTERDIROP_ACTIVE;
+  hasConfigure = true;
 }
 
 EnterDirOp::~EnterDirOp()
@@ -707,22 +710,23 @@ EnterDirOp::~EnterDirOp()
   _freesafe(dir);
 }
 
-EnterDirOp *EnterDirOp::duplicate()
+EnterDirOp *EnterDirOp::duplicate() const
 {
-  EnterDirOp *ta=new EnterDirOp(worker);
+  EnterDirOp *ta=new EnterDirOp();
   _freesafe(ta->dir);
   ta->dir=dupstring(dir);
   ta->enterdirmode=enterdirmode;
   return ta;
 }
 
-int EnterDirOp::run(ActionMessage *msg)
+int EnterDirOp::run( WPUContext *wpu, ActionMessage *msg )
 {
   Lister *l1;
   ListerMode *lm1;
   NormalMode *nm1;
   int erg;
-  char *buttonstr,*textstr,*return_str;
+  char *buttonstr,*return_str, *tstr;
+  const char *textstr;
 
   if(msg->mode==msg->AM_MODE_DNDACTION) {
     l1=((DNDMsg*)msg->extraInfo)->getDestLister();
@@ -741,20 +745,25 @@ int EnterDirOp::run(ActionMessage *msg)
   } else {
     if(enterdirmode==ENTERDIROP_SPECIAL) {
       if(strlen(dir)>0) {
-        l1=worker->getActiveLister();
+        l1 = msg->getWorker()->getActiveLister();
         if(l1!=NULL) {
           l1->switch2Mode(0);
           lm1=l1->getActiveMode();
           if(lm1!=NULL) {
             if(lm1->isType("NormalMode")==true) {
               nm1=(NormalMode*)lm1;
-              nm1->enterDir(dir);
+              
+              tstr = wpu->parse( dir, a_max( EXE_STRING_LEN - 1024, 256 ), false );
+              if ( tstr != NULL ) {
+                nm1->enterDir( tstr );
+                _freesafe( tstr );
+              }
             }
           }
         }
       }
     } else if(enterdirmode==ENTERDIROP_REQUEST) {
-      Requester *req=new Requester(worker->getAGUIX());
+      Requester *req=new Requester( msg->getWorker()->getAGUIX());
       textstr=catalog.getLocale(290);
       buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(11))+1+
                                   strlen(catalog.getLocale(8))+1);
@@ -763,7 +772,7 @@ int EnterDirOp::run(ActionMessage *msg)
       erg=req->string_request(catalog.getLocale(123),textstr,"",buttonstr,&return_str);
       _freesafe(buttonstr);
       if((erg==0)&&(strlen(return_str)>0)) {
-        l1=worker->getActiveLister();
+        l1 = msg->getWorker()->getActiveLister();
         if(l1!=NULL) {
           l1->switch2Mode(0);
           lm1=l1->getActiveMode();
@@ -779,7 +788,7 @@ int EnterDirOp::run(ActionMessage *msg)
       delete req;
     } else {
       // here we need the help of NormalMode
-      l1=worker->getActiveLister();
+      l1 = msg->getWorker()->getActiveLister();
       if(l1!=NULL) {
         lm1=l1->getActiveMode();
         if(lm1!=NULL) {
@@ -805,7 +814,7 @@ const char *EnterDirOp::getDescription()
 
 int EnterDirOp::configure()
 {
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   Button *okb,*cb;
   AWindow *win;
   CycleButton *rcyb;
@@ -856,10 +865,20 @@ int EnterDirOp::configure()
   ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(322),1));
   x+=ttext->getWidth()+5;
   tw=w-x-5;
-  if(tw<50) tw=50;
+  if ( tw < ( 30 * aguix->getCharWidth() ) ) tw = 30 * aguix->getCharWidth();
   sg=(StringGadget*)win->add(new StringGadget(aguix,x,y,tw,dir,0));
+  x += sg->getWidth();
+  Button *sgb = (Button*)win->add( new Button( aguix,
+                                               x,
+                                               y,
+                                               "F",
+                                               1,
+                                               0,
+                                               0 ) );
+  sgb->resize( sgb->getWidth(), sg->getHeight() );
+
   y+=sg->getHeight()+5;
-  tw=sg->getWidth()+5+x;
+  tw = sgb->getX() + sgb->getWidth() + 5;
   if(tw>w) w=tw;
   x=5;
 
@@ -886,8 +905,10 @@ int EnterDirOp::configure()
                                   0,
                                   0));
   y+=okb->getHeight()+5;
+  rcyb->resize( sgb->getX() + sgb->getWidth() - rcyb->getX(), rcyb->getHeight() );
   
   h=y;
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMaxSize(w,h);
   win->setMinSize(w,h);
@@ -902,6 +923,17 @@ int EnterDirOp::configure()
         case AG_BUTTONCLICKED:
           if(msg->button.button==okb) endmode=0;
           else if(msg->button.button==cb) endmode=1;
+          else if ( msg->button.button == sgb ) {
+            OwnOp *o;
+            
+            o = new OwnOp();
+            tstr = o->getFlag();
+            delete o;
+            if ( tstr != NULL ) {
+              sg->insertAtCursor( tstr );
+              _freesafe( tstr );
+            }
+          }
           break;
         case AG_STRINGGADGET_DEACTIVATE:
           rcyb->setOption(2);
@@ -939,62 +971,24 @@ int EnterDirOp::configure()
 
 bool EnterDirOp::save(Datei *fh)
 {
-  fh->putInt(Datei::getIntSize()+strlen(dir)+Datei::getUCharSize());
-  fh->putInt(strlen(dir));
-  fh->putString(dir);
-  switch(enterdirmode) {
+  if ( fh == NULL ) return false;
+  WConfig::configPutPairString( fh, "dir", dir );
+  switch ( enterdirmode ) {
     case ENTERDIROP_ACTIVE2OTHER:
-      fh->putUChar(1);
+      WConfig::configPutPair( fh, "mode", "active2other" );
       break;
     case ENTERDIROP_SPECIAL:
-      fh->putUChar(2);
+      WConfig::configPutPair( fh, "mode", "special" );
       break;
     case ENTERDIROP_REQUEST:
-      fh->putUChar(3);
+      WConfig::configPutPair( fh, "mode", "request" );
       break;
     default:
-      fh->putUChar(0);
+      WConfig::configPutPair( fh, "mode", "active" );
   }
   return true;
 }
 
-int EnterDirOp::load(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  int len;
-  char *str;
-  unsigned char c1;
-  if(chunksize>=(Datei::getIntSize()+Datei::getUCharSize())) {
-    len=fh->getInt();
-    chunksize-=Datei::getIntSize();
-    str=fh->getString(len);
-    _freesafe(dir);
-    dir=str;
-    chunksize-=len;
-    c1=fh->getUChar();
-    switch(c1) {
-      case 1:
-        enterdirmode=ENTERDIROP_ACTIVE2OTHER;
-        break;
-      case 2:
-        enterdirmode=ENTERDIROP_SPECIAL;
-        break;
-      case 3:
-        enterdirmode=ENTERDIROP_REQUEST;
-        break;
-      default:
-        enterdirmode=ENTERDIROP_ACTIVE;
-        break;
-    }
-    chunksize-=Datei::getUCharSize();
-  }
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
-  }
-  return 0;
-}
-
 void EnterDirOp::setMode(enterdir_t nv)
 {
   enterdirmode=nv;
