@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: dirsizeop.cc,v 1.6 2002/03/16 23:28:08 ralf Exp $ */
+/* $Id: dirsizeop.cc,v 1.8 2003/05/04 00:52:58 ralf Exp $ */
 
 #include "dirsizeop.h"
 #include "listermode.h"
@@ -27,7 +27,7 @@
 
 const char *DirSizeOp::name="DirSizeOp";
 
-DirSizeOp::DirSizeOp(Worker *tworker):FunctionProto(tworker)
+DirSizeOp::DirSizeOp() : FunctionProto()
 {
 }
 
@@ -36,9 +36,9 @@ DirSizeOp::~DirSizeOp()
 }
 
 DirSizeOp*
-DirSizeOp::duplicate()
+DirSizeOp::duplicate() const
 {
-  DirSizeOp *ta=new DirSizeOp(worker);
+  DirSizeOp *ta=new DirSizeOp();
   return ta;
 }
 
@@ -60,7 +60,7 @@ DirSizeOp::run(ActionMessage *msg)
   ListerMode *lm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       startlister=l1;
       lm1=startlister->getActiveMode();
@@ -92,6 +92,7 @@ DirSizeOp::normalmodedirsize()
   if(lm1->isType("NormalMode")!=true) return 1;
   nm1=(NormalMode*)lm1;
   
+  memset( &dsorder, 0, sizeof( dsorder ) );
   if(am->mode==am->AM_MODE_ONLYACTIVE)
     dsorder.source=dsorder.NM_ONLYACTIVE;
   else if(am->mode==am->AM_MODE_DNDACTION) {
