@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: enterpathop.cc,v 1.8 2001/11/03 17:10:02 ralf Exp $ */
+/* $Id: enterpathop.cc,v 1.11 2003/05/04 00:54:06 ralf Exp $ */
 
 #include "enterpathop.h"
 #include "listermode.h"
@@ -27,9 +27,10 @@
 
 const char *EnterPathOp::name="EnterPathOp";
 
-EnterPathOp::EnterPathOp(Worker *tworker):FunctionProto(tworker)
+EnterPathOp::EnterPathOp() : FunctionProto()
 {
   enterpathside=ENTERPATHOP_THISSIDE;
+  hasConfigure = true;
 }
 
 EnterPathOp::~EnterPathOp()
@@ -37,9 +38,9 @@ EnterPathOp::~EnterPathOp()
 }
 
 EnterPathOp*
-EnterPathOp::duplicate()
+EnterPathOp::duplicate() const
 {
-  EnterPathOp *ta=new EnterPathOp(worker);
+  EnterPathOp *ta=new EnterPathOp();
   ta->enterpathside=enterpathside;
   return ta;
 }
@@ -62,7 +63,7 @@ EnterPathOp::run(ActionMessage *msg)
   ListerMode *lm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       startlister=l1;
       lm1=startlister->getActiveMode();
@@ -77,52 +78,20 @@ EnterPathOp::run(ActionMessage *msg)
 bool
 EnterPathOp::save(Datei *fh)
 {
-  int size=Datei::getUCharSize();
-  fh->putInt(size);
-
+  if ( fh == NULL ) return false;
   switch(enterpathside) {
     case ENTERPATHOP_OTHERSIDE:
-      fh->putUChar(1);
+      WConfig::configPutPair( fh, "mode", "other" );
       break;
     case ENTERPATHOP_LEFTSIDE:
-      fh->putUChar(2);
+      WConfig::configPutPair( fh, "mode", "left" );
       break;
     case ENTERPATHOP_RIGHTSIDE:
-      fh->putUChar(3);
+      WConfig::configPutPair( fh, "mode", "right" );
       break;
     default:
-      fh->putUChar(0);
-  }
-  return true;
-}
-
-int
-EnterPathOp::load(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  int cs=Datei::getUCharSize();
-  unsigned char c1;
-  if(chunksize>=cs) {
-    c1=fh->getUChar();
-    switch(c1) {
-      case 1:
-        enterpathside=ENTERPATHOP_OTHERSIDE;
-        break;
-      case 2:
-        enterpathside=ENTERPATHOP_LEFTSIDE;
-        break;
-      case 3:
-        enterpathside=ENTERPATHOP_RIGHTSIDE;
-        break;
-      default:
-        enterpathside=ENTERPATHOP_THISSIDE;
-        break;
-    }
-    chunksize-=cs;
-  }
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
+      WConfig::configPutPair( fh, "mode", "current" );
+      break;
   }
   return true;
 }
@@ -140,8 +109,8 @@ EnterPathOp::normalmodeenterpath()
   ListerMode *lm1=NULL;
   Lister *ll,*lr,*tl;
   
-  ll=worker->getLister(0);
-  lr=worker->getLister(1);
+  ll = am->getWorker()->getLister(0);
+  lr = am->getWorker()->getLister(1);
   
   tl=NULL;
   switch(enterpathside) {
@@ -173,7 +142,7 @@ EnterPathOp::normalmodeenterpath()
 int
 EnterPathOp::configure()
 {
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   Button *okb,*cb;
   AWindow *win;
   CycleButton *rcyb;
@@ -245,6 +214,7 @@ EnterPathOp::configure()
   y+=okb->getHeight()+5;
   
   h=y;
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMaxSize(w,h);
   win->setMinSize(w,h);
