@@ -18,17 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: filereq.cc,v 1.7 2002/02/06 19:16:47 ralf Exp $ */
+/* $Id: filereq.cc,v 1.16 2003/12/09 22:37:43 ralf Exp $ */
 
 #include "filereq.h"
-#include "locale.h"
+#include "worker_locale.h"
 #include <aguix/aguix.h>
 #include <aguix/awindow.h>
 #include <aguix/button.h>
 #include <aguix/text.h>
 #include <aguix/message.h>
 #include <aguix/util.h>
-#include <aguix/listview.h>
+#include <aguix/fieldlistview.h>
 #include <aguix/stringgadget.h>
 #include <aguix/request.h>
 
@@ -48,11 +48,11 @@ FileRequester::~FileRequester()
   if(verz!=NULL) delete verz;
 }
 
-int FileRequester::request(char *title,
-                           char *dir,
-                           char *oktext,
-                           char *canceltext,
-                           char *infotext)
+int FileRequester::request( const char *title,
+                            const char *dir,
+                            const char *oktext,
+                            const char *canceltext,
+                            const char *infotext)
 {
   if((title==NULL)||(dir==NULL)||(oktext==NULL)||(canceltext==NULL)||(infotext==NULL)) return -1;
   if((actdir==NULL)&&(strlen(dir)==0)) return -1;
@@ -75,7 +75,7 @@ int FileRequester::request(char *title,
   ty=5;
   win->add(new Text(aguix,5,ty,infotext,1));
   ty+=aguix->getCharHeight()+5;
-  lv=(ListView*)win->add(new ListView(aguix,5,ty,w-10,150,0));
+  lv = (FieldListView*)win->add( new FieldListView( aguix, 5, ty, w - 10, 150, 0 ) );
   lv->setHBarState(2);
   lv->setVBarState(2);
   ty+=lv->getHeight();
@@ -91,6 +91,7 @@ int FileRequester::request(char *title,
 //  dh-=h-cnb->getY()+5;
   ty+=okb->getHeight()+5;
   h=ty;
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMinSize(w,h);
   win->show();
@@ -102,7 +103,7 @@ int FileRequester::request(char *title,
     setDir(actdir);
   }
   int endmode=-1;
-  ListViewContent *lastlvc=NULL;
+  int lastrow = -1;
   Time lasttime=0;
   for(;endmode==-1;) {
     msg=aguix->WaitMessage(win);
@@ -129,46 +130,43 @@ int FileRequester::request(char *title,
             setDir(sg->getText());
           }
           break;
-        case AG_LISTVIEW_ONESELECT:
-          if(msg->listview.lv==lv) {
-            if(lastlvc!=NULL) {
-              if(msg->listview.lvc==lastlvc) {
+        case AG_FIELDLV_ONESELECT:
+          if ( msg->fieldlv.lv == lv ) {
+            if ( lv->isValidRow( lastrow ) == true  ) {
+              if ( msg->fieldlv.row == lastrow ) {
                 // eventl. Doppelklick
-                if((msg->listview.time-lasttime)<350) {
-                  int x=lv->getPos(lastlvc);
-                  if(x>=0) {
-                    ArrayList *files=verz->getFiles();
-                    FileEntry *fe=(FileEntry*)files->getElementAt(x);
-                    if(fe!=NULL) {
-                      if(fe->isDir()==true) {
-                        char *tstr=(char*)_allocsafe(strlen(actdir)+1+strlen(fe->name)+2);
-                        sprintf(tstr,"%s/%s",actdir,fe->name);
-                        setDir(tstr);
-                        _freesafe(tstr);
-                      } else {
-                        endmode=2;
-                      }
+                if ( aguix->isDoubleClick( msg->fieldlv.time, lasttime ) == true ) {
+                  ArrayList *files = verz->getFiles();
+                  FileEntry *fe = (FileEntry*)files->getElementAt( lastrow );
+                  if ( fe != NULL ) {
+                    if ( fe->isDir() == true ) {
+                      char *tstr = (char*)_allocsafe( strlen( actdir ) + 1 + strlen( fe->name ) + 2 );
+                      sprintf( tstr, "%s/%s", actdir, fe->name );
+                      setDir( tstr );
+                      _freesafe( tstr );
+                    } else {
+                      endmode = 2;
                     }
                   }
                 }
-                if(endmode==-1) {
-                  lastlvc=NULL;
-                  lasttime=0;
+                if ( endmode == -1 ) {
+                  lastrow = -1;
+                  lasttime = 0;
                 }
               } else {
-                lastlvc=msg->listview.lvc;
-                lasttime=msg->listview.time;
+                lastrow = msg->fieldlv.row;
+                lasttime = msg->fieldlv.time;
               }
             } else {
-              lastlvc=msg->listview.lvc;
-              lasttime=msg->listview.time;
+              lastrow = msg->fieldlv.row;
+              lasttime = msg->fieldlv.time;
             }
           }
           break;
-        case AG_LISTVIEW_MULTISELECT:
-          if(msg->listview.lv==lv) {
-            lastlvc=NULL;
-            lasttime=0;
+        case AG_FIELDLV_MULTISELECT:
+          if ( msg->fieldlv.lv == lv ) {
+            lastrow = -1;
+            lasttime = 0;
           }
           break;
       }
@@ -180,26 +178,26 @@ int FileRequester::request(char *title,
     selfes=new List();
     ArrayList *files=verz->getFiles();
     FileEntry *fe=(FileEntry*)files->getElementAt(1);
-    ListViewContent *lvc=(ListViewContent*)lv->get(1);
-    while(fe!=NULL) {
-      if(lvc->getSelect()==true) {
-        selfes->addElement(fe->duplicate());
+    int row = 1;
+    while ( fe != NULL ) {
+      if ( lv->getSelect( row ) == true ) {
+        selfes->addElement( fe->duplicate() );
       }
-      fe=(FileEntry*)files->getNextElement();
-      lvc=(ListViewContent*)lv->getNext();
+      fe = (FileEntry*)files->getNextElement();
+      row++;
     }
   } else if(endmode==2) {
     selfes=new List();
     ArrayList *files=verz->getFiles();
     FileEntry *fe=(FileEntry*)files->getElementAt(1);
-    ListViewContent *lvc=(ListViewContent*)lv->get(1);
-    while(fe!=NULL) {
-      if(lvc==lastlvc) {
-        selfes->addElement(fe->duplicate());
+    int row = 1;
+    while ( fe != NULL ) {
+      if ( row == lastrow ) {
+        selfes->addElement( fe->duplicate() );
         break;
       }
-      fe=(FileEntry*)files->getNextElement();
-      lvc=(ListViewContent*)lv->getNext();
+      fe = (FileEntry*)files->getNextElement();
+      row++;
     }
     endmode=1;
   }
@@ -234,6 +232,8 @@ void FileRequester::reset()
 void FileRequester::setDir(const char *dir)
 {
   char *tstr=HandlePath(dir);
+  int row;
+  
   verz->closeDir();
   if(verz->readDir(tstr)!=0) {
     if(actdir!=NULL) {
@@ -248,12 +248,21 @@ void FileRequester::setDir(const char *dir)
   verz->sort(0);
   ArrayList *files=verz->getFiles();
   FileEntry *fe=(FileEntry*)files->getFirstElement();
-  lv->removeAll();
+  lv->setSize( 0 );
   while(fe!=NULL) {
+    row = lv->addRow();
+    lv->setText( row, 0, fe->name );
     if(fe->isDir()==false) {
-      lv->add(new ListViewContent(fe->name,1,2,1,2,0,1,0,1,0));
+      lv->setPreColors( row, FieldListView::PRECOLOR_ONLYSELECT );
     } else {
-      lv->add(new ListViewContent(fe->name,3,2,3,2,0,3,0,3,0));
+      lv->setFG( row, FieldListView::CC_NORMAL, 3 );
+      lv->setFG( row, FieldListView::CC_SELECT, 2 );
+      lv->setFG( row, FieldListView::CC_ACTIVE, 3 );
+      lv->setFG( row, FieldListView::CC_SELACT, 2 );
+      lv->setBG( row, FieldListView::CC_NORMAL, 0 );
+      lv->setBG( row, FieldListView::CC_SELECT, 3 );
+      lv->setBG( row, FieldListView::CC_ACTIVE, 0 );
+      lv->setBG( row, FieldListView::CC_SELACT, 3 );
     }
     fe=(FileEntry*)files->getNextElement();
   }
@@ -276,17 +285,17 @@ FileEntry *FileRequester::getNextFE()
 #define FR_RE_TESTOKAY \
 { \
   bool okay = false; \
-  ListViewContent *tlvc2; \
+  int trow2; \
   if ( sg2 != NULL ) { \
     if ( strlen( sg2->getText() ) > 0 ) { \
       endmode = 1; \
       okay = true; \
     } \
   } else { \
-    tlvc2 = lv->getActiveLVC(); \
-    if ( tlvc2 != NULL ) { \
-      if ( tlvc2->getData() == 0 ) { \
-        if ( strlen( tlvc2->getText() ) > 0 ) { \
+    trow2 = lv->getActiveRow(); \
+    if ( lv->isValidRow( trow2 ) == true ) { \
+      if ( lv->getData( trow2 ) == 0 ) { \
+        if ( lv->getText( trow2, 0 ).length() > 0 ) { \
           endmode = 1; \
           okay = true; \
         } \
@@ -303,13 +312,13 @@ FileEntry *FileRequester::getNextFE()
 /*
  * Works like normal request but only allow to select one entry
  * when allowEnterName is true, the user can enter an own name
- * ( i.e. for file creation)
+ * ( e.g. for file creation)
  */
-int FileRequester::request_entry( char *title,
-                                  char *dir,
-                                  char *oktext,
-                                  char *canceltext,
-                                  char *infotext,
+int FileRequester::request_entry( const char *title,
+                                  const char *dir,
+                                  const char *oktext,
+                                  const char *canceltext,
+                                  const char *infotext,
                                   bool allowEnterName )
 {
   int w, h, ty, tw;
@@ -320,10 +329,9 @@ int FileRequester::request_entry( char *title,
   Button *b[2];
   Text *ttext;
   char *use_dir = NULL;
-  ListViewContent *lastlvc = NULL, *tlvc;
+  int lastrow = -1, trow;
   Time lasttime=0;
   Requester *req;
-  bool firstkey;
   
   if ( ( title == NULL ) ||
        ( oktext == NULL ) ||
@@ -360,7 +368,7 @@ int FileRequester::request_entry( char *title,
   sg = (StringGadget*)win->add( new StringGadget( aguix, 5, ty, tw, use_dir, 0 ) );
   ty += sg->getHeight();
 
-  lv = (ListView*)win->add( new ListView( aguix, 5, ty, 200, 250, 0 ) );
+  lv = (FieldListView*)win->add( new FieldListView( aguix, 5, ty, 200, 250, 0 ) );
   lv->setHBarState( 2 );
   lv->setVBarState( 2 );
   ty += lv->getHeight() + 5;
@@ -377,13 +385,15 @@ int FileRequester::request_entry( char *title,
   b[1] = (Button*)win->add( new Button( aguix, 0, ty, canceltext, 1, 0, 0 ) );
   ty += b[0]->getHeight() + 5;
   
-  tw = AGUIX::scaleElementsW( w, 5, 10, -1, false, false, (GUIElement**)b, 2 );
+  tw = AGUIX::scaleElementsW( w, 5, 10, -1, false, false, (GUIElement**)b, NULL, 2 );
   if ( tw > w ) w = tw;
   sg->resize( w - 10, sg->getHeight() );
   if ( sg2 != NULL ) sg2->resize( w - 10, sg->getHeight() );
   lv->resize( w - 10, lv->getHeight() );
 
   h = ty;
+  b[0]->takeFocus();
+  win->setDoTabCycling( true );
   win->resize( w, h );
   win->setMinSize( w, h );
   win->show();
@@ -394,8 +404,6 @@ int FileRequester::request_entry( char *title,
 
   req = new Requester( aguix );
 
-  firstkey = true;
-  
   for( ; endmode == -1; ) {
     msg = aguix->WaitMessage( win );
     if( msg != NULL ) {
@@ -409,7 +417,7 @@ int FileRequester::request_entry( char *title,
           } else if ( msg->button.button == b[1] ) endmode = 0;
           break;
         case AG_SIZECHANGED:
-          AGUIX::scaleElementsW( msg->size.neww, 5, 10, -1, false, false, (GUIElement**)b, 2 );
+          AGUIX::scaleElementsW( msg->size.neww, 5, 10, -1, false, false, (GUIElement**)b, NULL, 2 );
           sg->resize( msg->size.neww - 10, sg->getHeight() );
           if ( sg2 != NULL ) sg2->resize( msg->size.neww - 10, sg->getHeight() );
           
@@ -428,83 +436,78 @@ int FileRequester::request_entry( char *title,
             setDirOneSelect( sg->getText() );
             //if ( sg2 != NULL ) sg2->setText( "" );
           } else if ( msg->stringgadget.sg == sg2 ) {
-            tlvc = lv->getActiveLVC();
-            if ( tlvc != NULL ) {
-              tlvc->setActive( false );
+            trow = lv->getActiveRow();
+            if ( lv->isValidRow( trow ) == true ) {
+              lv->setActiveRow( -1 );
             }
           }
           break;
-        case AG_LISTVIEW_ONESELECT:
-          if ( msg->listview.lv == lv ) {
-            if ( lastlvc != NULL ) {
-              if ( msg->listview.lvc == lastlvc ) {
+        case AG_FIELDLV_ONESELECT:
+          if ( msg->fieldlv.lv == lv ) {
+            if ( lv->isValidRow( lastrow ) == true ) {
+              if ( msg->fieldlv.row == lastrow ) {
                 // eventl. Doppelklick
-                if ( aguix->isDoubleClick( msg->listview.time, lasttime ) == true ) {
-                  int x = lv->getPos( lastlvc );
-                  if ( x >= 0 ) {
-                    ArrayList *files = verz->getFiles();
-                    FileEntry *fe = (FileEntry*)files->getElementAt( x );
-                    if ( fe != NULL ) {
-                      if ( fe->isDir() == true ) {
-                        char *tstr = (char*)_allocsafe( strlen( actdir ) + 1 + strlen( fe->name ) + 2 );
-                        sprintf( tstr, "%s/%s", actdir, fe->name );
-                        setDirOneSelect( tstr );
-                        _freesafe( tstr );
-                        //if ( sg2 != NULL ) sg2->setText( "" );
-                        lastlvc = NULL;
-                        lasttime = 0;
-                      } else {
-                        endmode = 2;
-                      }
-                    } else {
-                      lastlvc = NULL;
+                if ( aguix->isDoubleClick( msg->fieldlv.time, lasttime ) == true ) {
+                  ArrayList *files = verz->getFiles();
+                  FileEntry *fe = (FileEntry*)files->getElementAt( lastrow );
+                  if ( fe != NULL ) {
+                    if ( fe->isDir() == true ) {
+                      char *tstr = (char*)_allocsafe( strlen( actdir ) + 1 + strlen( fe->name ) + 2 );
+                      sprintf( tstr, "%s/%s", actdir, fe->name );
+                      setDirOneSelect( tstr );
+                      _freesafe( tstr );
+                      //if ( sg2 != NULL ) sg2->setText( "" );
+                      lastrow = -1;
                       lasttime = 0;
+                    } else {
+                      endmode = 2;
                     }
                   } else {
-                    lastlvc = NULL;
+                    lastrow = -1;
                     lasttime = 0;
                   }
                 } else {
-                  lastlvc = msg->listview.lvc;
-                  lasttime = msg->listview.time;
+                  lastrow = msg->fieldlv.row;
+                  lasttime = msg->fieldlv.time;
                 }
               } else {
-                lastlvc = msg->listview.lvc;
-                lasttime = msg->listview.time;
+                lastrow = msg->fieldlv.row;
+                lasttime = msg->fieldlv.time;
               }
             } else {
-              lastlvc = msg->listview.lvc;
-              lasttime = msg->listview.time;
+              lastrow = msg->fieldlv.row;
+              lasttime = msg->fieldlv.time;
             }
-            if ( lastlvc != NULL ) {
-              if ( lastlvc->getData() == 0 ) {
+            if ( lv->isValidRow( lastrow ) == true ) {
+              if ( lv->getData( lastrow ) == 0 ) {
                 // change sg2-text only for selected files
-                if ( sg2 != NULL ) sg2->setText( lastlvc->getText() );
+                if ( sg2 != NULL ) sg2->setText( lv->getText( lastrow, 0 ).c_str() );
               }
             }
           }
           break;
-        case AG_LISTVIEW_MULTISELECT:
-          if ( msg->listview.lv == lv ) {
-            lastlvc = NULL;
+        case AG_FIELDLV_MULTISELECT:
+          if ( msg->fieldlv.lv == lv ) {
+            lastrow = -1;
             lasttime = 0;
             
-            tlvc = lv->getActiveLVC();
-            if ( tlvc != NULL ) {
-              if ( tlvc->getData() == 0 ) {
+            trow = lv->getActiveRow();
+            if ( lv->isValidRow( trow ) == true ) {
+              if ( lv->getData( trow ) == 0 ) {
                 // change sg2-text only for selected files
-                if ( sg2 != NULL ) sg2->setText( tlvc->getText() );
+                if ( sg2 != NULL ) sg2->setText( lv->getText( trow, 0 ).c_str() );
               }
             }
           }
           break;
-        case AG_KEYRELEASED:
-          if ( ( win->isParent( msg->key.window, false ) == true ) &&
-               ( firstkey == false ) ) {
+        case AG_KEYPRESSED:
+          if ( win->isParent( msg->key.window, false ) == true ) {
             switch ( msg->key.key ) {
               case XK_KP_Enter:
               case XK_Return:
-                FR_RE_TESTOKAY;
+                if ( b[1]->hasFocus() == false ) {
+                  FR_RE_TESTOKAY;
+                }
                 break;
               case XK_Escape:
                 endmode = 0;
@@ -512,9 +515,6 @@ int FileRequester::request_entry( char *title,
             }
           }
           break;
-        case AG_KEYPRESSED:
-          firstkey = false;
-          break;
       }
       aguix->ReplyMessage( msg );
     }
@@ -531,11 +531,11 @@ int FileRequester::request_entry( char *title,
     if ( sg2 != NULL ) {
       last_entry_str += sg2->getText();
     } else {
-      tlvc = lv->getActiveLVC();
-      if ( tlvc != NULL ) {
-        if ( strlen( tlvc->getText() ) > 0 ) {
-          if ( tlvc->getData() == 0 ) {
-            last_entry_str += tlvc->getText();
+      trow = lv->getActiveRow();
+      if ( lv->isValidRow( trow ) == true ) {
+        if ( lv->getText( trow, 0 ).length() > 0 ) {
+          if ( lv->getData( trow ) == 0 ) {
+            last_entry_str += lv->getText( trow, 0 );
           } else last_entry_str = "";
         } else last_entry_str = "";
       } else last_entry_str = "";
@@ -558,6 +558,8 @@ int FileRequester::request_entry( char *title,
 void FileRequester::setDirOneSelect( const char *dir )
 {
   char *tstr=HandlePath( dir );
+  int row;
+  
   verz->closeDir();
   if(verz->readDir(tstr)!=0) {
     if(actdir!=NULL) {
@@ -572,12 +574,23 @@ void FileRequester::setDirOneSelect( const char *dir )
   verz->sort(0);
   ArrayList *files=verz->getFiles();
   FileEntry *fe=(FileEntry*)files->getFirstElement();
-  lv->removeAll();
+  lv->setSize( 0 );
   while(fe!=NULL) {
+    row = lv->addRow();
+    lv->setText( row, 0, fe->name );
     if(fe->isDir()==false) {
-      lv->add(new ListViewContent(fe->name,1,1,2,2,0,0,1,1,0));
+      lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
+      lv->setData( row, 0 );
     } else {
-      lv->add(new ListViewContent(fe->name,3,3,2,2,0,0,3,3,1));
+      lv->setFG( row, FieldListView::CC_NORMAL, 3 );
+      lv->setFG( row, FieldListView::CC_SELECT, 3 );
+      lv->setFG( row, FieldListView::CC_ACTIVE, 2 );
+      lv->setFG( row, FieldListView::CC_SELACT, 2 );
+      lv->setBG( row, FieldListView::CC_NORMAL, 0 );
+      lv->setBG( row, FieldListView::CC_SELECT, 0 );
+      lv->setBG( row, FieldListView::CC_ACTIVE, 3 );
+      lv->setBG( row, FieldListView::CC_SELACT, 3 );
+      lv->setData( row, 1 );
     }
     fe=(FileEntry*)files->getNextElement();
   }
