@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: fontreq.cc,v 1.5 2002/04/06 22:32:26 ralf Exp $ */
+/* $Id: fontreq.cc,v 1.12 2003/08/25 20:03:58 ralf Exp $ */
 
 #include "fontreq.h"
 #include <aguix/aguix.h>
@@ -27,8 +27,8 @@
 #include <aguix/text.h>
 #include <aguix/message.h>
 #include <aguix/stringgadget.h>
-#include <aguix/listview.h>
-#include "locale.h"
+#include <aguix/fieldlistview.h>
+#include "worker_locale.h"
 #include <aguix/util.h>
 #include <aguix/request.h>
 
@@ -70,10 +70,10 @@ int FontRequester::request(char *xname,char**return_xname)
   int act_font=0,act_fontsize=0;
   struct request_fonts *font1;
   int id;
-  char buffer[4+sizeof(int)+1];
+  char buffer[ 4 + A_BYTESFORNUMBER( int ) + 1 ];
   int w,h;
   int i,ty,tw,tx,t1,t2;
-  ListViewContent *tlvc,*tlvc2;
+  int trow, trow2;
   Requester *req;
   int nr_of_sfonts=sizeof(use_fonts)/sizeof(struct request_fonts);
   int ni,nj;
@@ -89,19 +89,19 @@ int FontRequester::request(char *xname,char**return_xname)
   tw=ttext->getWidth();
   if(tw>w) w=tw;
 
-  lv=(ListView*)win->add(new ListView(aguix,
-                                      5,
-                                      ty,
-                                      aguix->getCharWidth()*30,
-                                      aguix->getCharHeight()*10,
-                                      0));
+  lv = (FieldListView*)win->add( new FieldListView( aguix,
+                                                    5,
+                                                    ty,
+                                                    aguix->getCharWidth()*30,
+                                                    aguix->getCharHeight()*10,
+                                                    0 ) );
   tx=5+lv->getWidth()+5;
-  lv2=(ListView*)win->add(new ListView(aguix,
-                                       tx,
-                                       ty,
-                                       aguix->getCharWidth()*6,
-                                       aguix->getCharHeight()*10,
-                                       1));
+  lv2 = (FieldListView*)win->add( new FieldListView( aguix,
+                                                     tx,
+                                                     ty,
+                                                     aguix->getCharWidth()*6,
+                                                     aguix->getCharHeight()*10,
+                                                     1 ) );
   ty+=lv->getHeight()+5;
   tw=lv2->getX()+lv2->getWidth()-5;
   if(tw>w) w=tw;
@@ -177,13 +177,14 @@ int FontRequester::request(char *xname,char**return_xname)
   font1=(struct request_fonts*)fontlist->getFirstElement(id);
   i=0;
   while(font1!=NULL) {
-    tlvc=(ListViewContent*)lv->add(new ListViewContent(font1->name,1,1,2,2,0,0,1,1,0));
+    trow = lv->addRow();
+    lv->setText( trow, 0, font1->name );
+    lv->setPreColors( trow, FieldListView::PRECOLOR_ONLYACTIVE );
     if(i==act_font) {
-      tlvc->setActive(true);
+      lv->setActiveRow( trow );
       showsizes(i);
-      tlvc=lv2->get(act_fontsize);
-      if(tlvc!=NULL) {
-        tlvc->setActive(true);
+      if ( lv2->isValidRow( act_fontsize ) == true ) {
+        lv2->setActiveRow( act_fontsize );
       }
     }    
     font1=(struct request_fonts*)fontlist->getNextElement(id);
@@ -192,6 +193,8 @@ int FontRequester::request(char *xname,char**return_xname)
   fontlist->closeEnum(id);
 
   h=ty;
+  okb->takeFocus();
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMaxSize(w,h);
   win->setMinSize(w,h);
@@ -200,12 +203,13 @@ int FontRequester::request(char *xname,char**return_xname)
   updatefont();
   lv->showActive();
   lv2->showActive();
+  lv->redraw();
+  lv2->redraw();
 
   req=new Requester(aguix);
 
   AGMessage *msg;
   int endmode=-1,pos;
-  bool firstkey=true;
   for(;endmode==-1;) {
     msg=aguix->WaitMessage(win);
     if(msg!=NULL) {
@@ -213,16 +217,15 @@ int FontRequester::request(char *xname,char**return_xname)
         case AG_CLOSEWINDOW:
           if(msg->closewindow.window==win->getWindow()) endmode=1;
           break;
-        case AG_LISTVIEW_ONESELECT:
-        case AG_LISTVIEW_MULTISELECT:
-          if(msg->listview.lv==lv) {
-            tlvc=lv->getActiveLVC();
-            if(tlvc!=NULL) {
-              pos=lv->getPos(tlvc);
-              showsizes(pos);
+        case AG_FIELDLV_ONESELECT:
+        case AG_FIELDLV_MULTISELECT:
+          if ( msg->fieldlv.lv == lv ) {
+            trow = lv->getActiveRow();
+            if ( lv->isValidRow( trow ) == true ) {
+              showsizes( trow );
               updatefont();
             }
-          } else if(msg->listview.lv==lv2) {
+          } else if ( msg->fieldlv.lv == lv2 ) {
             updatefont();
           }
           break;
@@ -240,13 +243,13 @@ int FontRequester::request(char *xname,char**return_xname)
               if(aguix->getFont(rstr)!=NULL) {
                 // search for existing entry
                 if(findfont(rstr,&ni,&nj)==0) {
-                  tlvc=lv->get(ni);
-                  if(tlvc!=NULL) {
-                    tlvc->setActive(true);
+                  trow = ni;
+                  if ( lv->isValidRow( trow ) == true ) {
+                    lv->setActiveRow( trow );
                     showsizes(ni);
-                    tlvc2=lv2->get(nj);
-                    if(tlvc2!=NULL) {
-                      tlvc2->setActive(true);
+                    trow2 = nj;
+                    if ( lv2->isValidRow( trow2 ) == true ) {
+                      lv2->setActiveRow( trow2 );
                       updatefont();
                     }
                   }
@@ -262,8 +265,10 @@ int FontRequester::request(char *xname,char**return_xname)
                   font1->size=-1;
                   fontlist->addElement(font1);
 
-                  tlvc=(ListViewContent*)lv->add(new ListViewContent(font1->name,1,1,2,2,0,0,1,1,0));
-                  tlvc->setActive(true);
+                  trow = lv->addRow();
+                  lv->setText( trow, 0, font1->name );
+                  lv->setPreColors( trow, FieldListView::PRECOLOR_ONLYACTIVE );
+                  lv->setActiveRow( trow );
                   showsizes(fontlist->size()-1);
                   updatefont();
                 }
@@ -275,15 +280,17 @@ int FontRequester::request(char *xname,char**return_xname)
             _freesafe(buttonstr);
           }
           break;
-	case AG_KEYRELEASED:
-	  if((win->isParent(msg->key.window,false)==true)&&(firstkey==false)) {
+	case AG_KEYPRESSED:
+	  if ( win->isParent( msg->key.window, false ) == true ) {
 	    // key release only accept when not first key we get
 	    // means a key is pressed when we started and if we use this
 	    // key it could confuse the user
 	    switch(msg->key.key) {
 	      case XK_KP_Enter:
 	      case XK_Return:
-	        endmode=0;
+                if ( cb->hasFocus() == false ) {
+    	          endmode=0;
+                }
 		break;
 	      case XK_Escape:
 	        endmode=1;
@@ -291,9 +298,6 @@ int FontRequester::request(char *xname,char**return_xname)
 	    }
 	  }
 	  break;
-        case AG_KEYPRESSED:
-	  firstkey=false;
-	  break;
       }
       aguix->ReplyMessage(msg);
     }
@@ -301,9 +305,9 @@ int FontRequester::request(char *xname,char**return_xname)
 
   if(endmode==0) {
     endmode=1;
-    tlvc=lv->getActiveLVC();
-    if(tlvc!=NULL) {
-      pos=lv->getPos(tlvc);
+    trow = lv->getActiveRow();
+    if ( lv->isValidRow( trow ) == true ) {
+      pos = trow;
       font1=(struct request_fonts*)fontlist->getElementAt(pos);
       if(font1!=NULL) {  
         if(font1->type==request_fonts::FIXED) {
@@ -312,11 +316,11 @@ int FontRequester::request(char *xname,char**return_xname)
             *return_xname=dupstring(font1->xname);
           }
         } else {
-          tlvc=lv2->getActiveLVC();
-          if(tlvc!=NULL) nj=lv2->getPos(tlvc);
+          trow = lv2->getActiveRow();
+          if ( lv2->isValidRow( trow ) == true ) nj = trow;
           else nj=0;
           if(return_xname!=NULL) {
-            rstr=(char*)_allocsafe(strlen(font1->xname)+sizeof(int)+1);
+            rstr = (char*)_allocsafe( strlen( font1->xname ) + A_BYTESFORNUMBER( int ) + 1 );
             sprintf(rstr,font1->xname,getsize(pos,nj));
             *return_xname=rstr;
           }
@@ -336,8 +340,8 @@ int FontRequester::request(char *xname,char**return_xname)
 
 void FontRequester::showsizes(int i)
 {
-  ListViewContent *tlvc,*flvc=NULL;
-  char buffer[sizeof(int)+1];
+  int trow, frow = -1;
+  char buffer[ A_BYTESFORNUMBER( int ) + 1 ];
   int j,oj;
   struct request_fonts *font1;
 
@@ -345,65 +349,48 @@ void FontRequester::showsizes(int i)
   if(font1!=NULL) {  
     if(font1->type==request_fonts::FIXED) {
       lv2->setSize(1);
-      lv2->finish();
-      tlvc=lv2->get(0);
       sprintf(buffer,"%d",font1->size);
-      tlvc->setText(buffer);
-      tlvc->setFG(0,1);
-      tlvc->setFG(1,1);
-      tlvc->setFG(2,2);
-      tlvc->setFG(3,2);
-      tlvc->setBG(0,0);
-      tlvc->setBG(1,0);
-      tlvc->setBG(2,1);
-      tlvc->setBG(3,1);
-      tlvc->setActive(true);
+      lv2->setText( 0, 0, buffer );
+      lv2->setPreColors( 0, FieldListView::PRECOLOR_ONLYACTIVE );
+      lv2->setActiveRow( 0 );
     } else {
-      tlvc=lv2->getActiveLVC();
-      if(tlvc!=NULL) oj=lv2->getPos(tlvc);
+      trow = lv2->getActiveRow();
+      if ( lv2->isValidRow( trow ) == true ) oj = trow;
       else oj=0;
-      lv2->setSize(FONTREQ_STEPS);
-      lv2->finish();
+      lv2->setSize( FONTREQ_STEPS );
       for(j=0;j<FONTREQ_STEPS;j++) {
-        tlvc=lv2->get(j);
         sprintf(buffer,"%d",getsize(i,j));
-        tlvc->setText(buffer);
-        tlvc->setFG(0,1);
-        tlvc->setFG(1,1);
-        tlvc->setFG(2,2);
-        tlvc->setFG(3,2);
-        tlvc->setBG(0,0);
-        tlvc->setBG(1,0);
-        tlvc->setBG(2,1);
-        tlvc->setBG(3,1);
+        lv2->setText( j, 0, buffer );
+        lv2->setPreColors( j, FieldListView::PRECOLOR_ONLYACTIVE );
         
-        if(j==0) flvc=tlvc;
-        else if(j==oj) flvc=tlvc;
+        if(j==0) frow = j;
+        else if(j==oj) frow = j;
       }
-      if(flvc!=NULL) flvc->setActive(true);
+      if ( lv2->isValidRow( frow ) == true ) lv2->setActiveRow( frow );
     }
   }
+  lv2->redraw();
 }
 
 void FontRequester::updatefont()
 {
-  ListViewContent *tlvc;
+  int trow;
   int i,j,nw,nh,ty,tw,ow,oh;
   char *tstr,*buffer;
   struct request_fonts *font1;
   
-  tlvc=lv->getActiveLVC();
-  if(tlvc!=NULL) {
-    i=lv->getPos(tlvc);
-    tlvc=lv2->getActiveLVC();
-    if(tlvc!=NULL) {
-      j=lv2->getPos(tlvc);
+  trow = lv->getActiveRow();
+  if ( lv->isValidRow( trow ) == true ) {
+    i = trow;
+    trow = lv2->getActiveRow();
+    if ( lv2->isValidRow( trow ) == true ) {
+      j = trow;
 
       font1=(struct request_fonts*)fontlist->getElementAt(i);
       if(font1!=NULL) {  
         tstr=font1->xname;
         if(font1->type==request_fonts::SCALEABLE) {
-          buffer=(char*)_allocsafe(strlen(tstr)+sizeof(int)+1);
+          buffer = (char*)_allocsafe( strlen( tstr ) + A_BYTESFORNUMBER( int ) + 1 );
           sprintf(buffer,tstr,getsize(i,j));
           etext->setFont(buffer);
           etext->redraw();
@@ -442,12 +429,15 @@ void FontRequester::updatefont()
   
           text1->move(text1->getX(),xb->getY()-5-text1->getHeight());
           ftext->move(ftext->getX(),xb->getY()-5-text1->getHeight());
-  
+          
+          win->setMinSize( nw, nh );
+          win->setMaxSize( nw, nh );
           win->resize(nw,nh);
         }
       }
     }
   }
+  lv->redraw();
 }
 
 int FontRequester::getsize(int i,int j)
@@ -470,7 +460,7 @@ int FontRequester::findfont(const char *xn,int *return_i, int *return_j)
   while(font1!=NULL) {  
     tstr=font1->xname;
     if(font1->type==request_fonts::SCALEABLE) {
-      buffer=(char*)_allocsafe(strlen(tstr)+sizeof(int)+1);
+      buffer = (char*)_allocsafe( strlen( tstr ) + A_BYTESFORNUMBER( int ) + 1 );
 
       for(j=0;j<FONTREQ_STEPS;j++) {
         sprintf(buffer,tstr,getsize(i,j));
