@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: functionproto.cc,v 1.7 2001/11/03 17:10:08 ralf Exp $ */
+/* $Id: functionproto.cc,v 1.11 2003/06/04 23:26:48 ralf Exp $ */
 
 #include "functionproto.h"
 #include "worker.h"
@@ -35,9 +35,9 @@ const char *FunctionProto::getName()
   return name;
 }
 
-FunctionProto::FunctionProto(Worker *tworker)
+FunctionProto::FunctionProto()
 {
-  this->worker=tworker;
+  hasConfigure = false;
 }
 
 FunctionProto::~FunctionProto()
@@ -46,9 +46,9 @@ FunctionProto::~FunctionProto()
 
 int FunctionProto::configure()
 {
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   Requester *req;
-  char *textstr,*buttonstr;
+  const char *textstr,*buttonstr;
   
   req=new Requester(aguix);
   textstr=catalog.getLocale(292);
@@ -59,61 +59,63 @@ int FunctionProto::configure()
   return 0;
 }
 
-int FunctionProto::load(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
-  }
-  return 0;
-}
-
-FunctionProto *FunctionProto::duplicate()
+FunctionProto *FunctionProto::duplicate() const
 {
   return NULL;
 }
 
 bool FunctionProto::save(Datei *fh)
 {
-  fh->putInt(0);
   return false;
 }
 
-FunctionProto *FunctionProto::preload(Worker *w,Datei *fh)
-{
-  int id=fh->getInt();
-  FunctionProto *fp=w->getCommand4ID(id);
-  fp->load(fh);
-  return fp;
-}
-
 int FunctionProto::presave(Datei *fh,FunctionProto *fp)
 {
-  fh->putInt(Worker::getID4Command(fp));
-  fp->save(fh);
+  if ( ( fh == NULL ) || ( fp == NULL ) ) return 1;
+  WConfig::configOpenSection( fh, fp->getName() );
+  fp->save( fh );
+  WConfig::configCloseSection( fh );
   return 0;
 }
 
 int FunctionProto::run(ActionMessage *msg)
 {
+  am = msg;
   return 0;
 }
 
-ActionMessage::ActionMessage()
+int FunctionProto::run( WPUContext *wpu, ActionMessage *msg )
+{
+  /* at default do nothing with wpu */
+  return run( msg );
+}
+
+ActionMessage::ActionMessage( Worker *w)
 {
   flags=NULL;
   mode=AM_MODE_NORMAL;
   startLister=NULL;
   extraInfo=NULL;
+  worker = w;
 }
 
 ActionMessage::~ActionMessage()
 {
 }
 
+Worker *ActionMessage::getWorker() const
+{
+  return worker;
+}
+
 const char *FunctionProto::getDescription()
 {
   return "Prototype einer Aktion";
 }
 
+int FunctionProto::configureWhenAvail()
+{
+  if ( hasConfigure == false ) return -1;
+  return configure();
+}
+
