@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: functionproto.h,v 1.7 2001/11/03 17:10:09 ralf Exp $ */
+/* $Id: functionproto.h,v 1.10 2003/05/04 00:56:12 ralf Exp $ */
 
 #ifndef FUNCTIONPROTO_H
 #define FUNCTIONPROTO_H
@@ -30,11 +30,12 @@ class Worker;
 class AGUIX;
 class WCGlobalFlags;
 class Lister;
+class WPUContext;
 
 class ActionMessage
 {
 public:
-  ActionMessage();
+  ActionMessage( Worker *);
   ~ActionMessage();
   WCGlobalFlags *flags;
   
@@ -45,28 +46,32 @@ public:
   am_mode_t mode;
   Lister *startLister;
   void *extraInfo;
+  
+  Worker *getWorker() const;
+protected:
+  Worker *worker;
 };
 
 class FunctionProto
 {
 public:
-  FunctionProto(Worker*);
+  FunctionProto();
   virtual ~FunctionProto();
   virtual int configure();
-  virtual int load(Datei*);
-  virtual FunctionProto *duplicate();
+  virtual FunctionProto *duplicate() const;
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual bool save(Datei *);
   virtual int run(ActionMessage*);
+  virtual int run( WPUContext *, ActionMessage * );
   virtual const char *getDescription();
   
-  static FunctionProto *preload(Worker*,Datei*);
   static int presave(Datei*,FunctionProto*);
+  int configureWhenAvail();
 protected:
   static const char *name;
-  Worker *worker;
   ActionMessage *am;
+  bool hasConfigure;
 };
 
 #endif
