@@ -18,11 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: listermode.h,v 1.6 2001/11/03 17:10:18 ralf Exp $ */
+/* $Id: listermode.h,v 1.10 2003/11/23 21:21:02 ralf Exp $ */
 
 #ifndef LISTERMODE_H
 #define LISTERMODE_H
 
+#include "wdefines.h"
 #include <aguix/aguix.h>
 #include <aguix/message.h>
 #include "lister.h"
@@ -44,9 +45,10 @@ public:
   virtual const char *getType();
   virtual int configure();
   virtual void cyclicfunc(cyclicfunc_mode_t mode);
-  virtual const char *getName();
+  virtual const char *getLocaleName();
 
-  virtual int load(Datei*);
+  virtual int loadBin(Datei*);
+  virtual int load();
   virtual bool save(Datei *);
   
   virtual void reconfig();
@@ -55,6 +57,7 @@ public:
   
   virtual bool startdnd(DNDMsg *dm);
   virtual bool isyours(GUIElement *elem);
+  virtual void lvbDoubleClicked();
 protected:
   Lister *parentlister;
   AGUIX *aguix;
