@@ -1,6 +1,6 @@
 /* normalmode.h
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001 Ralf Hoffmann.
+ * Copyright (C) 2001-2004 Ralf Hoffmann.
  * You can contact me at: ralf.hoffmann@epost.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: normalmode.h,v 1.53 2002/04/05 16:26:57 ralf Exp $ */
+/* $Id: normalmode.h,v 1.92 2004/01/20 22:19:01 ralf Exp $ */
 
 #ifndef NORMALMODE_H
 #define NORMALMODE_H
@@ -26,8 +26,6 @@
 #include "wdefines.h"
 #include <aguix/aguix.h>
 #include <aguix/message.h>
-#include <aguix/listview.h>
-#include <aguix/listviewdnd.h>
 #include <aguix/fieldlistview.h>
 #include <aguix/fieldlistviewdnd.h>
 #include <aguix/stringgadget.h>
@@ -54,7 +52,8 @@ public:
   char *getPattern();
   void setCheck(int);
   int getCheck();
-  int load(Datei*);
+  int load();
+  int loadBin(Datei*);
   int save(Datei*);
   NM_Filter *duplicate();
 private:
@@ -66,11 +65,11 @@ class NMCacheEntry {
 public:
   NMCacheEntry();
   ~NMCacheEntry();
-  void calc();
+  void recalcStats();
   void subentry(FileEntry *fe);
-  void nextcheckfe();
+  void next_checkfe();
   void restartcheck();
-  void firstnullftcheck();
+  void reset_checkfe();
   void checkfordcd();
   void reset_dirsizes();
   void reset_firstnullft();
@@ -109,9 +108,10 @@ struct NM_copyorder {
        _NEVER: no overwriting */
   enum {NM_COPYMODE_NORMAL,NM_COPYMODE_NODATABASE} copymode;
   CopyOpWin *cowin;
-  char *cp_com[2];
 
   int error;  // for returning an error
+  bool ignoreLosedAttr;  // true when the user don't want to get a requester
+                         // when SUID/SGID bit couldn't be restored
 };
 
 struct NM_deleteorder {
@@ -124,8 +124,10 @@ struct NM_deleteorder {
   int error;  // for returning an error
 };
 
-struct NM_externorder {
-  bool withapo;
+class NM_externorder {
+public:
+  NM_externorder();
+  ~NM_externorder();
   bool separate_each_entry;
   bool recursive;
   bool inbackground;
@@ -143,6 +145,8 @@ struct NM_externorder {
   
   // this is for DND-actions for identifying the destination (for the dp-flag)
   class NormalMode *destmode;
+  class WPUContext *wpu;
+  bool take_dirs;
 };
 
 struct NM_startprogorder {
@@ -180,6 +184,12 @@ struct NM_chmodorder {
   List *sources; // contains elements of struct NM_specialsource
 };
 
+struct NM_chownorder {
+  bool ondirs,onfiles,recursive;
+  enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
+  List *sources; // contains elements of struct NM_specialsource
+};
+
 struct NM_changesymlinkorder {
   enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
   List *sources; // contains elements of struct NM_specialsource
@@ -219,6 +229,8 @@ public:
   unsigned long files,dirs;
   loff_t bytes;
   bool ok; // only use this dir if ok==true
+  bool user_abort; // true when user selected cancel from one of
+                   // the requesters used inside
   
   unsigned long error_counter; // files/dirs not finished (recursive)
 };
@@ -229,6 +241,27 @@ struct NM_specialsource {
   NM_CopyOp_Dir *cod; // when fe is dir, this will be the corresponding structure
 };
 
+class NMExtList
+{
+public:
+  NMExtList();
+  ~NMExtList();
+  int getSize();
+  int getRealSize();
+  void setTakeDirs( bool nv);
+  int initEnum();
+  void closeEnum( int );
+  NM_extern_fe *getFirstElement( int id );
+  NM_extern_fe *getNextElement( int id );
+  int removeElement( NM_extern_fe * );
+  void createExtList( class NormalMode *nm, bool rec );
+  void createExtList( class NormalMode *nm, ActionMessage *msg, bool rec );
+protected:
+  List *extlist;
+  int files, dirs;
+  bool take_dirs;
+};
+
 class NormalMode:public ListerMode
 {
 public:
@@ -244,10 +277,13 @@ public:
   virtual const char *getType();
   virtual int configure();
   virtual void cyclicfunc(cyclicfunc_mode_t mode);
-  virtual const char* getName();
-  virtual int load(Datei*);
+  virtual const char* getLocaleName();
+  virtual int load();
+  virtual int loadBin(Datei*);
   virtual bool save(Datei *);
   virtual void reconfig();
+  static const char *getStaticLocaleName();
+  static const char *getStaticType();
   
   int enterDir(const char *dirstr);
   void enterDirActive();
@@ -269,22 +305,23 @@ public:
   void update(bool reset_dirsizes);
   void update(bool reset_dirsizes, bool keep_filetypes );
   void makedir();
-  void externprog(struct NM_externorder *extorder);
+  void externprog(class NM_externorder *extorder);
   void renamef(struct NM_renameorder *renorder);
   void dirsize(struct NM_dirsizeorder *dsorder);
   void simdd();
   void startprog(struct NM_startprogorder *sporder);
   void searchentry();
+  void searchentry( bool ignore_case );
   void enterpath();
   void scrolllister(int dir);
   void createsymlink(struct NM_createsymlinkorder *cslorder);
   void changesymlink(struct NM_changesymlinkorder *chslorder);
   void chmodf(struct NM_chmodorder *cmorder);
+  void chownf(struct NM_chownorder *coorder);
   
   void toggleHiddenE();
   void setShowHiddenE(bool);
-  char *getActualDir();
-  static bool isCorrectCPProg(const char *str);
+  char *getCurrentDir();
   static bool isCorrectViewProg(const char *str);
   
   typedef enum {NM_GETFILES_ONLYACTIVE,
@@ -307,16 +344,64 @@ public:
 #ifdef WANT_THREADS
   void slavehandler();
 #endif
+  int createExtList( bool rec, List *extlist );
+  static void freeExtList( List *extlist );
+  char *parseComStrExt(char *sstr,class NM_externorder *extorder,int maxlen, NMExtList *entries[2], bool quote);
+  /* deselect for extern access, check for existance of fe => slow for big dirs */
+  void deselect( FileEntry *fe );
+  int createExtList( ActionMessage *amsg, bool rec, List *extlist );
+  static int getEntriesRealSize( List* );
+  virtual void lvbDoubleClicked();
+  
+  int showCacheList();
+  int request( const char *title,
+               const char *text,
+               const char *buttons,
+               Requester::request_flags_t flags = Requester::REQUEST_NONE );
+  int string_request( const char *title,
+                      const char *lines,
+                      const char *default_str,
+                      const char *buttons,
+                      char **return_str,
+                      Requester::request_flags_t flags = Requester::REQUEST_NONE );
+  int request( CopyOpWin *cowin,
+               const char *title,
+               const char *text,
+               const char *buttons,
+               Requester::request_flags_t flags = Requester::REQUEST_NONE );
+  int string_request( CopyOpWin *cowin,
+                      const char *title,
+                      const char *lines,
+                      const char *default_str,
+                      const char *buttons,
+                      char **return_str,
+                      Requester::request_flags_t flags = Requester::REQUEST_NONE );
+  int request( DeleteOpWin *dowin,
+               const char *title,
+               const char *text,
+               const char *buttons,
+               Requester::request_flags_t flags = Requester::REQUEST_NONE );
+  int string_request( DeleteOpWin *dowin,
+                      const char *title,
+                      const char *lines,
+                      const char *default_str,
+                      const char *buttons,
+                      char **return_str,
+                      Requester::request_flags_t flags = Requester::REQUEST_NONE );
+  int tryApplyOwnerPerm( const char *filename,
+                         const uid_t wanted_user,
+                         const gid_t wanted_group,
+                         const mode_t wanted_mode,
+                         struct NM_copyorder *copyorder );
 protected:
-  void reconf(int,int,int,int);
-  void buildLister(Verzeichnis *verz);
+  void reconf( int nx, int ny, int nw, int nh, bool force = false );
   void buildLister();
   void deleteCache();
   void deleteFilters();
   void showcache(int);
-  void checkFilters(Verzeichnis*);
+  void showCacheAbs( int pos );
+  void checkFilters( NMCacheEntry* );
   void showCacheState();
-  void check(NMCacheEntry*);
   void toggleHidden();
   void setShowHidden(bool);
   void startAction(FileEntry*);
@@ -353,19 +438,20 @@ protected:
                bool dir);
 
   NMFieldListViewDND *lv;
-  Button *hb[2];
+  Button *hb[2], *parentb;
   StringGadget *sg;
   static const char *type;
   int x,y,w,h;
   NMCacheEntry *ce;
+  int stored_cacheentry;
   List *cache;
-  int actcacheentry;
   List *filters;
   bool showHidden;
   Requester *req;
   #define BUFSIZE 128*1024
   void *buf;
   bool searchmodeon;
+  bool searchmode_ignore_case;
   char *oldstr;
   char *lastsearch;
   List *copy_deletefe_list,*copy_lvc_list;
@@ -376,37 +462,57 @@ protected:
   int updatetime;
   time_t lastfsupdate;
   char *namestr;
+  char *freespacestr;
   
   struct timeval lastclick;
   int lastrow;
-  bool isDoubleClick(struct timeval *,struct timeval*);
   void removeEntry(FileEntry *fe, int row,bool quick);
+  void fixSpecialSourceList( List *l, int deleted_row );
   int createExtList(List *splist,bool rec,List *extlist);
   int create_externfe_rec(List *extlist,
                           char *fullname,
                           bool rec,
                           FileEntry *fe,
                           int row );
-  char *parseComStr(char *sstr,struct NM_externorder *extorder,int maxlen,List *entries[2]);
+  char *parseComStr(char *sstr,class NM_externorder *extorder,int maxlen, NMExtList *entries[2], bool quote );
   void deselect(FileEntry *fe, int row );
-  char *StringRequest(char *sstr,struct NM_externorder *extorder,int maxlen,List *entries[2]);
-  int getEntriesRealSize(List*);
+  char *StringRequest(char *sstr,class NM_externorder *extorder,int maxlen, NMExtList *entries[2]);
   void startsearchmode();
   void finishsearchmode();
   bool showfirstentry(const char *str);
   bool shownextentry();
-  bool shownextentry(const char *str,int pos);
-  void buildchmodfelist(List *cmlist,List *cmfelist);
-  void buildchmodfelist_rec(NM_CopyOp_Dir *cod, int row,List *cmfelist);
+  bool shownextentry( const char *str, int pos );
+
+  typedef struct changemod_info{
+    changemod_info();
+    bool forAll;
+    mode_t newmode;
+  } changemod_info_t;
+  int worker_changemod( const struct NM_specialsource *ss1,
+                        changemod_info_t *cminfo,
+                        const NM_chmodorder *cmorder );
+  int applyNewMode( const NM_specialsource *ss1, mode_t newmode );
+  typedef struct changeown_info{
+    changeown_info();
+    bool forAll;
+    uid_t newuid;
+    gid_t newgid;
+  } changeown_info_t;
+  int worker_changeown( const struct NM_specialsource *ss1,
+                        changeown_info_t *coinfo,
+                        const NM_chownorder *coorder );
+  int applyNewOwner( const NM_specialsource *ss1, uid_t newuid, gid_t newgid );
+
   int requestNewMode(FileEntry *fe,mode_t *return_mode);
   void setFilters(List*);
   void configureFilters(bool*,List*);
   void setLVC4Filter( FieldListView *filv, int row, NM_Filter *fi);
   int configureFilter(NM_Filter *fi);
-  void recheck(NMCacheEntry *ce,bool forcerebuild);
+  void checkRebuild( NMCacheEntry *ce, bool forcerebuild );
   void setName();
+  void updateName();
   void showFreeSpace(bool force);
-  void showFreeSpace(int ti);
+  int updateFreeSpaceStr( int ti );
   void setShowFreeSpace(bool v);
   void setUpdatetime(int nv);
   
@@ -418,22 +524,30 @@ protected:
   static int maxfilestrlen,maxdirstrlen,maxbytesstrlen,maxwbytesstrlen,maxnochanges;
   
 #ifdef WANT_THREADS
+public:   // why the h*** need the Sun compiler a public definition for a
+          // protected struct?
   typedef enum { THREAD_NOP,
                  THREAD_WAIT,
                  THREAD_RUN,
                  THREAD_EXIT,
                  THREAD_REINIT } thread_order_t;
+protected:
   struct {
     int running;
     thread_order_t order;
     thread_order_t status;
+    CondVar statusvar;
     pthread_t th;
     
-    CondVar waitvar;
-    
     List *filetypes;
     CondVar filetype_ex;
+    
   } slave;
+  int switchOrder( thread_order_t neworder );
+  int waitForNoOrder( void );
+  int waitForNewOrder( void );
+  int switchStatus( thread_order_t newstatus );
+  int waitForStatus( thread_order_t waitstatus );
   // true if thread creation was successful
   // false otherwise and also when WANT_THREADS is not defined
 #endif
@@ -531,6 +645,33 @@ protected:
   bool finish_done;
   
   void setupLVFields();
+
+  typedef struct {
+    union {
+      uid_t uid;
+      gid_t gid;
+    } id;
+    char *name;
+  } chownrequest_id_name_t;
+  
+  void freeOwnerRequestInfos();
+  void buildOwnerRequestInfos();
+  int requestNewOwner( FileEntry *fe, uid_t *return_owner, gid_t *return_group );  
+  List *chownUserList;
+  List *chownGroupList;
+  
+  bool isColumnVisible( const WorkerTypes::listcol_t c );
+  bool safeParse;
+  bool visChanged;
+  int oldlvy;
+  FileEntry *findFileEntryForData( int data, int startpos );
+  NM_specialsource *getSpecialsourceForFE( FileEntry *searchfe );
+  
+  void showInvalidList( ArrayList *list, const char *descr );
+  void changeSortModeForField( int field );
+  
+  int busyFlag;
+  time_t lastBusyFlagUpdate;
 };
 
 class NMFieldListViewDND:public FieldListViewDND
