@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: normalops.cc,v 1.8 2001/11/03 17:10:57 ralf Exp $ */
+/* $Id: normalops.cc,v 1.19 2004/01/20 22:18:55 ralf Exp $ */
 
 #include "normalops.h"
 #include "listermode.h"
@@ -41,18 +41,19 @@ const char *ChangeListerSetOp::getName()
   return name;
 }
 
-ChangeListerSetOp::ChangeListerSetOp(Worker *tworker):FunctionProto(tworker)
+ChangeListerSetOp::ChangeListerSetOp() : FunctionProto()
 {
-  mode=CLS_ACTUAL_LISTER;
+  mode=CLS_CURRENT_LISTER;
+  hasConfigure = true;
 }
 
 ChangeListerSetOp::~ChangeListerSetOp()
 {
 }
 
-ChangeListerSetOp *ChangeListerSetOp::duplicate()
+ChangeListerSetOp *ChangeListerSetOp::duplicate() const
 {
-  ChangeListerSetOp *ta=new ChangeListerSetOp(worker);
+  ChangeListerSetOp *ta=new ChangeListerSetOp();
   ta->mode=mode;
   return ta;
 }
@@ -62,16 +63,16 @@ int ChangeListerSetOp::run(ActionMessage *msg)
   Lister *l1;
   switch(mode) {
     case CLS_LEFT_LISTER:
-      l1=worker->getLister(0);
+      l1 = msg->getWorker()->getLister(0);
       break;
     case CLS_RIGHT_LISTER:
-      l1=worker->getLister(1);
+      l1 = msg->getWorker()->getLister(1);
       break;
     case CLS_OTHER_LISTER:
-      l1=worker->getOtherLister(worker->getActiveLister());
+      l1 = msg->getWorker()->getOtherLister( msg->getWorker()->getActiveLister() );
       break;
     default:
-      l1=worker->getActiveLister();
+      l1 = msg->getWorker()->getActiveLister();
   }
   if(l1!=NULL) l1->configure();
   return 0;
@@ -84,7 +85,7 @@ const char *ChangeListerSetOp::getDescription()
 
 int ChangeListerSetOp::configure()
 {
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   Button *okb,*cb;
   AWindow *win;
   CycleButton *rcyb;
@@ -116,7 +117,7 @@ int ChangeListerSetOp::configure()
     case CLS_RIGHT_LISTER:
       rcyb->setOption(1);
       break;
-    case CLS_ACTUAL_LISTER:
+    case CLS_CURRENT_LISTER:
       rcyb->setOption(2);
       break;
     case CLS_OTHER_LISTER:
@@ -157,6 +158,7 @@ int ChangeListerSetOp::configure()
   y+=okb->getHeight()+5;
   
   h=y;
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMaxSize(w,h);
   win->setMinSize(w,h);
@@ -184,7 +186,7 @@ int ChangeListerSetOp::configure()
         mode=CLS_RIGHT_LISTER;
         break;
       case 2:
-        mode=CLS_ACTUAL_LISTER;
+        mode=CLS_CURRENT_LISTER;
         break;
       case 3:
         mode=CLS_OTHER_LISTER;
@@ -203,51 +205,20 @@ int ChangeListerSetOp::configure()
 
 bool ChangeListerSetOp::save(Datei *fh)
 {
-  int size=Datei::getUCharSize();
-  fh->putInt(size);
-
+  if ( fh == NULL ) return false;
   switch(mode) {
     case CLS_RIGHT_LISTER:
-      fh->putUChar(1);
+      WConfig::configPutPair( fh, "mode", "right" );
       break;
-    case CLS_ACTUAL_LISTER:
-      fh->putUChar(2);
+    case CLS_CURRENT_LISTER:
+      WConfig::configPutPair( fh, "mode", "current" );
       break;
     case CLS_OTHER_LISTER:
-      fh->putUChar(3);
+      WConfig::configPutPair( fh, "mode", "other" );
       break;
     default:
-      fh->putUChar(0);
-  }
-  return true;
-}
-
-int ChangeListerSetOp::load(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  int cs=Datei::getUCharSize();
-  unsigned char c1;
-  if(chunksize>=cs) {
-    c1=fh->getUChar();
-    switch(c1) {
-      case 1:
-        mode=CLS_RIGHT_LISTER;
-        break;
-      case 2:
-        mode=CLS_ACTUAL_LISTER;
-        break;
-      case 3:
-        mode=CLS_OTHER_LISTER;
-        break;
-      default:
-        mode=CLS_LEFT_LISTER;
-        break;
-    }
-    chunksize-=cs;
-  }
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
+      WConfig::configPutPair( fh, "mode", "left" );
+      break;
   }
   return true;
 }
@@ -258,7 +229,7 @@ void ChangeListerSetOp::setMode(int nv)
     case CLS_RIGHT_LISTER:
       mode=nv;
       break;
-    case CLS_ACTUAL_LISTER:
+    case CLS_CURRENT_LISTER:
       mode=nv;
       break;
     case CLS_OTHER_LISTER:
@@ -289,7 +260,7 @@ const char *SwitchListerOp::getName()
   return name;
 }
 
-SwitchListerOp::SwitchListerOp(Worker *tworker):FunctionProto(tworker)
+SwitchListerOp::SwitchListerOp() : FunctionProto()
 {
 }
 
@@ -297,16 +268,16 @@ SwitchListerOp::~SwitchListerOp()
 {
 }
 
-SwitchListerOp *SwitchListerOp::duplicate()
+SwitchListerOp *SwitchListerOp::duplicate() const
 {
-  SwitchListerOp *ta=new SwitchListerOp(worker);
+  SwitchListerOp *ta=new SwitchListerOp();
   return ta;
 }
 
 int SwitchListerOp::run(ActionMessage *msg)
 {
   Lister *l1;
-  l1=worker->getOtherLister(worker->getActiveLister());
+  l1 = msg->getWorker()->getOtherLister( msg->getWorker()->getActiveLister() );
   if(l1!=NULL) l1->makeActive();
   return 0;
 }
@@ -325,6 +296,11 @@ const char *SwitchListerOp::getDescription()
 *******/
 
 const char *FilterSelectOp::name="FilterSelectOp";
+int FilterSelectOp::instance_counter = 0;
+List *FilterSelectOp::filter_history = NULL;
+int FilterSelectOp::history_size = 30;
+int FilterSelectOp::lastw = -1;
+int FilterSelectOp::lasth = -1;
 
 bool FilterSelectOp::isName(const char *str)
 {
@@ -336,23 +312,33 @@ const char *FilterSelectOp::getName()
   return name;
 }
 
-FilterSelectOp::FilterSelectOp(Worker *tworker):FunctionProto(tworker)
+FilterSelectOp::FilterSelectOp() : FunctionProto()
 {
   filter=(char*)_allocsafe(2);
   strcpy(filter,"*");
   actual_filter=(char*)_allocsafe(1);
   actual_filter[0]=0;
+  
+  if ( instance_counter == 0 ) {
+    initHistory();
+  }
+  instance_counter++;
+  hasConfigure = true;
 }
 
 FilterSelectOp::~FilterSelectOp()
 {
   _freesafe(filter);
   _freesafe(actual_filter);
+  instance_counter--;
+  if ( instance_counter == 0 ) {
+    closeHistory();
+  }
 }
 
-FilterSelectOp *FilterSelectOp::duplicate()
+FilterSelectOp *FilterSelectOp::duplicate() const
 {
-  FilterSelectOp *ta=new FilterSelectOp(worker);
+  FilterSelectOp *ta=new FilterSelectOp();
   _freesafe(ta->filter);
   ta->filter=dupstring(filter);
   return ta;
@@ -365,7 +351,7 @@ int FilterSelectOp::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    l1=worker->getActiveLister();
+    l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -392,7 +378,7 @@ int FilterSelectOp::configure()
   int w,h,tw,ty,tx;
   int t1,t2;
   StringGadget *sg;
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   AWindow *win=new AWindow(aguix);
   char *tstr;
 
@@ -439,6 +425,7 @@ int FilterSelectOp::configure()
                                                  0));
   h=okb->getY()+okb->getHeight()+5;
   sg->resize(w-10-tt1->getWidth()-5,sg->getHeight());
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMinSize(w,h);
   win->setMaxSize(w,h);
@@ -469,42 +456,31 @@ int FilterSelectOp::configure()
 
 bool FilterSelectOp::save(Datei *fh)
 {
-  fh->putInt(Datei::getIntSize()+strlen(filter));
-  fh->putInt(strlen(filter));
-  fh->putString(filter);
+  if ( fh == NULL ) return false;
+  WConfig::configPutPairString( fh, "filter", filter );
   return true;
 }
 
-int FilterSelectOp::load(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  int len;
-  char *str;
-  if(chunksize>=Datei::getIntSize()) {
-    len=fh->getInt();
-    chunksize-=Datei::getIntSize();
-    str=fh->getString(len);
-    _freesafe(filter);
-    filter=str;
-    chunksize-=len;
-  }
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
-  }
-  return 0;
-}
-
 bool FilterSelectOp::requestfilter()
 {
+  if ( filter_history == NULL ) return false;  // this can only happen when the stupid
+                                               // programmer calls closeHistory
+                                               // but to be sure...
+
   int w,h,tw,ty;
   int t1,t2;
   StringGadget *sg;
-  AGUIX *aguix=worker->getAGUIX();
+  FieldListView *lv;
+  AGUIX *aguix = Worker::getAGUIX();
   AWindow *win=new AWindow(aguix);
-  win->create(NULL,10,10,10,10,0,catalog.getLocale(268));
-  w=0;
-  h=0;
+  int id, row;
+  char *tstr;
+  bool lvstill;
+  int lastrow;
+  GUIElement *bs[2];
+
+  w = h = 10;
+  win->create( NULL, 10, 10, w, h, 0, catalog.getLocale( 268 ) );
   ty=5;
   Text *tt1=(Text*)win->add(new Text(aguix,5,ty,catalog.getLocale(270),1));
   tw=tt1->getWidth()+10;
@@ -515,7 +491,31 @@ bool FilterSelectOp::requestfilter()
   if(tw>w) w=tw;
   ty+=tt1->getHeight()+5;
 
-  sg=(StringGadget*)win->add(new StringGadget(aguix,5,ty,tw-10,
+  lv = (FieldListView*)win->add( new FieldListView( aguix, 5, ty, a_max( w - 10, 10 * aguix->getCharWidth() ),
+                                                    7 * aguix->getCharHeight(), 0 ) );
+
+  lv->setHBarState( 2 );
+  lv->setVBarState( 2 );
+  lv->setDisplayFocus( true );
+  lv->setAcceptFocus( true );
+
+  id = filter_history->initEnum();
+  tstr = (char*)filter_history->getFirstElement( id );
+  while ( tstr != NULL ) {
+    row = lv->insertRow( 0 );
+    lv->setText( row, 0, tstr );
+    lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
+    tstr = (char*)filter_history->getNextElement( id );
+  }
+  filter_history->closeEnum( id );
+  lv->showRow( filter_history->size() - 1 );
+  lv->redraw();
+
+  tw = lv->getWidth() + 10;
+  if ( tw > w ) w = tw;
+  ty += lv->getHeight();
+
+  sg=(StringGadget*)win->add(new StringGadget(aguix,5,ty,w-10,
                                               strlen(actual_filter)>0?actual_filter:filter,
                                               0));
   tw=sg->getWidth()+10;
@@ -544,16 +544,31 @@ bool FilterSelectOp::requestfilter()
                                                  1,
                                                  0,
                                                  0));
-  h=okb->getY()+okb->getHeight()+5;
-  sg->resize(w-10,sg->getHeight());
+  bs[0] = okb;
+  bs[1] = cancelb;
+
+  h = okb->getY() + okb->getHeight() + 5;
+  win->setMinSize( w, h );
+  
+  w = ( lastw > 0 ) ? lastw : w;
+  h = ( lasth > 0 ) ? lasth : h;
+
+  AGUIX::scaleElementsW( w, 5, 5, -1, false, false, bs, NULL, 2 );
+  okb->move( okb->getX(), h - 5 - okb->getHeight() );
+  cancelb->move( cancelb->getX(), h - 5 - cancelb->getHeight() );
+  sg->resize( w - 10, sg->getHeight() );
+  sg->move( sg->getX(), okb->getY() - 5 - sg->getHeight() );
+  lv->resize( sg->getWidth(), sg->getY() - lv->getY() );
+  
+  win->setDoTabCycling( true );
   win->resize(w,h);
-  win->setMinSize(w,h);
-  win->setMaxSize(w,h);
   win->show();
   AGMessage *msg;
   while((msg=aguix->GetMessage(NULL))!=NULL) aguix->ReplyMessage(msg);
   int ende=0;
-  sg->activate();
+  sg->takeFocus();
+  lastrow = -1;
+  lvstill = false;
   while(ende==0) {
     msg=aguix->WaitMessage(win);
     if(msg!=NULL) {
@@ -566,6 +581,60 @@ bool FilterSelectOp::requestfilter()
         if(msg->stringgadget.sg==sg) ende=1;
       } else if(msg->type==AG_STRINGGADGET_CANCEL) {
         if(msg->stringgadget.sg==sg) ende=-1;
+      } else if ( msg->type == AG_KEYPRESSED ) {
+        if ( win->isParent( msg->key.window, false ) == true ) {
+          switch ( msg->key.key ) {
+          case XK_Up:
+            if ( sg->isActive() == true ) {
+              lv->setActiveRow( lv->getElements() - 1 );
+              lv->takeFocus();
+              lv->showActive();
+              lastrow = lv->getActiveRow();
+              tstr = (char*)filter_history->getElementAt( filter_history->size() - 1 - lastrow );
+              if ( tstr != NULL ) {
+                sg->setText( tstr );
+              }
+            }
+            lvstill = false;
+            break;
+          case XK_Down:
+            if ( lv->hasFocus() == true ) {
+              if ( ( lv->getActiveRow() == ( lv->getElements() - 1 ) ) &&
+                 ( lvstill == true ) ) {
+                lv->setActiveRow( -1 );
+                sg->takeFocus();
+              }
+            }
+            break;
+          case XK_Escape:
+            ende = -1;
+            break;
+          case XK_Return:
+            if ( cancelb->hasFocus() == false )
+              ende = 1;
+            break;
+          }
+        }
+      } else if ( ( msg->type == AG_FIELDLV_ONESELECT ) || ( msg->type == AG_FIELDLV_MULTISELECT ) ) {
+        row = lv->getActiveRow();
+        if ( row == lastrow ) lvstill = true;
+        else lvstill = false;
+        lastrow = row;
+        if ( lv->isValidRow( row ) == true ) {
+          tstr = (char*)filter_history->getElementAt( filter_history->size() - 1 - row );
+          if ( tstr != NULL ) {
+            sg->setText( tstr );
+          }
+        }
+      } else if ( msg->type == AG_SIZECHANGED ) {
+        if ( msg->size.window == win->getWindow() ) {
+          AGUIX::scaleElementsW( msg->size.neww, 5, 5, -1, false, false, bs, NULL, 2 );
+          okb->move( okb->getX(), msg->size.newh - 5 - okb->getHeight() );
+          cancelb->move( cancelb->getX(), msg->size.newh - 5 - cancelb->getHeight() );
+          sg->resize( msg->size.neww - 10, sg->getHeight() );
+          sg->move( sg->getX(), okb->getY() - 5 - sg->getHeight() );
+          lv->resize( sg->getWidth(), sg->getY() - lv->getY() );
+        }
       }
       aguix->ReplyMessage(msg);
     }
@@ -573,12 +642,110 @@ bool FilterSelectOp::requestfilter()
   if(ende==1) {
     _freesafe(actual_filter);
     actual_filter=dupstring(sg->getText());
+    addHistoryItem( actual_filter );
   }
+  lastw = win->getWidth();
+  lasth = win->getHeight();
   win->close();
   delete win;
   return (ende==1?true:false);
 }
 
+void FilterSelectOp::initHistory()
+{
+  if ( filter_history != NULL ) return;
+  char *tstr;
+  Datei *fp = new Datei();
+  std::string str1;
+
+  filter_history = new List();
+
+  tstr = getenv( "HOME" );
+  if ( tstr != NULL ) {
+    str1 = tstr;
+    str1 += "/.worker/filterselect-history";
+    fp->open( str1.c_str(), "r" );
+    
+    while ( fp->isEOF() == false ) {
+      tstr = fp->getLine();
+      if ( tstr == NULL ) break;
+      if ( strlen( tstr ) > 0 ) {
+        filter_history->addElement( tstr );
+      } else {
+        _freesafe( tstr );
+      }
+    }
+    
+    fp->close();
+  }
+  delete fp;
+}
+
+void FilterSelectOp::closeHistory()
+{
+  if ( filter_history == NULL ) return;
+  char *tstr;
+  int id;
+  Datei *fp = new Datei();
+  std::string str1;
+
+  str1 = wconfig->getHome();
+  str1 += "/filterselect-history";
+  fp->open( str1.c_str(), "w" );
+  
+  id = filter_history->initEnum();
+  tstr = (char*)filter_history->getFirstElement( id );
+  while ( tstr != NULL ) {
+    fp->putLine( tstr );
+    _freesafe( tstr );
+    filter_history->removeFirstElement();
+    tstr = (char*)filter_history->getFirstElement( id );
+  }
+  filter_history->closeEnum( id );
+  delete filter_history;
+  filter_history = NULL;
+  
+  fp->close();
+  delete fp;
+}
+
+void FilterSelectOp::addHistoryItem( const char *str )
+{
+  char *tstr;
+  int id;
+  
+  id = filter_history->initEnum();
+  tstr = (char*)filter_history->getFirstElement( id );
+  while ( tstr != NULL ) {
+    if ( strcmp( str, tstr ) == 0 ) break;
+    tstr = (char*)filter_history->getNextElement( id );
+  }
+  filter_history->closeEnum( id );
+
+  if ( tstr == NULL ) {
+    // not already in the list
+    if ( filter_history->size() >= history_size ) {
+      tstr = (char*)filter_history->getElementAt( filter_history->size() - 1 );
+      if ( tstr != NULL ) {
+        _freesafe( tstr );
+      }
+      filter_history->removeElementAt( filter_history->size() - 1 );
+    }
+    filter_history->addElementAt( 0, dupstring( str ) );
+  } else {
+    // in the list so put it at the beginning
+    filter_history->removeElement( tstr );
+    filter_history->addElementAt( 0, tstr );
+  }
+}
+
+void FilterSelectOp::setFilter( const char *str )
+{
+  if ( str == NULL ) return;
+  if ( filter != NULL ) _freesafe( filter );
+  filter = dupstring( str );
+}
+
 /*************
  Ende FilterSelectOp
 **************/
@@ -599,7 +766,7 @@ const char *FilterUnSelectOp::getName()
   return name;
 }
 
-FilterUnSelectOp::FilterUnSelectOp(Worker *tworker):FilterSelectOp(tworker)
+FilterUnSelectOp::FilterUnSelectOp() : FilterSelectOp()
 {
 }
 
@@ -607,9 +774,9 @@ FilterUnSelectOp::~FilterUnSelectOp()
 {
 }
 
-FilterUnSelectOp *FilterUnSelectOp::duplicate()
+FilterUnSelectOp *FilterUnSelectOp::duplicate() const
 {
-  FilterUnSelectOp *ta=new FilterUnSelectOp(worker);
+  FilterUnSelectOp *ta=new FilterUnSelectOp();
   _freesafe(ta->filter);
   ta->filter=dupstring(filter);
   return ta;
@@ -622,7 +789,7 @@ int FilterUnSelectOp::run(ActionMessage *msg)
   NormalMode *nm1;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    l1=worker->getActiveLister();
+    l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
@@ -649,7 +816,7 @@ int FilterUnSelectOp::configure()
   int w,h,tw,ty,tx;
   int t1,t2;
   StringGadget *sg;
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   AWindow *win=new AWindow(aguix);
   char *tstr;
 
@@ -696,6 +863,7 @@ int FilterUnSelectOp::configure()
                                                  0));
   h=okb->getY()+okb->getHeight()+5;
   sg->resize(w-10-tt1->getWidth()-5,sg->getHeight());
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMinSize(w,h);
   win->setMaxSize(w,h);
@@ -726,11 +894,21 @@ int FilterUnSelectOp::configure()
 
 bool FilterUnSelectOp::requestfilter()
 {
+  if ( filter_history == NULL ) return false;  // this can only happen when the stupid
+                                               // programmer calls closeHistory
+                                               // but to be sure...
+
   int w,h,tw,ty;
   int t1,t2;
   StringGadget *sg;
-  AGUIX *aguix=worker->getAGUIX();
+  FieldListView *lv;
+  AGUIX *aguix = Worker::getAGUIX();
   AWindow *win=new AWindow(aguix);
+  int id, row;
+  char *tstr;
+  bool lvstill;
+  int lastrow;
+
   win->create(NULL,10,10,10,10,0,catalog.getLocale(269));
   w=0;
   h=0;
@@ -744,6 +922,30 @@ bool FilterUnSelectOp::requestfilter()
   if(tw>w) w=tw;
   ty+=tt1->getHeight()+5;
 
+  lv = (FieldListView*)win->add( new FieldListView( aguix, 5, ty, a_max( w - 10, 10 * aguix->getCharWidth() ),
+                                                    7 * aguix->getCharHeight(), 0 ) );
+
+  lv->setHBarState( 2 );
+  lv->setVBarState( 2 );
+  lv->setDisplayFocus( true );
+  lv->setAcceptFocus( true );
+
+  id = filter_history->initEnum();
+  tstr = (char*)filter_history->getFirstElement( id );
+  while ( tstr != NULL ) {
+    row = lv->insertRow( 0 );
+    lv->setText( row, 0, tstr );
+    lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
+    tstr = (char*)filter_history->getNextElement( id );
+  }
+  filter_history->closeEnum( id );
+  lv->showRow( filter_history->size() - 1 );
+  lv->redraw();
+
+  tw = lv->getWidth() + 10;
+  if ( tw > w ) w = tw;
+  ty += lv->getHeight();
+
   sg=(StringGadget*)win->add(new StringGadget(aguix,5,ty,tw-10,
                                               strlen(actual_filter)>0?actual_filter:filter,
                                               0));
@@ -775,6 +977,8 @@ bool FilterUnSelectOp::requestfilter()
                                                  0));
   h=okb->getY()+okb->getHeight()+5;
   sg->resize(w-10,sg->getHeight());
+  lv->resize( w - 10, lv->getHeight() );
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMinSize(w,h);
   win->setMaxSize(w,h);
@@ -782,7 +986,9 @@ bool FilterUnSelectOp::requestfilter()
   AGMessage *msg;
   while((msg=aguix->GetMessage(NULL))!=NULL) aguix->ReplyMessage(msg);
   int ende=0;
-  sg->activate();
+  sg->takeFocus();
+  lastrow = -1;
+  lvstill = false;
   while(ende==0) {
     msg=aguix->WaitMessage(win);
     if(msg!=NULL) {
@@ -795,6 +1001,51 @@ bool FilterUnSelectOp::requestfilter()
         if(msg->stringgadget.sg==sg) ende=1;
       } else if(msg->type==AG_STRINGGADGET_CANCEL) {
         if(msg->stringgadget.sg==sg) ende=-1;
+      } else if ( msg->type == AG_KEYPRESSED ) {
+        if ( win->isParent( msg->key.window, false ) == true ) {
+          switch ( msg->key.key ) {
+          case XK_Up:
+            if ( sg->isActive() == true ) {
+              lv->setActiveRow( lv->getElements() - 1 );
+              lv->takeFocus();
+              lv->showActive();
+              lastrow = lv->getActiveRow();
+              tstr = (char*)filter_history->getElementAt( filter_history->size() - 1 - lastrow );
+              if ( tstr != NULL ) {
+                sg->setText( tstr );
+              }
+            }
+            lvstill = false;
+            break;
+          case XK_Down:
+            if ( lv->hasFocus() == true ) {
+              if ( ( lv->getActiveRow() == ( lv->getElements() - 1 ) ) &&
+                 ( lvstill == true ) ) {
+                lv->setActiveRow( -1 );
+                sg->takeFocus();
+              }
+            }
+            break;
+          case XK_Escape:
+            ende = -1;
+            break;
+          case XK_Return:
+            if ( cancelb->hasFocus() == false )
+              ende = 1;
+            break;
+          }
+        }
+      } else if ( msg->type == AG_FIELDLV_ONESELECT ) {
+        row = lv->getActiveRow();
+        if ( row == lastrow ) lvstill = true;
+        else lvstill = false;
+        lastrow = row;
+        if ( lv->isValidRow( row ) == true ) {
+          tstr = (char*)filter_history->getElementAt( filter_history->size() - 1 - row );
+          if ( tstr != NULL ) {
+            sg->setText( tstr );
+          }
+        }
       }
       aguix->ReplyMessage(msg);
     }
@@ -802,6 +1053,7 @@ bool FilterUnSelectOp::requestfilter()
   if(ende==1) {
     _freesafe(actual_filter);
     actual_filter=dupstring(sg->getText());
+    addHistoryItem( actual_filter );
   }
   win->close();
   delete win;
@@ -828,7 +1080,7 @@ const char *Path2OSideOp::getName()
   return name;
 }
 
-Path2OSideOp::Path2OSideOp(Worker *tworker):FunctionProto(tworker)
+Path2OSideOp::Path2OSideOp() : FunctionProto()
 {
 }
 
@@ -836,9 +1088,9 @@ Path2OSideOp::~Path2OSideOp()
 {
 }
 
-Path2OSideOp *Path2OSideOp::duplicate()
+Path2OSideOp *Path2OSideOp::duplicate() const
 {
-  Path2OSideOp *ta=new Path2OSideOp(worker);
+  Path2OSideOp *ta=new Path2OSideOp();
   return ta;
 }
 
@@ -850,18 +1102,18 @@ int Path2OSideOp::run(ActionMessage *msg)
   char *dir=NULL;
   am=msg;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    l1=worker->getActiveLister();
+    l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
         if(lm1->isType("NormalMode")==true) {
           nm1=(NormalMode*)lm1;
-          dir=nm1->getActualDir();
+          dir = nm1->getCurrentDir();
         }
       }
     }
     if(dir!=NULL) {
-      l1=worker->getOtherLister(worker->getActiveLister());
+      l1 = msg->getWorker()->getOtherLister( msg->getWorker()->getActiveLister());
       if(l1!=NULL) {
         l1->switch2Mode(0);
         lm1=l1->getActiveMode();
@@ -902,25 +1154,26 @@ const char *QuitOp::getName()
   return name;
 }
 
-QuitOp::QuitOp(Worker *tworker):FunctionProto(tworker)
+QuitOp::QuitOp() : FunctionProto()
 {
   mode=Q_NORMAL_QUIT;
+  hasConfigure = true;
 }
 
 QuitOp::~QuitOp()
 {
 }
 
-QuitOp *QuitOp::duplicate()
+QuitOp *QuitOp::duplicate() const
 {
-  QuitOp *ta=new QuitOp(worker);
+  QuitOp *ta=new QuitOp();
   ta->mode=mode;
   return ta;
 }
 
 int QuitOp::run(ActionMessage *msg)
 {
-  worker->quit(mode==Q_QUICK_QUIT?1:0);
+  msg->getWorker()->quit( mode == Q_QUICK_QUIT ? 1 : 0 );
   return 0;
 }
 
@@ -931,7 +1184,7 @@ const char *QuitOp::getDescription()
 
 int QuitOp::configure()
 {
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   Button *okb,*cb;
   AWindow *win;
   CycleButton *rcyb;
@@ -996,6 +1249,7 @@ int QuitOp::configure()
   y+=okb->getHeight()+5;
   
   h=y;
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMaxSize(w,h);
   win->setMinSize(w,h);
@@ -1036,39 +1290,14 @@ int QuitOp::configure()
 
 bool QuitOp::save(Datei *fh)
 {
-  int size=Datei::getUCharSize();
-  fh->putInt(size);
-
+  if ( fh == NULL ) return false;
   switch(mode) {
     case Q_QUICK_QUIT:
-      fh->putUChar(1);
+      WConfig::configPutPair( fh, "mode", "quick" );
       break;
     default:
-      fh->putUChar(0);
-  }
-  return true;
-}
-
-int QuitOp::load(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  int cs=Datei::getUCharSize();
-  unsigned char c1;
-  if(chunksize>=cs) {
-    c1=fh->getUChar();
-    switch(c1) {
-      case 1:
-        mode=Q_QUICK_QUIT;
-        break;
-      default:
-        mode=Q_NORMAL_QUIT;
-        break;
-    }
-    chunksize-=cs;
-  }
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
+      WConfig::configPutPair( fh, "mode", "normal" );
+      break;
   }
   return true;
 }
