@@ -18,29 +18,29 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: normalops.h,v 1.5 2001/11/03 17:10:29 ralf Exp $ */
+/* $Id: normalops.h,v 1.9 2003/11/23 21:21:08 ralf Exp $ */
 
 #ifndef NORMALOPS_H
 #define NORMALOPS_H
 
+#include "wdefines.h"
 #include "functionproto.h"
-class Worker;
+#include <aguix/util.h>
 
 class ChangeListerSetOp:public FunctionProto
 {
 public:
-  ChangeListerSetOp(Worker*);
+  ChangeListerSetOp();
   virtual ~ChangeListerSetOp();
-  virtual ChangeListerSetOp *duplicate();
+  virtual ChangeListerSetOp *duplicate() const;
   virtual int configure();
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
   virtual const char *getDescription();
-  virtual int load(Datei*);
   virtual bool save(Datei *);
 
-  enum { CLS_LEFT_LISTER,CLS_RIGHT_LISTER,CLS_ACTUAL_LISTER,CLS_OTHER_LISTER};
+  enum { CLS_LEFT_LISTER,CLS_RIGHT_LISTER,CLS_CURRENT_LISTER,CLS_OTHER_LISTER};
   void setMode(int);
 protected:
   static const char *name;
@@ -50,9 +50,9 @@ protected:
 class SwitchListerOp:public FunctionProto
 {
 public:
-  SwitchListerOp(Worker*);
+  SwitchListerOp();
   virtual ~SwitchListerOp();
-  virtual SwitchListerOp *duplicate();
+  virtual SwitchListerOp *duplicate() const;
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
@@ -64,30 +64,39 @@ protected:
 class FilterSelectOp:public FunctionProto
 {
 public:
-  FilterSelectOp(Worker*);
+  FilterSelectOp();
   virtual ~FilterSelectOp();
-  virtual FilterSelectOp *duplicate();
+  virtual FilterSelectOp *duplicate() const;
   virtual int configure();
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
   virtual const char *getDescription();
-  virtual int load(Datei*);
   virtual bool save(Datei *);
   
   bool requestfilter();
+  void setFilter( const char *str );
 protected:
   static const char *name;
   char *filter;
   char *actual_filter;
+  
+  void addHistoryItem( const char *str );
+  static List *filter_history;
+private:
+  static int instance_counter;
+  static int history_size;
+  static int lastw, lasth;
+  void initHistory();
+  void closeHistory();
 };
 
 class FilterUnSelectOp:public FilterSelectOp
 {
 public:
-  FilterUnSelectOp(Worker*);
+  FilterUnSelectOp();
   virtual ~FilterUnSelectOp();
-  virtual FilterUnSelectOp *duplicate();
+  virtual FilterUnSelectOp *duplicate() const;
   virtual int configure();
   virtual bool isName(const char *);
   virtual const char *getName();
@@ -102,9 +111,9 @@ protected:
 class Path2OSideOp:public FunctionProto
 {
 public:
-  Path2OSideOp(Worker*);
+  Path2OSideOp();
   virtual ~Path2OSideOp();
-  virtual Path2OSideOp *duplicate();
+  virtual Path2OSideOp *duplicate() const;
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
@@ -116,15 +125,14 @@ protected:
 class QuitOp:public FunctionProto
 {
 public:
-  QuitOp(Worker*);
+  QuitOp();
   virtual ~QuitOp();
-  virtual QuitOp *duplicate();
+  virtual QuitOp *duplicate() const;
   virtual int configure();
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
   virtual const char *getDescription();
-  virtual int load(Datei*);
   virtual bool save(Datei *);
   
   void setMode(int);
