@@ -18,66 +18,28 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: ownop.cc,v 1.18 2002/03/18 18:57:49 ralf Exp $ */
+/* $Id: ownop.cc,v 1.29 2003/06/04 23:28:31 ralf Exp $ */
 
 #include "ownop.h"
 #include "listermode.h"
 #include "normalmode.h"
 #include "worker.h"
+#include "filereq.h"
 
 const char *OwnOp::name="OwnOp";
 
-struct {char *flag;
-        int catcode;} flags[]={{"{Rs{Infotext}{Defaulttext}}",0},
-                               {"{p}",1},
-                               {"{op}",2},
-                               {"{lp}",3},
-                               {"{rp}",4},
-                               {"{dp}",37},
-                               {"{f}",5},
-                               {"{F}",6},
-                               {"{a}",7},
-                               {"{A}",8},
-                               {"{fE}",9},
-                               {"{FE}",10},
-                               {"{aE}",11},
-                               {"{AE}",12},
-                               {"{uf}",13},
-                               {"{uF}",14},
-                               {"{ua}",15},
-                               {"{uA}",16},
-                               {"{ufE}",17},
-                               {"{uFE}",18},
-                               {"{uaE}",19},
-                               {"{uAE}",20},
-                               {"{of}",21},
-                               {"{oF}",22},
-                               {"{oa}",23},
-                               {"{oA}",24},
-                               {"{ofE}",25},
-                               {"{oFE}",26},
-                               {"{oaE}",27},
-                               {"{oAE}",28},
-                               {"{ouf}",29},
-                               {"{ouF}",30},
-                               {"{oua}",31},
-                               {"{ouA}",32},
-                               {"{oufE}",33},
-                               {"{ouFE}",34},
-                               {"{ouaE}",35},
-                               {"{ouAE}",36}};
-
-OwnOp::OwnOp(Worker *tworker):FunctionProto(tworker)
+OwnOp::OwnOp() : FunctionProto()
 {
-  withapo=false;
   separate_each_entry=false;
   recursive=false;
   inbackground = false;
+  take_dirs = false;
 
   ownstart=OWNOP_START_NORMAL;
 
   com_str=dupstring("");
   view_str=dupstring("");
+  hasConfigure = true;
 }
 
 OwnOp::~OwnOp()
@@ -87,13 +49,13 @@ OwnOp::~OwnOp()
 }
 
 OwnOp*
-OwnOp::duplicate()
+OwnOp::duplicate() const
 {
-  OwnOp *ta=new OwnOp(worker);
-  ta->withapo=withapo;
+  OwnOp *ta=new OwnOp();
   ta->separate_each_entry=separate_each_entry;
   ta->recursive=recursive;
   ta->inbackground = inbackground;
+  ta->take_dirs = take_dirs;
 
   ta->ownstart=ownstart;
 
@@ -117,15 +79,16 @@ OwnOp::getName()
 }
 
 int
-OwnOp::run(ActionMessage *msg)
+OwnOp::run( WPUContext *wpu, ActionMessage *msg )
 {
   ListerMode *lm1;
   am=msg;
+  use_wpu = wpu;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1=worker->getActiveLister();
+    Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       startlister=l1;
-      endlister=worker->getOtherLister(startlister);
+      endlister = msg->getWorker()->getOtherLister(startlister);
       lm1=startlister->getActiveMode();
       if(lm1!=NULL)
         if(lm1->isType("NormalMode")==true)
@@ -138,88 +101,29 @@ OwnOp::run(ActionMessage *msg)
 bool
 OwnOp::save(Datei *fh)
 {
-  int size=5*Datei::getUCharSize()+2*Datei::getIntSize();
-  size+=strlen(com_str);
-  size+=strlen(view_str);
-  fh->putInt(size);
+  if ( fh == NULL ) return false;
 
-  fh->putUChar((withapo==true)?1:0);
-  fh->putUChar((separate_each_entry==true)?1:0);
-  fh->putUChar((recursive==true)?1:0);
-  switch(ownstart) {
+  WConfig::configPutPairBool( fh, "separateeachentry", separate_each_entry );
+  WConfig::configPutPairBool( fh, "recursive", recursive );
+  switch ( ownstart ) {
     case OWNOP_START_IN_TERMINAL:
-      fh->putUChar(1);
+      WConfig::configPutPair( fh, "start", "terminal" );
       break;
     case OWNOP_START_IN_TERMINAL_AND_WAIT4KEY:
-      fh->putUChar(2);
+      WConfig::configPutPair( fh, "start", "terminalwait" );
       break;
     case OWNOP_SHOW_OUTPUT:
-      fh->putUChar(3);
+      WConfig::configPutPair( fh, "start", "showoutput" );
       break;
     default:
-      fh->putUChar(0);
+      WConfig::configPutPair( fh, "start", "normal" );
+      break;
   }
-  fh->putInt(strlen(com_str));
-  fh->putString(com_str);
-  fh->putInt(strlen(view_str));
-  fh->putString(view_str);
+  WConfig::configPutPairString( fh, "com", com_str );
+  WConfig::configPutPairString( fh, "viewstr", view_str );
   
-  fh->putUChar( ( inbackground == true ) ? 1 : 0 );
-  return true;
-}
-
-int
-OwnOp::load(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  int cs=Datei::getUCharSize(),
-      is=Datei::getIntSize(),
-      len;
-  unsigned char c1;
-  if(chunksize>=(4*cs+2*is)) {
-    if(fh->getUChar()==1) withapo=true; else withapo=false;
-    if(fh->getUChar()==1) separate_each_entry=true; else separate_each_entry=false;
-    if(fh->getUChar()==1) recursive=true; else recursive=false;
-    c1=fh->getUChar();
-    switch(c1) {
-      case 1:
-        ownstart=OWNOP_START_IN_TERMINAL;
-        break;
-      case 2:
-        ownstart=OWNOP_START_IN_TERMINAL_AND_WAIT4KEY;
-        break;
-      case 3:
-        ownstart=OWNOP_SHOW_OUTPUT;
-        break;
-      default:
-        ownstart=OWNOP_START_NORMAL;
-    }
-    chunksize-=4*cs;
-
-    len=fh->getInt();
-    chunksize-=is;
-    _freesafe(com_str);
-    com_str=fh->getString(len);
-    if(com_str==NULL) com_str=dupstring("");
-    chunksize-=len;
-
-    len=fh->getInt();
-    chunksize-=is;
-    _freesafe(view_str);
-    view_str=fh->getString(len);
-    if(view_str==NULL) view_str=dupstring("");
-    chunksize-=len;
-    
-    // new >2.3.1
-    if ( chunksize >= cs ) {
-      if ( fh->getUChar() == 1 ) inbackground = true; else inbackground = false;
-      chunksize -= cs;
-    }
-  }
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
-  }
+  WConfig::configPutPairBool( fh, "inbackground", inbackground );
+  WConfig::configPutPairBool( fh, "takedirs", take_dirs );
   return true;
 }
 
@@ -232,7 +136,7 @@ OwnOp::getDescription()
 int
 OwnOp::normalmodeown()
 {
-  struct NM_externorder extorder;
+  class NM_externorder extorder;
   NormalMode *nm1=NULL,
              *nm2=NULL;
   ListerMode *lm1=NULL;
@@ -261,6 +165,7 @@ OwnOp::normalmodeown()
     }
   }
   
+  extorder.wpu = use_wpu;
   extorder.destmode=NULL;
   if(am->mode==am->AM_MODE_ONLYACTIVE)
     extorder.source=extorder.NM_ONLYACTIVE;
@@ -283,10 +188,10 @@ OwnOp::normalmodeown()
   } else
     extorder.source=extorder.NM_ALLENTRIES;
 
-  extorder.withapo=withapo;
   extorder.separate_each_entry=separate_each_entry;
   extorder.recursive=recursive;
   extorder.inbackground = inbackground;
+  extorder.take_dirs = take_dirs;
   switch(ownstart) {
     case OWNOP_START_IN_TERMINAL:
       extorder.extstart=extorder.NM_EXT_START_IN_TERMINAL;
@@ -318,21 +223,25 @@ OwnOp::normalmodeown()
 int
 OwnOp::configure()
 {
-  AGUIX *aguix=worker->getAGUIX();
-  Button *okb,*cb,*fb;
+  AGUIX *aguix = Worker::getAGUIX();
+  Button *okb,*cb,*fb, *comreqb, *comreqb2;
   StringGadget *sg[2];
   AWindow *win;
-  ChooseButton *wacb,*scb,*rcb, *ibcb;
+  ChooseButton *scb,*rcb, *ibcb, *tdcb;
   CycleButton *cyb;
   Text *ttext;
   int tw,w,h,x,y,t1,t2;
   AGMessage *msg;
   int endmode=-1;
   Requester *req;
-  char *textstr,*buttonstr,*tstr;
+  FileRequester *freq;
+  const char *textstr;
+  char *buttonstr,*tstr;
   int erg;
+  std::string str1;
   
   req=new Requester(aguix);
+  freq = new FileRequester( aguix );
   w=h=10;
   x=y=5;
   win=new AWindow(aguix);
@@ -342,12 +251,6 @@ OwnOp::configure()
   win->create(NULL,10,10,w,h,0,tstr);
   _freesafe(tstr);
 
-  wacb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,(withapo==true)?1:0,
-                                                catalog.getLocale(328),LABEL_RIGHT,1,0));
-  y+=wacb->getHeight()+5;
-  tw=wacb->getWidth()+10;
-  if(tw>w) w=tw;
-
   scb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,(separate_each_entry==true)?1:0,
                                                catalog.getLocale(329),LABEL_RIGHT,1,0));
   y+=scb->getHeight()+5;
@@ -360,6 +263,12 @@ OwnOp::configure()
   tw=rcb->getWidth()+10;
   if(tw>w) w=tw;
 
+  tdcb = (ChooseButton*)win->add( new ChooseButton( aguix, x, y, ( take_dirs == true ) ? 1 : 0,
+                                                    catalog.getLocale( 513 ), LABEL_RIGHT, 1, 0 ) );
+  y += tdcb->getHeight() + 5;
+  tw = tdcb->getWidth() + 10;
+  if ( tw > w ) w = tw;
+
   ibcb = (ChooseButton*)win->add( new ChooseButton( aguix,
                                                     x,
                                                     y,
@@ -402,12 +311,16 @@ OwnOp::configure()
 
   ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(336),1));
   x+=ttext->getWidth()+5;
+  comreqb = (Button*)win->add( new Button( aguix, x, y, catalog.getLocale( 458 ), 1, 0, 0 ) );
+  x += comreqb->getWidth();
+
   tw=w-x-5;
-  if(tw<50) tw=50;
+  if ( tw < ( 20 * aguix->getCharWidth() ) ) tw = 20 * aguix->getCharWidth();
   sg[0]=(StringGadget*)win->add(new StringGadget(aguix,x,y,tw,com_str,0));
   
   fb=(Button*)win->add(new Button(aguix,x+sg[0]->getWidth(),y,sg[0]->getHeight(),
                                   sg[0]->getHeight(),"O",1,0,0));
+  comreqb->resize( comreqb->getWidth(), sg[0]->getHeight() );
   
   y+=sg[0]->getHeight()+5;
   tw=sg[0]->getWidth()+5+x;
@@ -415,9 +328,14 @@ OwnOp::configure()
   x=5;
   ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(337),1));
   x+=ttext->getWidth()+5;
+  comreqb2 = (Button*)win->add( new Button( aguix, x, y, catalog.getLocale( 458 ), 1, 0, 0 ) );
+  x += comreqb2->getWidth();
+  
   tw=w-x-5;
-  if(tw<50) tw=50;
+  if ( tw < ( 20 * aguix->getCharWidth() ) ) tw = 20 * aguix->getCharWidth();
   sg[1]=(StringGadget*)win->add(new StringGadget(aguix,x,y,tw,view_str,0));
+  comreqb2->resize( comreqb2->getWidth(), sg[1]->getHeight() );
+  
   y+=sg[1]->getHeight()+5;
   tw=sg[1]->getWidth()+5+x;
   if(tw>w) w=tw;
@@ -452,6 +370,7 @@ OwnOp::configure()
   fb->move(w-5-fb->getWidth(),fb->getY());
 
   h=y;
+  win->setDoTabCycling( true );
   win->resize(w,h);
   win->setMaxSize(w,h);
   win->setMinSize(w,h);
@@ -496,6 +415,48 @@ OwnOp::configure()
               _freesafe(newstr2);
               _freesafe(tstr);
             }
+          } else if ( msg->button.button == comreqb ) {
+            if ( freq->request_entry( catalog.getLocale( 265 ),
+                                      NULL,
+                                      catalog.getLocale( 11 ),
+                                      catalog.getLocale( 8 ),
+                                      catalog.getLocale( 265 ),
+                                      true ) > 0 ) {
+              str1 = freq->getLastEntryStr();
+              if ( str1.length() > 0 ) {
+                const char *newstr1 = sg[ 0 ]->getText();
+                char *newstr2;
+                int pos = sg[ 0 ]->getCursor();
+
+                newstr2 = (char*)_allocsafe( strlen( newstr1 ) + strlen( str1.c_str() ) + 1 );
+                strncpy( newstr2, newstr1, pos );
+                strcpy( newstr2 + pos, str1.c_str() );
+                strcpy( newstr2 + pos + strlen( str1.c_str() ), newstr1 + pos );
+                sg[ 0 ]->setText( newstr2 );
+                _freesafe( newstr2 );
+              }
+            }
+          } else if ( msg->button.button == comreqb2 ) {
+            if ( freq->request_entry( catalog.getLocale( 265 ),
+                                      NULL,
+                                      catalog.getLocale( 11 ),
+                                      catalog.getLocale( 8 ),
+                                      catalog.getLocale( 265 ),
+                                      true ) > 0 ) {
+              str1 = freq->getLastEntryStr();
+              if ( str1.length() > 0 ) {
+                const char *newstr1 = sg[ 1 ]->getText();
+                char *newstr2;
+                int pos = sg[ 1 ]->getCursor();
+
+                newstr2 = (char*)_allocsafe( strlen( newstr1 ) + strlen( str1.c_str() ) + 1 );
+                strncpy( newstr2, newstr1, pos );
+                strcpy( newstr2 + pos, str1.c_str() );
+                strcpy( newstr2 + pos + strlen( str1.c_str() ), newstr1 + pos );
+                sg[ 1 ]->setText( newstr2 );
+                _freesafe( newstr2 );
+              }
+            }
           }
           break;
 	case AG_STRINGGADGET_DEACTIVATE:
@@ -525,9 +486,9 @@ OwnOp::configure()
   
   if(endmode==0) {
     // ok
-    withapo=(wacb->getState()==1)?true:false;
     separate_each_entry=(scb->getState()==1)?true:false;
     recursive=(rcb->getState()==1)?true:false;
+    take_dirs = ( tdcb->getState() == 1 ) ? true : false;
     inbackground = ( ibcb->getState() == 1 ) ? true : false;
     _freesafe(com_str);
     _freesafe(view_str);
@@ -552,6 +513,7 @@ OwnOp::configure()
   win->close();
   delete win;
   delete req;
+  delete freq;
 
   return endmode;
 }
@@ -559,7 +521,7 @@ OwnOp::configure()
 char*
 OwnOp::getFlag()
 {
-  AGUIX *aguix=worker->getAGUIX();
+  AGUIX *aguix = Worker::getAGUIX();
   Button *okb,*cb;
   AWindow *win;
   FieldListView *lv;
@@ -569,7 +531,7 @@ OwnOp::getFlag()
   int endmode=-1;
   char *returnstr;
   Button *b2[2];
-  int i, trow;
+  int i, trow, nr;
   
   w=400;
   h=300;
@@ -593,10 +555,12 @@ OwnOp::getFlag()
   lv->setNrOfFields( 3 );
   lv->setFieldWidth( 1, 3 );
   
-  for( i=0;i<CATFLAGENTRY;i++) {
+  nr = sizeof( ownop_flags ) / sizeof( ownop_flags[0] );
+
+  for( i=0;i<nr;i++) {
     trow = lv->addRow();
-    lv->setText( trow, 0, flags[ i ].flag );
-    lv->setText( trow, 2, catalog.getLocaleFlag( flags[ i ].catcode ) );
+    lv->setText( trow, 0, ownop_flags[ i ].flag );
+    lv->setText( trow, 2, catalog.getLocaleFlag( ownop_flags[ i ].catcode ) );
     lv->setPreColors( trow, FieldListView::PRECOLOR_ONLYACTIVE );
     lv->setData( trow, i );
   }
@@ -629,6 +593,7 @@ OwnOp::getFlag()
   b2[1] = cb;
   
   h=y;
+  win->setDoTabCycling( true );
   win->resize(w,h);
   //win->setMaxSize(w,h);
   win->setMinSize(w,h);
@@ -645,7 +610,7 @@ OwnOp::getFlag()
           else if(msg->button.button==cb) endmode=1;
           break;
         case AG_SIZECHANGED:
-          AGUIX::scaleElementsW( msg->size.neww, 5, 10, -1, false, false, (GUIElement**)b2, 2 );
+          AGUIX::scaleElementsW( msg->size.neww, 5, 10, -1, false, false, (GUIElement**)b2, NULL, 2 );
           for ( i = 0; i < 2; i++ ) {
             b2[i]->move( b2[i]->getX(), msg->size.newh - 5 - b2[i]->getHeight() );
           }
@@ -661,8 +626,8 @@ OwnOp::getFlag()
     // ok
     trow = lv->getActiveRow();
     if ( lv->isValidRow( trow ) == true ) {
-      int nr = lv->getData( trow );
-      returnstr = dupstring( flags[nr].flag );
+      nr = lv->getData( trow );
+      returnstr = dupstring( ownop_flags[nr].flag );
     }
   }
   
@@ -672,11 +637,6 @@ OwnOp::getFlag()
   return returnstr;
 }
 
-void OwnOp::setWithApo(bool nv)
-{
-  withapo=nv;
-}
-
 void OwnOp::setSeparateEachEntry(bool nv)
 {
   separate_each_entry=nv;
@@ -694,12 +654,25 @@ void OwnOp::setOwnStart(ownstart_t nv)
 
 void OwnOp::setComStr(const char* nv)
 {
+  if ( nv == NULL ) return;
   _freesafe(com_str);
   com_str=dupstring(nv);
 }
 
+void OwnOp::setViewStr( const char* nv )
+{
+  if ( nv == NULL ) return;
+  _freesafe( view_str );
+  view_str = dupstring( nv );
+}
+
 void OwnOp::setInBackground( bool nv )
 {
   inbackground = nv;
 }
 
+void OwnOp::setTakeDirs( bool nv )
+{
+  take_dirs = nv;
+}
+
