@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: renameop.h,v 1.5 2001/11/03 17:10:38 ralf Exp $ */
+/* $Id: renameop.h,v 1.6 2003/05/04 01:09:53 ralf Exp $ */
 
 #ifndef RENAMEOP_H
 #define RENAMEOP_H
@@ -26,14 +26,12 @@
 #include "wdefines.h"
 #include "functionproto.h"
 
-class Worker;
-
 class RenameOp:public FunctionProto
 {
 public:
-  RenameOp(Worker*);
+  RenameOp();
   virtual ~RenameOp();
-  virtual RenameOp *duplicate();
+  virtual RenameOp *duplicate() const;
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
