@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: searchentryop.h,v 1.3 2001/11/03 17:10:41 ralf Exp $ */
+/* $Id: searchentryop.h,v 1.5 2003/10/16 21:11:18 ralf Exp $ */
 
 #ifndef SEARCHENTRYOP_H
 #define SEARCHENTRYOP_H
@@ -26,21 +26,23 @@
 #include "wdefines.h"
 #include "functionproto.h"
 
-class Worker;
-
 class SearchEntryOp:public FunctionProto
 {
 public:
-  SearchEntryOp(Worker*);
+  SearchEntryOp();
   virtual ~SearchEntryOp();
-  virtual SearchEntryOp *duplicate();
+  virtual SearchEntryOp *duplicate() const;
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
   virtual const char *getDescription();
+  virtual bool save(Datei*);
+  void setIgnoreCase( bool );
+  virtual int configure();
 protected:
   static const char *name;
   // Infos to save
+  bool ignore_case;
 
   // temp variables
   Lister *startlister;
