@@ -18,32 +18,36 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: setfilterop.h,v 1.3 2001/11/03 17:10:42 ralf Exp $ */
+/* $Id: setfilterop.h,v 1.5 2003/05/04 01:14:13 ralf Exp $ */
 
 #ifndef SETFILTEROP_H
 #define SETFILTEROP_H
 
 #include "wdefines.h"
 #include "functionproto.h"
+#include <aguix/util.h>
 
 class Worker;
 
 class SetFilterOp:public FunctionProto
 {
 public:
-  SetFilterOp(Worker*);
+  SetFilterOp();
   virtual ~SetFilterOp();
-  virtual SetFilterOp *duplicate();
+  virtual SetFilterOp *duplicate() const;
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
   virtual bool save(Datei*);
-  virtual int load(Datei*);
   virtual const char *getDescription();
   virtual int configure();
 
   int doconfigure(int mode);
   typedef enum {INCLUDE_FILTER=0,EXCLUDE_FILTER,UNSET_FILTER,UNSET_ALL} setfilterop_mode;
+  
+  void setRequestFlags( bool );
+  void setFilter( const char * );
+  void setFiltermode( setfilterop_mode );
 protected:
   static const char *name;
   // Infos to save
@@ -59,6 +63,14 @@ protected:
   setfilterop_mode tfiltermode;
 
   int normalmodesf();
+
+  void addHistoryItem( const char *str );
+  static List *filter_history;
+private:
+  static int instance_counter;
+  static int history_size;
+  void initHistory();
+  void closeHistory();
 };
 
 #endif
