@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: simddop.h,v 1.4 2001/11/03 17:10:46 ralf Exp $ */
+/* $Id: simddop.h,v 1.5 2003/05/04 01:17:59 ralf Exp $ */
 
 #ifndef SIMDDEOP_H
 #define SIMDDOP_H
@@ -26,14 +26,12 @@
 #include "wdefines.h"
 #include "functionproto.h"
 
-class Worker;
-
 class SimDDOp:public FunctionProto
 {
 public:
-  SimDDOp(Worker*);
+  SimDDOp();
   virtual ~SimDDOp();
-  virtual SimDDOp *duplicate();
+  virtual SimDDOp *duplicate() const;
   virtual bool isName(const char *);
   virtual const char *getName();
   virtual int run(ActionMessage*);
