@@ -1,6 +1,6 @@
 /* wconfig.h
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001 Ralf Hoffmann.
+ * Copyright (C) 2001-2004 Ralf Hoffmann.
  * You can contact me at: ralf.hoffmann@epost.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: wconfig.h,v 1.28 2002/02/28 00:05:59 ralf Exp $ */
+/* $Id: wconfig.h,v 1.47 2004/03/07 22:35:46 ralf Exp $ */
 
 #ifndef WCONFIG_H
 #define WCONFIG_H
@@ -28,13 +28,16 @@
 #include <aguix/awindow.h>
 #include <aguix/stringgadget.h>
 #include <aguix/bevelbox.h>
-#include <aguix/listview.h>
+#include <aguix/fieldlistview.h>
 #include <aguix/solidbutton.h>
 #include <aguix/choosebutton.h>
 #include <aguix/cyclebutton.h>
-#include "locale.h"
+#include "worker_locale.h"
 #include "functionproto.h"
 #include "datei.h"
+#include "worker_types.h"
+#include <vector>
+#include <list>
 
 class Worker;
 class WCPath;
@@ -47,11 +50,16 @@ class WC_Color
 {
 public:
   WC_Color();
-  WC_Color(int,int,int);
+  WC_Color( int r, int g, int b);
   WC_Color *duplicate();
-  void save(Datei*);
-  bool load(Datei *);
-  int red,green,blue;
+  int getRed() const;
+  int getGreen() const;
+  int getBlue() const;
+  void setRed( int );
+  void setGreen( int );
+  void setBlue( int );
+protected:
+  int red, green, blue;
 };
 
 class WCShortkey
@@ -65,14 +73,37 @@ public:
   KeySym getShortkey();
   unsigned int getMod();
   bool isShortkey(KeySym,unsigned int);
-  bool save(Datei*);
-  bool load(Datei*);
   bool isReal();
 protected:
   KeySym shortkey;
   unsigned int mod;
 };
 
+class WCDoubleShortkey
+{
+public:
+  WCDoubleShortkey();
+  ~WCDoubleShortkey();
+  WCDoubleShortkey *duplicate() const;
+  void setKeySym( KeySym k, int pos );
+  KeySym getKeySym( int pos );
+  void setMod( unsigned int m, int pos);
+  unsigned int getMod( int pos );
+  bool isShortkey( KeySym k, unsigned int m );
+  bool isShortkey( KeySym k1, unsigned int m1, KeySym k2, unsigned int m2 );
+  bool save( Datei* );
+  bool isReal();
+
+  enum shortkey_type { WCDS_NORMAL, WCDS_DOUBLE };
+
+  void setType( enum shortkey_type nv );
+  enum shortkey_type getType();
+protected:
+  KeySym key[2];
+  unsigned int mod[2];
+  enum shortkey_type type;
+};
+
 class WConfig
 {
 public:
@@ -81,7 +112,7 @@ public:
   WConfig(Worker *);
   ~WConfig();
   WConfig *duplicate();
-  void setLang(char*);
+  void setLang( const char* );
   void setTerminalBin(const char*);
   void setRows(unsigned int);
   void setColumns(unsigned int);
@@ -90,7 +121,7 @@ public:
   void setVBarLeft(int,bool);
   void setHBarHeight(int,unsigned int);
   void setVBarWidth(int,unsigned int);
-  void setDisplay(int,int*);
+  void setVisCols( int side, const std::vector<WorkerTypes::listcol_t> *nv );
   char *getLang();
   char *getTerminalBin();
   unsigned int getRows();
@@ -100,7 +131,7 @@ public:
   bool getVBarLeft(int);
   unsigned int getHBarHeight(int);
   unsigned int getVBarWidth(int);
-  int *getDisplay(int);
+  const std::vector<WorkerTypes::listcol_t> *getVisCols( int side ) const;
   void setStatusbar(int,int);
   void setSelLVB(int,int);
   void setUnselLVB(int,int);
@@ -114,19 +145,21 @@ public:
   void setUnselFileAct(int,int);
   void setClockbar(int,int);
   void setRequest(int,int);
-  int getStatusbar(int);
-  int getSelLVB(int);
-  int getUnselLVB(int);
-  int getSelDir(int);
-  int getUnselDir(int);
-  int getSelFile(int);
-  int getUnselFile(int);
-  int getSelDirAct(int);
-  int getUnselDirAct(int);
-  int getSelFileAct(int);
-  int getUnselFileAct(int);
-  int getClockbar(int);
-  int getRequest(int);
+  void setLVHeader( int, int );
+  int getStatusbar(int) const;
+  int getSelLVB(int) const;
+  int getUnselLVB(int) const;
+  int getSelDir(int) const;
+  int getUnselDir(int) const;
+  int getSelFile(int) const;
+  int getUnselFile(int) const;
+  int getSelDirAct(int) const;
+  int getUnselDirAct(int) const;
+  int getSelFileAct(int) const;
+  int getUnselFileAct(int) const;
+  int getClockbar(int) const;
+  int getRequest(int) const;
+  int getLVHeader( int ) const;
   void setStartDir(int,const char*);
   char *getStartDir(int);
   void setFont(int,const char*);
@@ -141,6 +174,8 @@ public:
   List *getButtons();
   void setColors(List*);
   List *getColors();
+  void setColor( int nr, int red, int green, int blue );
+  void resetColors();
   void cleanUpPaths();
   void cleanUpButtons();
   void setOwnerstringtype(int);
@@ -167,11 +202,12 @@ public:
   bool saveas(char*);
   bool load();
   bool loadfrom(char*);
-  int getMaxFiletypeLen();
   
   const char *getHome();
   WCFiletype *getnotyettype();
   WCFiletype *getdirtype();
+  WCFiletype *getvoidtype();
+  WCFiletype *getunknowntype();
   void applyLanguage();
   
   int savesize(int w,int h);
@@ -194,9 +230,47 @@ public:
   const char *getTimeFormatString();
   void setDateBeforeTime( bool nv );
   bool getDateBeforeTime();
+  
+  void setShowHeader( int side, bool nv );
+  bool getShowHeader( int side ) const;
 
   int getDatelen( const struct tm *timeptr );
   int writeDateToString( char *buf, int maxlen, struct tm *timeptr );
+  
+  bool checkForNewerConfig();
+  time_t getFileConfigMod();
+  
+  void clearVisCols( int side );
+  void addListCol( int side, WorkerTypes::listcol_t nv );
+  
+  static void configPutPair( Datei *fh, const char *id, const char *info );
+  static void configPutPairNum( Datei *fh, const char *id, int num );
+  static void configPutPairNum( Datei *fh, const char *id, int num1, int num2 );
+  static void configPutPairString( Datei *fh, const char *id, const char *str );
+  static void configPutPairBool( Datei *fh, const char *id, bool val );
+  static void configPutInfo( Datei *fh, const char *str, bool semicolon );
+  static void configPutString( Datei *fh, const char *str, bool semicolon );
+  static void configOpenSection( Datei *fh, const char *sectioname );
+  static void configCloseSection( Datei *fh );
+
+  typedef enum { LO_STATEBAR,
+                 LO_CLOCKBAR,
+                 LO_BUTTONS,
+                 LO_LISTVIEWS,
+                 LO_BLL,
+                 LO_LBL,
+                 LO_LLB,
+                 LO_BL,
+                 LO_LB,
+                 LO_DUMMY } layoutID_t;
+  void setLayoutOrders( std::list<layoutID_t> &nl );
+  std::list<layoutID_t> &getLayoutOrders();
+  void setLayoutButtonVert( bool nv );
+  bool getLayoutButtonVert() const;
+  void setLayoutListviewVert( bool nv );
+  bool getLayoutListviewVert() const;
+  void clearLayoutOrders();
+  void layoutAddEntry( layoutID_t nv );
 protected:
   char *lang;
   unsigned int rows,columns;
@@ -206,7 +280,7 @@ protected:
   bool vbar_left[2];
   unsigned int hbar_height[2];
   unsigned int vbar_width[2];
-  int display[2][9];   /* Name,Size,Type,Perm,Owner,Dest,Mod,Acc,Change */
+  std::vector<WorkerTypes::listcol_t> viscols[2];
   int statusbar[2];
   int sel_lvb[2];
   int unsel_lvb[2];
@@ -220,6 +294,7 @@ protected:
   int unsel_file_act[2];
   int clockbar[2];
   int request[2];
+  int lvheader[2];
   char *dir[2];  // left=0, right=1
   char *fonts[4];  // global, buttons, left lister, right lister
   List *paths;  // Elemente sind WCPath
@@ -231,6 +306,12 @@ protected:
   char *clockbar_command;
   List *dontcheckdirs;
   
+  struct {
+    std::list<layoutID_t> orders;
+    bool buttonVert;
+    bool listviewVert;
+  } layoutConf;
+  
   char *terminalbin;
   
   int ownerstringtype;
@@ -238,8 +319,8 @@ protected:
   bool showStringForDirSize;
   char *stringForDirSize;
   int stringForDirSizeLen;
+  bool showHeader[2];
   
-  int maxfiletypelen;
   #define NORMALTYPE 0
   #define NOTYETTYPE 1
   #define UNKNOWNTYPE 2
@@ -247,6 +328,8 @@ protected:
   #define DIRTYPE 4
   WCFiletype *notyettype; // Nur zum schnelleren Auslesen, muss nicht abgespeichert werden
   WCFiletype *dirtype; // Nur zum schnelleren Auslesen, muss nicht abgespeichert werden
+  WCFiletype *voidtype; // Nur zum schnelleren Auslesen, muss nicht abgespeichert werden
+  WCFiletype *unknowntype; // Nur zum schnelleren Auslesen, muss nicht abgespeichert werden
   
   char *home;
   Worker *worker;
@@ -266,25 +349,35 @@ protected:
   bool changeFileSets();
   bool changeDisplaySets(int);
   bool changeClockbarSets();
+  bool configureLayout();
+
   void showPathBank(int,Button**);
-  void showButtonBank(int,Button*[][2]);
+  void showButtonBank( int, Button***);
   
   bool configurePath(WCPath*);
-  int getShortkey(const WCShortkey *ignore_sk, WCShortkey **return_sk);
+  int grabKey( const char *title, const char *text, KeySym *return_key, unsigned int *return_mod );
+  int getDoubleShortkey( bool doublek,
+                         const WCButton *ignore_b,
+                         const WCHotkey *ignore_h,
+                         const WCPath *ignore_p,
+                         WCDoubleShortkey **return_dk);
+  int findDoubleShortkey( WCDoubleShortkey *dk,
+                          WCButton **return_button,
+                          WCPath **return_path,
+                          WCHotkey **return_hotkey );
   bool configureHotkey(WCHotkey*);
   FunctionProto *requestCommand();
   static int requestCommand_sf(void*,void*);
 
   bool configureButton(WCButton*);
   bool configureFiletype(WCFiletype*);
-  void showFTActions(char*,WCFiletype*,ListView*,ListView*,List**,WCGlobalFlags**);
-  void applyGlobalFlags(ListView *,WCGlobalFlags *);
+  void showFTActions( const char*, WCFiletype*, FieldListView*, FieldListView*, List**, WCGlobalFlags** );
+  void applyGlobalFlags( FieldListView *, WCGlobalFlags * );
   bool changeFiledesc(WCFiletype*);
   char *getStr4Char(int,short);
   int WCEditByte(int ch);
   bool FTautocreate(WCFiletype *);
-  void recalcMaxFiletypeLen();
-  void initFixTypes(List*);
+  void initFixTypes();
   void setDefaultConfig();
   int configureDCD(List *);
 
@@ -299,7 +392,6 @@ protected:
   void changeRS(const char *,char **);
   bool showOldRequest;
   
-  int findShortkey(WCShortkey*,WCButton**,WCPath**,WCHotkey**);
   static bool isCorrectTerminalBin(const char*);
   int listerbg;
   
@@ -324,9 +416,54 @@ protected:
   
   bool date_before_time;
   int configureDateTime();
+  
+  int configureButtonSize[2];
+  int changeTypesSize[2];
+  int changeHotkeysSize[2];
+  int configureFiletypeSize[2];
+  int configureHotkeySize[2];
+  char *lastRequestCommand;
+  time_t currentConfigTime;
+  
+  bool loadBinConfig( const char * );
+  int loadBinColor( Datei *fh, WC_Color *col );
+  int loadBinPath( Datei *fh, WCPath *p1 );
+  int loadBinFiletype( Datei *fh, WCFiletype *f1 );
+  int loadBinHotkey( Datei *fh, WCHotkey *h1 );
+  int loadBinButton( Datei *fh, WCButton *b1 );
+  int loadBinShortkey( Datei *fh, WCShortkey *key );
+  int loadBinDoubleShortkey( Datei *fh, WCDoubleShortkey *dk );
+  int loadBinGlobalFlags( Datei *fh );
+  List *loadBinFunction( Datei *fh );
+  
+  static int sectiondepth;
 };
 
-class WCPath
+class WCKey
+{
+public:
+  List *getDoubleKeys();
+  void setDoubleKeys( List *dklist );
+  bool hasKey( KeySym k, unsigned int m );
+  bool hasKey( KeySym k1, unsigned int m1, KeySym k2, unsigned int m2 );
+  void removeKey( KeySym k, unsigned int m );
+  void removeKey( KeySym k1, unsigned int m1, KeySym k2, unsigned int m2 );
+  void removeKey( WCDoubleShortkey *dk );
+  bool conflictKey( KeySym k, unsigned int m );
+  bool conflictKey( KeySym k1, unsigned int m1, KeySym k2, unsigned int m2 );
+  bool conflictKey( WCDoubleShortkey *dk );
+  void addKey( const KeySym k, const unsigned int m );
+  void addKey( const KeySym k1, const unsigned int m1, const KeySym k2, const unsigned int m2 );
+  void addKey( const WCDoubleShortkey *dk );
+protected:
+  List *dkeys;    // list of WCDoubleShortkey
+  
+  void removeAllKeys();
+  WCKey();
+  ~WCKey();
+};
+
+class WCPath:public WCKey
 {
 public:
   WCPath();
@@ -340,18 +477,14 @@ public:
   int getFG();
   int getBG();
   char *getPath();
-  WCShortkey *getKey();
-  void setKey(WCShortkey*);
   void setCheck(bool);
   bool getCheck();
   bool configure(AGUIX*);
   bool save(Datei*);
-  bool load(Worker*,Datei*);
 protected:
   char *name;
   int fg,bg;
   char *path;
-  WCShortkey *key;
   bool check;
 };
 
@@ -364,7 +497,6 @@ public:
   unsigned long getFlags();
   void setFlags(unsigned long);
   bool save(Datei*);
-  bool load(Datei*);
 
   static List* getNameOfFlags();
   static void freeNameOfFlags(List*);
@@ -394,10 +526,15 @@ public:
   List *getRawShowActions();
   List *getUserActions(int);
   void setDNDActions(List*);
+  void addComDNDAction( const FunctionProto * );
   void setDoubleClickActions(List*);
+  void addComDoubleClickAction( const FunctionProto * );
   void setShowActions(List*);
+  void addComShowAction( const FunctionProto * );
   void setRawShowActions(List*);
+  void addComRawShowAction( const FunctionProto * );
   void setUserActions(int,List*);
+  void addComUserAction( int i, const FunctionProto * fp );
   WCGlobalFlags *getDNDActionsGF();
   WCGlobalFlags *getDoubleClickActionsGF();
   WCGlobalFlags *getShowActionsGF();
@@ -409,7 +546,6 @@ public:
   void setRawShowActionsGF(WCGlobalFlags*);
   void setUserActionsGF(int,WCGlobalFlags*);
   bool save(Datei*);
-  bool load(Worker*,Datei*);
   int getinternID();
   void setinternID(int);
   int getNameLen();
@@ -419,6 +555,18 @@ public:
   void setUseFileExp( bool );
   bool getUseFileExp();
   bool fileExpMatchString( const char * );
+  void setPatternIgnoreCase( bool nv );
+  bool getPatternIgnoreCase();
+  void setPatternUseRegExp( bool nv );
+  bool getPatternUseRegExp();
+  bool patternMatchString( const char * );
+  void setPatternUseFullname( bool nv );
+  bool getPatternUseFullname() const;
+  
+  static int sortfunction( void *, void *, int );
+  
+  void clearFiledesc();
+  void setFiledesc( int pos, short v );
 protected:
   char *name;
   char *pattern;
@@ -447,9 +595,19 @@ protected:
   void free_compexp();
   void compile_exp();
 #endif
+
+  bool patternIgnoreCase;
+  bool patternUseRegExp;
+  bool patternUseFullname;
+#ifdef HAVE_REGEX
+  regex_t patternReg;
+  bool patternRegCompiled;
+  void freeCompPatternReg();
+  void compilePatternReg();
+#endif
 };
 
-class WCHotkey
+class WCHotkey:public WCKey
 {
 public:
   WCHotkey();
@@ -459,12 +617,12 @@ public:
   char *getName();
   WCShortkey *getKey();
   List *getComs();
+  void addCom( const FunctionProto *fp );
   void setKey(WCShortkey*);
   void setComs(List*);
   WCGlobalFlags *getComsGF();
   void setComsGF(WCGlobalFlags*);
   bool save(Datei*);
-  bool load(Worker*,Datei*);
 protected:
   char *name;
   WCShortkey *key;
@@ -472,7 +630,7 @@ protected:
   WCGlobalFlags *flags;
 };
 
-class WCButton
+class WCButton:public WCKey
 {
 public:
   WCButton();
@@ -484,20 +642,17 @@ public:
   char *getText();
   int getFG();
   int getBG();
-  WCShortkey *getKey();
-  void setKey(WCShortkey*);
   List *getComs();
   void setComs(List*);
+  void addCom( const FunctionProto *fp );
   void setCheck(bool);
   bool getCheck();
   WCGlobalFlags *getComsGF();
   void setComsGF(WCGlobalFlags*);
   bool save(Datei*);
-  bool load(Worker*,Datei*);
 protected:
   char *text;
   int fg,bg;
-  WCShortkey *key;
   List *com;
   WCGlobalFlags *flags;
   bool check;
