@@ -18,18 +18,38 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: wdefines.h,v 1.18 2002/02/21 00:11:49 ralf Exp $ */
+/* $Id: wdefines.h,v 1.33 2003/11/26 19:22:25 ralf Exp $ */
 
 #ifndef WDEFINES_H
 #define WDEFINES_H
 
+#ifndef CONFIG_H_INCLUDED
+#  include <aguix/aguixconfig.h>
+#  define CONFIG_H_INCLUDED
+#endif
+
+#include "worker_lfs.h"
+
+/* this following include isn't really needed at all
+ * but the streambuf in (some?) stl implementations
+ * undefing open/close (for whatever reason)
+ * and this leads to problem on system supporting
+ * large files with open to open64 define
+ * including vector from stl (not streambuf directly
+ * because it's not available everywhere) here should
+ * avoid such problems but open in streambuf is not
+ * accessable anymore
+ * TODO: a better solution is highly wanted
+ */
+#include <vector>
+
+#include <unistd.h>
 #include <time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <fcntl.h>
 #include <sys/param.h>  /* for MAXPATHLEN ... */
 #include <sys/stat.h>
-#include <unistd.h>
 #include <dirent.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,27 +64,23 @@
 #include <ctype.h>
 #include <utime.h>
 
-#ifdef HAVE_CONFIG_H
-  #ifndef CONFIG_H_INCLUDED
-    #include "../config.h"
-    #define CONFIG_H_INCLUDED
-  #endif
-#endif
-
 #if defined(HAVE_SYS_STATVFS_H)
   #include <sys/statvfs.h>
-#elif ( defined(BSD) && (BSD >= 199506) )
-  /* statfs on FreeBSD-2.2.2, BSD is defined in sys/param.h */
-  #include <sys/mount.h>
 #endif
 #ifdef HAVE_SYS_VFS_H
   #include <sys/vfs.h>
 #endif
+#ifdef HAVE_SYS_PARAM_H
+  #include <sys/param.h>
+#endif
+#ifdef HAVE_SYS_MOUNT_H
+  #include <sys/mount.h>
+#endif
 #ifdef HAVE_SYS_SYSINFO_H
   #include <sys/sysinfo.h>
 #endif
 
-#if 0  // for filetype checking with file
+#ifdef HAVE_REGEX
 #include <regex.h>
 #endif
 
@@ -74,8 +90,10 @@
   #include <zlib.h>
 #endif
 
-#define CP_COMMAND "cp -a %s %s"
-#define CP_COMMAND_FORCE "cp -a -f %s %s"
+#ifdef HAVE_LIBBZ2
+  #include <bzlib.h>
+#endif
+
 #define OUTPUT_BIN "xmessage -file %s"
 #define TERMINAL_BIN "xterm -e %s"
 
@@ -99,13 +117,6 @@
   #define USEOWNCONFIGFILES
 #endif
 
-#ifndef PREFIX
-  #define GLOBALDATADIR "/usr/local/share/worker"
-#else
-  #define GLOBALDATADIR PREFIX"/share/worker"
-#endif
-
-#define max(a,b) (((a)>(b))?(a):(b))
 #define w_max(a,b) (((a)>(b))?(a):(b))
 #define w_min(a,b) (((a)<(b))?(a):(b))
 
