@@ -1,6 +1,6 @@
 /* worker.h
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001 Ralf Hoffmann.
+ * Copyright (C) 2001,2004 Ralf Hoffmann.
  * You can contact me at: ralf.hoffmann@epost.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: worker.h,v 1.26 2002/03/10 21:55:38 ralf Exp $ */
+/* $Id: worker.h,v 1.37 2004/02/26 22:54:29 ralf Exp $ */
 
 #ifndef WORKER_H
 #define WORKER_H
@@ -36,6 +36,7 @@
 #include "wconfig.h"
 #include "dnd.h"
 #include "partspace.h"
+#include <aguix/acontainer.h>
 
 class ListerMode;
 class Lister;
@@ -46,12 +47,14 @@ public:
   Worker(int argc,char **argv);
   ~Worker();
   void run();
-  AGUIX *getAGUIX();
-  int getMaxModeNr();
+  static AGUIX *getAGUIX();
+  static int getMaxModeNr();
   static int getID4Mode(ListerMode*);
+  static const char *getNameOfMode( int nr );
+  static const char *getLocaleNameOfMode( int nr );
+  static int getID4Name( const char *str );
   ListerMode *getMode4ID(int,Lister *);
   AWindow *getMainWin();
-  void getGeometry(Lister*,int*,int*,int*,int*);
   void setStatebarText(const char*);
   static int getNrOfCommands();
   FunctionProto *getCommand4ID(int);
@@ -61,6 +64,7 @@ public:
   Lister *getLister(int);
   Lister *getActiveLister();
   int interpret(List*,ActionMessage *);
+  AWindow *getListerWin( Lister *l );
   
   int quit(int mode);
   void checkfirststart( char **setlang );
@@ -71,43 +75,56 @@ public:
 
   int PS_readSpace( const char*name );
   
-  long PS_getBlocksize();
-  long PS_getFreeSpace();
-  long PS_getSpace();
+  loff_t PS_getBlocksize();
+  loff_t PS_getFreeSpace();
+  loff_t PS_getSpace();
   
-  long PS_getFreeSpaceH(char **unit_return);
-  long PS_getSpaceH(char **unit_return);
+  loff_t PS_getFreeSpaceH(char **unit_return);
+  loff_t PS_getSpaceH(char **unit_return);
+  void PS_setLifetime( double t );
+  void showLicense();
+  void setWaitCursor();
+  void unsetWaitCursor();
+  static const std::string getDataDir();
 protected:
-  void buildMainWin(bool);
-  void resizeMainWin(int,int);
+  void createMainWin();
+  void setupMainWin();
+  void closeMainWin();
   void shufflePath();
   void shuffleButton( int dir );
+  void shuffleButton( int dir, bool nocycle );
   void about();
   bool configure();
   void showPathBank();
   void showButtonBank();
   void setPath(int);
+  void setPath( WCPath *);
   void activateShortkey(AGMessage*);
   void activateButton(int,int,int);
   int saveListerState();
   int initListerState();
   void updateTime();
 
-  AGUIX *aguix;
+  static AGUIX *aguix;
+  static int workerInstances;
   Lister *lister[2];
   AWindow *mainwin;
   Button *aboutb,*configureb,*clockbar;
   SolidButton *statebar;
   Button **pathbs;
   Button **buttons;
+  AWindow *listerwin[2];
+  AContainer *maincont;
+  
   int w,h;
-  unsigned int oldrows,oldcolumns;
+  unsigned int curRows,curColumns;
   int pbanknr,bbanknr;
   
   enum {WORKER_NORMAL,WORKER_QUIT};
   int runningmode;
-  int argc;
-  char **argv;
+  static int argc;
+  static char **argv;
+  static std::string startcwd;
   time_t lasttimep;
   
   DNDMsg *dm;
@@ -119,6 +136,29 @@ protected:
                            WCButton *b;
                            WCPath *p;};
   PartSpace *freesp;
+  bool isRoot;
+  
+  KeySym lastkey;
+  unsigned int lastmod;
+  
+  typedef struct _keyhash {
+    WCButton *b;
+    WCHotkey *h;
+    WCPath *p;
+    struct _keyhash *next;
+  } keyhash;
+  keyhash **keyhashtable;
+  int keyhashtable_size;
+  void buildtable();
+  void deletetable();
+  unsigned int keyhashfunc( KeySym k, unsigned int m );
+  void insertkey( KeySym k, unsigned int m, WCButton *b, WCHotkey *h, WCPath *p );
+  int findkey( KeySym k, unsigned int m, WCButton **b, WCHotkey **h, WCPath **p );
+  int findkey( KeySym k1, unsigned int m1, KeySym k2, unsigned int m2, WCButton **b, WCHotkey **h, WCPath **p );
+  int waitcursorcount;
+  
+  int getMaxButtonsWidth();
+  int getMaxButtonsLen();
 };
 
 #endif
