@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: worker_commands.h,v 1.19 2001/12/11 23:15:56 ralf Exp $ */
+/* $Id: worker_commands.h,v 1.23 2003/05/04 01:35:31 ralf Exp $ */
 
 #ifndef WORKER_COMMANDS_H
 #define WORKER_COMMANDS_H
@@ -46,5 +46,13 @@
 #include "setsortmodeop.h"
 #include "setfilterop.h"
 #include "shortkeyfromlistop.h"
+#include "chownop.h"
+#include "scriptop.h"
+#include "showdircacheop.h"
+
+#include "listermode.h"
+#include "normalmode.h"
+#include "showimagemode.h"
+#include "informationmode.h"
 
 #endif
